@@ -2,12 +2,12 @@
 #define _CPU_MON_H_
 
 #include <vdr/plugin.h>
-#include "i18n.h"
+
 #include "parms.h"
 #include "cpumonsetup.h"
 #include "cpumonosd.h"
 
-static const char *VERSION        = "0.0.6";
+static const char *VERSION        = "0.0.6a";
 static const char *DESCRIPTION    = "CPU Monitor";
 static const char *MAINMENUENTRY  = "CPU Monitor";
 
@@ -15,7 +15,6 @@ static const char *MAINMENUENTRY  = "CPU Monitor";
 
 class cPluginCpuMon : public cPlugin {
 private:
-  // Add any member variables or functions you may need here.
   cCpuMonFetcher *Values_Fetcher;
   cParameters *parms;  
 public:
@@ -25,7 +24,7 @@ public:
   virtual ~cPluginCpuMon();
 
   virtual const char *Version(void) { return VERSION; };
-  virtual const char *Description(void) { return tr(DESCRIPTION); };
+  virtual const char *Description(void) { return trNOOP(DESCRIPTION); };
   virtual const char *CommandLineHelp(void);
   virtual bool ProcessArgs(int argc, char *argv[]);
   virtual bool Initialize(void);
@@ -34,7 +33,7 @@ public:
   virtual void Housekeeping(void);
   virtual void MainThreadHook(void);
   virtual cString Active(void);
-  virtual const char *MainMenuEntry(void) { return tr(MAINMENUENTRY); };
+  virtual const char *MainMenuEntry(void) { return trNOOP(MAINMENUENTRY); };
   virtual cOsdObject *MainMenuAction(void);
   virtual cMenuSetupPage *SetupMenu(void);
   virtual bool SetupParse(const char *Name, const char *Value);
