@@ -3,7 +3,7 @@
 #define _CPUMON_SETUP_H_
 
 #include <vdr/plugin.h>
-#include "i18n.h"
+// #include "i18n.h"
 #include "parms.h"
 
 class cCpuMonSetup : public cMenuSetupPage
