@@ -2,10 +2,10 @@
  * See the README file for copyright information and how to reach the author.
  */
 
+#include <langinfo.h>
 #include <string>
 #include <fstream>
 #include <iostream>
-#include <langinfo.h>
 #include <vdr/plugin.h>
 #include <vdr/videodir.h>
 #include <vdr/recording.h>
@@ -213,10 +213,10 @@ char *myRecListItem::StripEpisodeName(char *s)
     if(SortByName)
       *s1=1;
     else
-      *(s2+1)=255;
+      *(s2+1)=(char)255;
   }
   else
-    *s1=255;
+    *s1=(char)255;
  
   if(s1 && s2 && !SortByName)
     memmove(s1+1,s2,t-s2+1);
@@ -283,6 +283,8 @@ void WorkerThread::Action()
 {
   CutterListItem *cutteritem=NULL;
   MoveListItem *moveitem=NULL;
+
+  SetPriority(19);
   
   while(Running())
   {
@@ -297,7 +299,7 @@ void WorkerThread::Action()
       if(editedfilename && RemoveVideoFile(editedfilename) && MakeDirs(editedfilename,true))
       {
         char *s=strdup(editedfilename);
-        char *e=strrchr(s,'.');
+        char *e=(char*)strrchr(s,'.'); //TODO
         if(e)
         {
           if(!strcmp(e,".rec"))
@@ -375,9 +377,39 @@ void WorkerThread::Cut(string From,string To)
   cMarks frommarks,tomarks;
   cMark *mark;
   const char *error=NULL;
-  uchar filenumber,picturetype,buffer[MAXFRAMESIZE];
-  int fileoffset,length,index,currentfilenumber=0,filesize=0,lastiframe=0;
+  uchar buffer[MAXFRAMESIZE];
+  int index,currentfilenumber=0,filesize=0,lastiframe=0;
   bool lastmark=false,cutin=true;
+  off_t maxVideoFileSize=MEGABYTE(Setup.MaxVideoFileSize);
+
+#if VDRVERSNUM >= 10703
+  bool isPesRecording;
+  uint16_t filenumber;
+  off_t fileoffset;
+  int length;
+  bool picturetype;
+
+  cRecording Recording(From.c_str());
+  isPesRecording=Recording.IsPesRecording();
+  if(frommarks.Load(From.c_str(),Recording.FramesPerSecond(),isPesRecording) && frommarks.Count())
+  {
+    fromfilename=new cFileName(From.c_str(),false,true,isPesRecording);
+    tofilename=new cFileName(To.c_str(),true,false,isPesRecording);
+    fromindex=new cIndexFile(From.c_str(),false,isPesRecording);
+    toindex=new cIndexFile(To.c_str(),true,isPesRecording);
+    tomarks.Load(To.c_str(),Recording.FramesPerSecond(),isPesRecording);
+    if(isPesRecording && maxVideoFileSize > MEGABYTE(MAXVIDEOFILESIZEPES))
+      maxVideoFileSize=MEGABYTE(MAXVIDEOFILESIZEPES);
+  }
+  else
+  {
+    esyslog("[extrecmenu] no editing marks found for %s",From.c_str());
+    return;
+  }
+#else
+  uchar filenumber;
+  int fileoffset,length;
+  uchar picturetype;
 
   if(frommarks.Load(From.c_str()) && frommarks.Count())
   {
@@ -392,6 +424,8 @@ void WorkerThread::Cut(string From,string To)
     esyslog("[extrecmenu] no editing marks found for %s",From.c_str());
     return;
   }
+#endif
+
 
   if((mark=frommarks.First())!=NULL)
   {
@@ -439,11 +473,15 @@ void WorkerThread::Cut(string From,string To)
       error="fromfile";
       break;
     }
+#if VDRVERSNUM >= 10703
+    if(picturetype)
+#else
     if(picturetype==I_FRAME)
+#endif
     {
       if(lastmark)
         break;
-      if(filesize > MEGABYTE(Setup.MaxVideoFileSize))
+      if(filesize > maxVideoFileSize)
       {
         tofile=tofilename->NextFile();
         if(!tofile)
@@ -456,7 +494,14 @@ void WorkerThread::Cut(string From,string To)
       lastiframe=0;
       if(cutin)
       {
+#if VDRVERSNUM >= 10706
+        if(isPesRecording)
+          cRemux::SetBrokenLink(buffer,length);
+        else
+          TsSetTeiOnBrokenPackets(buffer,length);
+#else
         cRemux::SetBrokenLink(buffer,length);
+#endif
         cutin=false;
       }
     }
@@ -465,7 +510,7 @@ void WorkerThread::Cut(string From,string To)
       error="safe_write";
       break;
     }
-    if(!toindex->Write(picturetype,tofilename->Number(),filesize))
+    if(!toindex->Write(picturetype,(uint16_t)(tofilename->Number()),filesize))
     {
       error="toindex";
       break;
@@ -501,6 +546,9 @@ void WorkerThread::Cut(string From,string To)
       else
         lastmark=true;
     }
+
+    if(mysetup.LimitBandwidth)
+      usleep(10);
   }
   if(!Running() || cancelcut || error)
   {
