@@ -5,7 +5,12 @@
  *
  */
 
+#ifndef __STDC_FORMAT_MACROS
+#define __STDC_FORMAT_MACROS    
+#endif
+
 #include <ctype.h>
+#include <math.h>
 #include "femoncfg.h"
 #include "femonreceiver.h"
 #include "femontools.h"
@@ -15,19 +20,28 @@
 #define CHANNELINPUT_TIMEOUT      1000
 #define SVDRPPLUGIN               "svdrpservice"
 
-#define OSDHEIGHT                 femonConfig.osdheight   // in pixels
-#define OSDWIDTH                  600                     // in pixels
+#define OSDWIDTH                  m_OsdWidth              // in pixels
+#define OSDHEIGHT                 m_OsdHeight             // in pixels
 #define OSDROWHEIGHT              m_Font->Height()        // in pixels
 #define OSDINFOHEIGHT             (OSDROWHEIGHT * 13)     // in pixels (13 rows)
 #define OSDSTATUSHEIGHT           (OSDROWHEIGHT * 6)      // in pixels (6 rows)
 #define OSDSPACING                5
 #define OSDROUNDING               10
 #define IS_OSDROUNDING            (femonConfig.skin == eFemonSkinElchi)
-
+#define IS_OSDRESOLUTION(r1, r2)  (abs(r1 - r2) < 20)
 #define OSDINFOWIN_Y(offset)      (femonConfig.position ? (OSDHEIGHT - OSDINFOHEIGHT + offset) : offset)
-#define OSDINFOWIN_X(col)         ((col == 4) ? 455 : (col == 3) ? 305 : (col == 2) ? 155 : 15)
+#define OSDINFOWIN_X(col)         ((col == 4) ? int(round(OSDWIDTH * 0.76)) : \
+                                   (col == 3) ? int(round(OSDWIDTH * 0.51)) : \
+                                   (col == 2) ? int(round(OSDWIDTH * 0.26)) : \
+                                                int(round(OSDWIDTH * 0.025)))
 #define OSDSTATUSWIN_Y(offset)    (femonConfig.position ? offset : (OSDHEIGHT - OSDSTATUSHEIGHT + offset))
-#define OSDSTATUSWIN_X(col)       ((col == 7) ? 475 : (col == 6) ? 410 : (col == 5) ? 275 : (col == 4) ? 220 : (col == 3) ? 125 : (col == 2) ? 70 : 15)
+#define OSDSTATUSWIN_X(col)       ((col == 7) ? int(round(OSDWIDTH * 0.79)) : \
+                                   (col == 6) ? int(round(OSDWIDTH * 0.68)) : \
+                                   (col == 5) ? int(round(OSDWIDTH * 0.46)) : \
+                                   (col == 4) ? int(round(OSDWIDTH * 0.37)) : \
+                                   (col == 3) ? int(round(OSDWIDTH * 0.21)) : \
+                                   (col == 2) ? int(round(OSDWIDTH * 0.12)) : \
+                                                int(round(OSDWIDTH * 0.025)))
 #define OSDSTATUSWIN_XSYMBOL(c,w) (c * ((OSDWIDTH - (5 * w)) / 6) + ((c - 1) * w))
 #define OSDBARWIDTH(x)            (OSDWIDTH * x / 100)
 
@@ -53,12 +67,12 @@
 
 #define OSDDRAWSTATUSBAR(value) \
         if (value > 0) { \
-           value = OSDBARWIDTH(value); \
-           m_Osd->DrawRectangle(0, OSDSTATUSWIN_Y(offset) + 3, min(OSDBARWIDTH(femonConfig.redlimit), value), OSDSTATUSWIN_Y(offset) + OSDROWHEIGHT - 3, femonTheme[femonConfig.theme].clrRed); \
-           if (value > OSDBARWIDTH(femonConfig.redlimit)) \
-              m_Osd->DrawRectangle(OSDBARWIDTH(femonConfig.redlimit), OSDSTATUSWIN_Y(offset) + 3, min((OSDWIDTH * femonConfig.greenlimit / 100), value), OSDSTATUSWIN_Y(offset) + OSDROWHEIGHT - 3, femonTheme[femonConfig.theme].clrYellow); \
-           if (value > OSDBARWIDTH(femonConfig.greenlimit)) \
-              m_Osd->DrawRectangle(OSDBARWIDTH(femonConfig.greenlimit), OSDSTATUSWIN_Y(offset) + 3, value, OSDSTATUSWIN_Y(offset) + OSDROWHEIGHT - 3, femonTheme[femonConfig.theme].clrGreen); \
+           int barvalue = OSDBARWIDTH(value); \
+           m_Osd->DrawRectangle(0, OSDSTATUSWIN_Y(offset) + 3, min(OSDBARWIDTH(femonConfig.redlimit), barvalue), OSDSTATUSWIN_Y(offset) + OSDROWHEIGHT - 3, femonTheme[femonConfig.theme].clrRed); \
+           if (barvalue > OSDBARWIDTH(femonConfig.redlimit)) \
+              m_Osd->DrawRectangle(OSDBARWIDTH(femonConfig.redlimit), OSDSTATUSWIN_Y(offset) + 3, min((OSDWIDTH * femonConfig.greenlimit / 100), barvalue), OSDSTATUSWIN_Y(offset) + OSDROWHEIGHT - 3, femonTheme[femonConfig.theme].clrYellow); \
+           if (barvalue > OSDBARWIDTH(femonConfig.greenlimit)) \
+              m_Osd->DrawRectangle(OSDBARWIDTH(femonConfig.greenlimit), OSDSTATUSWIN_Y(offset) + 3, barvalue, OSDSTATUSWIN_Y(offset) + OSDROWHEIGHT - 3, femonTheme[femonConfig.theme].clrGreen); \
            }
 
 #define OSDDRAWSTATUSTITLEBAR(title) \
@@ -116,12 +130,28 @@
 #define OSDCLEARINFO() \
         m_Osd->DrawRectangle(0, OSDINFOWIN_Y(0), OSDWIDTH, OSDINFOWIN_Y(OSDINFOHEIGHT) - 1, clrTransparent)
 
+#ifndef MINFONTSIZE
+#define MINFONTSIZE 10
+#endif
+
+#ifndef MAXFONTSIZE
+#define MAXFONTSIZE 64
+#endif
+
+class cFemonDummyFont : public cFont {
+public:
+  virtual int Width(uint c) const { return 10; }
+  virtual int Width(const char *s) const { return 50; }
+  virtual int Height(void) const { return 20; }
+  virtual void DrawText(cBitmap *Bitmap, int x, int y, const char *s, tColor ColorFg, tColor ColorBg, int Width) const {}
+};
+
 cFemonOsd *cFemonOsd::pInstance = NULL;
 
 cFemonOsd *cFemonOsd::Instance(bool create)
 {
-  Dprintf("%s()\n", __PRETTY_FUNCTION__);
-  if (pInstance == NULL && create)
+  debug("%s()\n", __PRETTY_FUNCTION__);
+  if ((pInstance == NULL) && create)
   {
      pInstance = new cFemonOsd();
   }
@@ -140,31 +170,48 @@ cFemonOsd::cFemonOsd()
   m_Number(0),
   m_OldNumber(0),
   m_SNR(0),
+  m_SNRValid(false),
   m_Signal(0),
+  m_SignalValid(false),
   m_BER(0),
+  m_BERValid(false),
   m_UNC(0),
+  m_UNCValid(false),
+  m_FrontendStatusValid(false),
   m_DisplayMode(femonConfig.displaymode),
+  m_OsdWidth(cOsd::OsdWidth() * (100 - femonConfig.downscale) / 100),
+  m_OsdHeight(cOsd::OsdHeight() * (100 - femonConfig.downscale) / 100),
+  m_OsdLeft(cOsd::OsdLeft() + (cOsd::OsdWidth() * femonConfig.downscale / 200)),
+  m_OsdTop(cOsd::OsdTop() + (cOsd::OsdHeight() * femonConfig.downscale / 200)),
   m_InputTime(0),
   m_Sleep(),
   m_Mutex()
 {
-  Dprintf("%s()\n", __PRETTY_FUNCTION__);
+  int tmp;
+  debug("%s()\n", __PRETTY_FUNCTION__);
+  memset(&m_FrontendStatus, 0, sizeof(m_FrontendStatus));
+  memset(&m_FrontendInfo, 0, sizeof(m_FrontendInfo));
   m_SvdrpConnection.handle = -1;
-  if (Setup.UseSmallFont == 0) {
-     // Dirty hack to force the small fonts...
-     Setup.UseSmallFont = 1;
-     m_Font = cFont::GetFont(fontSml);
-     Setup.UseSmallFont = 0;
+  m_Font = cFont::CreateFont(Setup.FontSml, min(max(Setup.FontSmlSize, MINFONTSIZE), MAXFONTSIZE));
+  if (!m_Font || !m_Font->Height()) {
+     m_Font = new cFemonDummyFont;
+     error("cFemonOsd::cFemonOsd() cannot create required font.");
+     }
+  tmp = 5 * bmSymbol[SYMBOL_LOCK].Width() + 6 * OSDSPACING;
+  if (OSDWIDTH < tmp) {
+     error("cFemonOsd::cFemonOsd() OSD width (%d) smaller than required (%d).", OSDWIDTH, tmp);
+     OSDWIDTH = tmp;
+     }
+  tmp = OSDINFOHEIGHT + OSDROWHEIGHT + OSDSTATUSHEIGHT;
+  if (OSDHEIGHT < tmp) {
+     error("cFemonOsd::cFemonOsd() OSD height (%d) smaller than required (%d).", OSDHEIGHT, tmp);
+     OSDHEIGHT = tmp;
      }
-  else
-     m_Font = cFont::GetFont(fontSml);
-  if (OSDHEIGHT < (OSDINFOHEIGHT + OSDROWHEIGHT + OSDSTATUSHEIGHT))
-     OSDHEIGHT = (OSDINFOHEIGHT + OSDROWHEIGHT + OSDSTATUSHEIGHT);
 }
 
 cFemonOsd::~cFemonOsd(void)
 {
-  Dprintf("%s()\n", __PRETTY_FUNCTION__);
+  debug("%s()\n", __PRETTY_FUNCTION__);
   m_Sleep.Signal();
   if (Running())
      Cancel(3);
@@ -179,6 +226,12 @@ cFemonOsd::~cFemonOsd(void)
      }
   if (m_Osd)
      DELETENULL(m_Osd);
+  if (m_Font)
+     DELETENULL(m_Font);
+  if (m_Frontend >= 0) {
+     close(m_Frontend);
+     m_Frontend = -1;
+     }
   pInstance = NULL;
 }
 
@@ -186,8 +239,6 @@ void cFemonOsd::DrawStatusWindow(void)
 {
   cMutexLock lock(&m_Mutex);
   cBitmap *bm = NULL;
-  int snr = m_SNR / 655;
-  int signal = m_Signal / 655;
   int offset = 0;
   int x = OSDWIDTH - OSDROUNDING;
   int y = 0;
@@ -208,6 +259,7 @@ void cFemonOsd::DrawStatusWindow(void)
        case 5:  bm = &bmSymbol[SYMBOL_FIVE];  break;
        case 6:  bm = &bmSymbol[SYMBOL_SIX];   break;
        case 7:  bm = &bmSymbol[SYMBOL_SEVEN]; break;
+       case 8:  bm = &bmSymbol[SYMBOL_EIGHT]; break;
        default: bm = &bmSymbol[SYMBOL_ZERO];  break;
        }
      OSDDRAWSTATUSBM(OSDSPACING);
@@ -222,6 +274,7 @@ void cFemonOsd::DrawStatusWindow(void)
            case 5:  bm = &bmSymbol[SYMBOL_FIVE];  break;
            case 6:  bm = &bmSymbol[SYMBOL_SIX];   break;
            case 7:  bm = &bmSymbol[SYMBOL_SEVEN]; break;
+           case 8:  bm = &bmSymbol[SYMBOL_EIGHT]; break;
            default: bm = &bmSymbol[SYMBOL_ZERO];  break;
            }
         OSDDRAWSTATUSBM(OSDSPACING);
@@ -239,6 +292,17 @@ void cFemonOsd::DrawStatusWindow(void)
         OSDDRAWSTATUSBM(OSDSPACING);
         }
      if (m_Receiver) {
+        if (IS_OSDRESOLUTION(m_Receiver->VideoVerticalSize(), 1080))
+           bm = &bmSymbol[SYMBOL_FORMAT_1080];
+        else if (IS_OSDRESOLUTION(m_Receiver->VideoVerticalSize(), 720))
+           bm = &bmSymbol[SYMBOL_FORMAT_720];
+        else if (IS_OSDRESOLUTION(m_Receiver->VideoVerticalSize(), 576))
+           bm = &bmSymbol[SYMBOL_FORMAT_576];
+        else if (IS_OSDRESOLUTION(m_Receiver->VideoVerticalSize(), 480))
+           bm = &bmSymbol[SYMBOL_FORMAT_480];
+        else
+           bm = NULL;
+        OSDDRAWSTATUSBM(OSDSPACING);
         switch (m_Receiver->VideoCodec()) {
            case VIDEO_CODEC_MPEG2: bm = &bmSymbol[SYMBOL_MPEG2]; break;
            case VIDEO_CODEC_H264:  bm = &bmSymbol[SYMBOL_H264];  break;
@@ -265,24 +329,30 @@ void cFemonOsd::DrawStatusWindow(void)
         OSDDRAWSTATUSBM(OSDSPACING);
         }
      offset += OSDROWHEIGHT;
-     OSDDRAWSTATUSBAR(signal);
+     if (m_SignalValid)
+        OSDDRAWSTATUSBAR(m_Signal / 655);
      offset += OSDROWHEIGHT;
-     OSDDRAWSTATUSBAR(snr);
+     if (m_SNRValid)
+        OSDDRAWSTATUSBAR(m_SNR / 655);
      offset += OSDROWHEIGHT;
-     OSDDRAWSTATUSVALUES("STR:", *cString::sprintf("%04x", m_Signal), *cString::sprintf("(%2d%%)", m_Signal / 655), "BER:", *cString::sprintf("%08x", m_BER),
-                         *cString::sprintf("%s:", tr("Video")), *getBitrateMbits(m_Receiver ? m_Receiver->VideoBitrate() : (m_SvdrpFrontend >= 0 ? m_SvdrpVideoBitrate : -1.0)));
+     OSDDRAWSTATUSVALUES("STR:", m_SignalValid ? *cString::sprintf("%04x", m_Signal) : "", m_SignalValid ? *cString::sprintf("(%2d%%)", m_Signal / 655) : "",
+                         "BER:", m_BERValid ? *cString::sprintf("%08x", m_BER) : "", *cString::sprintf("%s:", tr("Video")),
+                         *getBitrateMbits(m_Receiver ? m_Receiver->VideoBitrate() : (m_SvdrpFrontend >= 0 ? m_SvdrpVideoBitrate : -1.0)));
      offset += OSDROWHEIGHT;
-     OSDDRAWSTATUSVALUES("SNR:", *cString::sprintf("%04x", m_SNR), *cString::sprintf("(%2d%%)", m_SNR / 655), "UNC:", *cString::sprintf("%08x", m_UNC),
+     OSDDRAWSTATUSVALUES("SNR:", m_SNRValid ? *cString::sprintf("%04x", m_SNR) : "", m_SNRValid ? *cString::sprintf("(%2d%%)", m_SNR / 655) : "",
+                         "UNC:", m_UNCValid ? *cString::sprintf("%08x", m_UNC) : "",
                          *cString::sprintf("%s:", (m_Receiver && m_Receiver->AC3Valid() && IS_DOLBY_TRACK(track)) ? tr("AC-3") : tr("Audio")),
                          *getBitrateKbits(m_Receiver ? ((m_Receiver->AC3Valid() && IS_DOLBY_TRACK(track)) ? m_Receiver->AC3Bitrate() : m_Receiver->AudioBitrate()) : (m_SvdrpFrontend >= 0 ? m_SvdrpAudioBitrate : -1.0)));
      offset += OSDROWHEIGHT;
      x = bmSymbol[SYMBOL_LOCK].Width();
      y = (OSDROWHEIGHT - bmSymbol[SYMBOL_LOCK].Height()) / 2;
-     OSDDRAWSTATUSFRONTEND(1, bmSymbol[SYMBOL_LOCK],    FE_HAS_LOCK);
-     OSDDRAWSTATUSFRONTEND(2, bmSymbol[SYMBOL_SIGNAL],  FE_HAS_SIGNAL);
-     OSDDRAWSTATUSFRONTEND(3, bmSymbol[SYMBOL_CARRIER], FE_HAS_CARRIER);
-     OSDDRAWSTATUSFRONTEND(4, bmSymbol[SYMBOL_VITERBI], FE_HAS_VITERBI);
-     OSDDRAWSTATUSFRONTEND(5, bmSymbol[SYMBOL_SYNC],    FE_HAS_SYNC);
+     if (m_FrontendStatusValid) {
+        OSDDRAWSTATUSFRONTEND(1, bmSymbol[SYMBOL_LOCK],    FE_HAS_LOCK);
+        OSDDRAWSTATUSFRONTEND(2, bmSymbol[SYMBOL_SIGNAL],  FE_HAS_SIGNAL);
+        OSDDRAWSTATUSFRONTEND(3, bmSymbol[SYMBOL_CARRIER], FE_HAS_CARRIER);
+        OSDDRAWSTATUSFRONTEND(4, bmSymbol[SYMBOL_VITERBI], FE_HAS_VITERBI);
+        OSDDRAWSTATUSFRONTEND(5, bmSymbol[SYMBOL_SYNC],    FE_HAS_SYNC);
+        }
      OSDDRAWSTATUSBOTTOMBAR();
      m_Osd->Flush();
      }
@@ -296,6 +366,7 @@ void cFemonOsd::DrawInfoWindow(void)
   eTrackType track = cDevice::PrimaryDevice()->GetCurrentAudioTrack();
 
   if (m_Osd && channel) {
+     cDvbTransponderParameters dtp(channel->Parameters());
      switch (m_DisplayMode) {
        case eFemonModeTransponder:
             OSDDRAWINFOTITLEBAR(tr("Transponder Information"));
@@ -315,55 +386,52 @@ void cFemonOsd::DrawInfoWindow(void)
             OSDDRAWINFOLEFT(    tr("Tid"),  *cString::sprintf("%d", channel->Tid()));
             OSDDRAWINFORIGHT(   tr("Rid"),  *cString::sprintf("%d", channel->Rid()));
             offset += OSDROWHEIGHT;
-            OSDDRAWINFOLEFT( trVDR("CA"),   *getCAids(channel, femonConfig.showcasystem));
+            OSDDRAWINFOLEFT( trVDR("CA"),   *getCAids(channel));
             offset += OSDROWHEIGHT;
-            switch (m_FrontendInfo.type) {
-              case FE_QPSK:
-                   OSDDRAWINFOLINE(*cString::sprintf("%s #%d - %s", tr("Satellite Card"), (m_SvdrpFrontend >= 0) ? m_SvdrpFrontend : cDevice::ActualDevice()->CardIndex(), m_FrontendInfo.name));
+            switch (channel->Source() & cSource::st_Mask) {
+              case cSource::stSat:
+                   OSDDRAWINFOLINE(*cString::sprintf("DVB-S%s #%d - %s", (m_FrontendInfo.caps & 0x10000000) ? "2" : "", (m_SvdrpFrontend >= 0) ? m_SvdrpFrontend : cDevice::ActualDevice()->CardIndex(), m_FrontendInfo.name));
                    offset += OSDROWHEIGHT;
                    OSDDRAWINFOLEFT( trVDR("Frequency"),    *getFrequencyMHz(channel->Frequency()));
                    OSDDRAWINFORIGHT(trVDR("Source"),       *cSource::ToString(channel->Source()));
                    offset += OSDROWHEIGHT;
                    OSDDRAWINFOLEFT( trVDR("Srate"),        *cString::sprintf("%d", channel->Srate()));
-                   OSDDRAWINFORIGHT(trVDR("Polarization"), *cString::sprintf("%c", toupper(channel->Polarization())));
+                   OSDDRAWINFORIGHT(trVDR("Polarization"), *cString::sprintf("%c", toupper(dtp.Polarization())));
                    offset += OSDROWHEIGHT;
-                   OSDDRAWINFOLEFT( trVDR("Inversion"),    *getInversion(channel->Inversion()));
-                   OSDDRAWINFORIGHT(trVDR("CoderateH"),    *getCoderate(channel->CoderateH()));
-                   //offset += OSDROWHEIGHT;
-                   //OSDDRAWINFOLEFT( trVDR("System"),     *getSystem(channel->System()));
-                   //OSDDRAWINFORIGHT(trVDR("RollOff"),    *getRollOff(channel->RollOff()));
+                   OSDDRAWINFOLEFT( trVDR("Inversion"),    *getInversion(dtp.Inversion()));
+                   OSDDRAWINFORIGHT(trVDR("CoderateH"),    *getCoderate(dtp.CoderateH()));
+                   offset += OSDROWHEIGHT;
+                   OSDDRAWINFOLEFT( trVDR("System"),       *getSystem(dtp.System()));
+                   OSDDRAWINFORIGHT(trVDR("RollOff"),      *getRollOff(dtp.RollOff()));
                    break;
 
-             case FE_QAM:
-                   OSDDRAWINFOLINE(*cString::sprintf("%s #%d - %s", tr("Cable Card"), (m_SvdrpFrontend >= 0) ? m_SvdrpFrontend : cDevice::ActualDevice()->CardIndex(), m_FrontendInfo.name));
+              case cSource::stCable:
+                   OSDDRAWINFOLINE(*cString::sprintf("DVB-C #%d - %s", (m_SvdrpFrontend >= 0) ? m_SvdrpFrontend : cDevice::ActualDevice()->CardIndex(), m_FrontendInfo.name));
                    offset += OSDROWHEIGHT;
                    OSDDRAWINFOLEFT( trVDR("Frequency"),    *getFrequencyMHz(channel->Frequency()));
                    OSDDRAWINFORIGHT(trVDR("Source"),       *cSource::ToString(channel->Source()));
                    offset += OSDROWHEIGHT;
                    OSDDRAWINFOLEFT( trVDR("Srate"),        *cString::sprintf("%d", channel->Srate()));
-                   OSDDRAWINFORIGHT(trVDR("Modulation"),   *getModulation(channel->Modulation()));
+                   OSDDRAWINFORIGHT(trVDR("Modulation"),   *getModulation(dtp.Modulation()));
                    offset += OSDROWHEIGHT;
-                   OSDDRAWINFOLEFT( trVDR("Inversion"),    *getInversion(channel->Inversion()));
-                   OSDDRAWINFORIGHT(trVDR("CoderateH"),    *getCoderate(channel->CoderateH()));
+                   OSDDRAWINFOLEFT( trVDR("Inversion"),    *getInversion(dtp.Inversion()));
+                   OSDDRAWINFORIGHT(trVDR("CoderateH"),    *getCoderate(dtp.CoderateH()));
                    break;
 
-              case FE_OFDM:
-                   OSDDRAWINFOLINE(*cString::sprintf("%s #%d - %s", tr("Terrestrial Card"), (m_SvdrpFrontend >= 0) ? m_SvdrpFrontend : cDevice::ActualDevice()->CardIndex(), m_FrontendInfo.name));
+              case cSource::stTerr:
+                   OSDDRAWINFOLINE(*cString::sprintf("DVB-T #%d - %s", (m_SvdrpFrontend >= 0) ? m_SvdrpFrontend : cDevice::ActualDevice()->CardIndex(), m_FrontendInfo.name));
                    offset += OSDROWHEIGHT;
                    OSDDRAWINFOLEFT( trVDR("Frequency"),    *getFrequencyMHz(channel->Frequency()));
-                   OSDDRAWINFORIGHT(trVDR("Transmission"), *getTransmission(channel->Transmission()));
+                   OSDDRAWINFORIGHT(trVDR("Transmission"), *getTransmission(dtp.Transmission()));
                    offset += OSDROWHEIGHT;
-                   OSDDRAWINFOLEFT( trVDR("Bandwidth"),    *getBandwidth(channel->Bandwidth()));
-                   OSDDRAWINFORIGHT(trVDR("Modulation"),   *getModulation(channel->Modulation()));
+                   OSDDRAWINFOLEFT( trVDR("Bandwidth"),    *getBandwidth(dtp.Bandwidth()));
+                   OSDDRAWINFORIGHT(trVDR("Modulation"),   *getModulation(dtp.Modulation()));
                    offset += OSDROWHEIGHT;
-                   OSDDRAWINFOLEFT( trVDR("Inversion"),    *getInversion(channel->Inversion()));
-                   OSDDRAWINFORIGHT(tr   ("Coderate"),     *cString::sprintf("%s (H) %s (L)", *getCoderate(channel->CoderateH()), *getCoderate(channel->CoderateL())));
+                   OSDDRAWINFOLEFT( trVDR("Inversion"),    *getInversion(dtp.Inversion()));
+                   OSDDRAWINFORIGHT(tr   ("Coderate"),     *cString::sprintf("%s (H) %s (L)", *getCoderate(dtp.CoderateH()), *getCoderate(dtp.CoderateL())));
                    offset += OSDROWHEIGHT;
-                   OSDDRAWINFOLEFT( trVDR("Hierarchy"),    *getHierarchy(channel->Hierarchy()));
-                   OSDDRAWINFORIGHT(trVDR("Guard"),        *getGuard(channel->Guard()));
-                   //offset += OSDROWHEIGHT;
-                   //OSDDRAWINFOLEFT( trVDR("Alpha"),      *getAlpha(channel->Alpha()));
-                   //OSDDRAWINFORIGHT(trVDR("Priority"),   *getPriority(channel->Priority()));
+                   OSDDRAWINFOLEFT( trVDR("Hierarchy"),    *getHierarchy(dtp.Hierarchy()));
+                   OSDDRAWINFORIGHT(trVDR("Guard"),        *getGuard(dtp.Guard()));
                    break;
 
               default:
@@ -421,7 +489,7 @@ void cFemonOsd::DrawInfoWindow(void)
                offset += OSDROWHEIGHT;
                OSDDRAWINFOINACTIVE(tr("Dolby Surround Mode"),    *getAC3DolbySurroundMode(m_Receiver->AC3DolbySurroundMode()));
                offset += OSDROWHEIGHT;
-               OSDDRAWINFOINACTIVE(tr("Low Frequency Effects"),  *cString::sprintf("%s", m_Receiver->AC3Lfe() ? tr("on") : tr("off")));
+               OSDDRAWINFOINACTIVE(tr("Low Frequency Effects"),  *cString::sprintf("%s", m_Receiver->AC3Lfe() ? trVDR("on") : trVDR("off")));
                offset += OSDROWHEIGHT;
                OSDDRAWINFOINACTIVE(tr("Dialogue Normalization"), *getAC3DialogLevel(m_Receiver->AC3DialogLevel()));
                }
@@ -438,7 +506,7 @@ void cFemonOsd::DrawInfoWindow(void)
 
 void cFemonOsd::Action(void)
 {
-  Dprintf("%s()\n", __PRETTY_FUNCTION__);
+  debug("%s()\n", __PRETTY_FUNCTION__);
   cTimeMs t;
   SvdrpCommand_v1_0 cmd;
   cmd.command = cString::sprintf("PLUG %s INFO\r\n", PLUGIN_NAME_I18N);
@@ -448,41 +516,57 @@ void cFemonOsd::Action(void)
     m_SvdrpVideoBitrate = -1.0;
     m_SvdrpAudioBitrate = -1.0;
     if (m_Frontend != -1) {
-       CHECK(ioctl(m_Frontend, FE_READ_STATUS, &m_FrontendStatus));
-       CHECK(ioctl(m_Frontend, FE_READ_SIGNAL_STRENGTH, &m_Signal));
-       CHECK(ioctl(m_Frontend, FE_READ_SNR, &m_SNR));
-       CHECK(ioctl(m_Frontend, FE_READ_BER, &m_BER));
-       CHECK(ioctl(m_Frontend, FE_READ_UNCORRECTED_BLOCKS, &m_UNC));
+       m_FrontendStatusValid = (ioctl(m_Frontend, FE_READ_STATUS, &m_FrontendStatus) >= 0);
+       m_SignalValid = (ioctl(m_Frontend, FE_READ_SIGNAL_STRENGTH, &m_Signal) >= 0);
+       m_SNRValid = (ioctl(m_Frontend, FE_READ_SNR, &m_SNR) >= 0);
+       m_BERValid = (ioctl(m_Frontend, FE_READ_BER, &m_BER) >= 0);
+       m_UNCValid = (ioctl(m_Frontend, FE_READ_UNCORRECTED_BLOCKS, &m_UNC) >= 0);
        DrawInfoWindow();
        DrawStatusWindow();
        }
     else if (m_SvdrpConnection.handle >= 0) {
-       cmd.handle = m_SvdrpConnection.handle; 
+       cmd.handle = m_SvdrpConnection.handle;
        m_SvdrpPlugin->Service("SvdrpCommand-v1.0", &cmd);
        if (cmd.responseCode == 900) {
+          m_FrontendStatusValid = false;
+          m_SignalValid = false;
+          m_SNRValid = false;
+          m_BERValid = false;
+          m_UNCValid = false;
           for (cLine *line = cmd.reply.First(); line; line = cmd.reply.Next(line)) {
-             const char *s = line->Text();
-	      if (strncasecmp(s, "CARD:", 5) == 0)
-                m_SvdrpFrontend = strtol(s + 5, NULL, 10);
-             else if (strncasecmp(s, "TYPE:", 5) == 0)
-                m_FrontendInfo.type = (fe_type_t) strtol(s + 5, NULL, 10);
-             else if (strncasecmp(s, "NAME:", 5) == 0)
-                strn0cpy(m_FrontendInfo.name, s + 5, sizeof(m_FrontendInfo.name));
-             else if (strncasecmp(s, "STAT:", 5) == 0)
-                m_FrontendStatus = (fe_status_t) strtol(s + 5, NULL, 16);
-             else if (strncasecmp(s, "SGNL:", 5) == 0)
-                m_Signal = strtol(s + 5, NULL, 16);
-             else if (strncasecmp(s, "SNRA:", 5) == 0)
-                m_SNR = strtol(s + 5, NULL, 16);
-             else if (strncasecmp(s, "BERA:", 5) == 0)
-                m_BER = strtol(s + 5, NULL, 16);
-             else if (strncasecmp(s, "UNCB:", 5) == 0)
-                m_UNC = strtol(s + 5, NULL, 16);
-             else if (strncasecmp(s, "VIBR:", 5) == 0)
-                m_SvdrpVideoBitrate = strtol(s + 5, NULL, 10);
-             else if (strncasecmp(s, "AUBR:", 5) == 0)
-                m_SvdrpAudioBitrate = strtol(s + 5, NULL, 10);
-             }
+              const char *s = line->Text();
+	      if (!strncasecmp(s, "CARD:", 5))
+                 m_SvdrpFrontend = (int)strtol(s + 5, NULL, 10);
+              else if (!strncasecmp(s, "TYPE:", 5))
+                 m_FrontendInfo.type = (fe_type_t)strtol(s + 5, NULL, 10);
+              else if (!strncasecmp(s, "NAME:", 5)) {
+                 strn0cpy(m_FrontendInfo.name, s + 5, sizeof(m_FrontendInfo.name));
+                 }
+              else if (!strncasecmp(s, "STAT:", 5)) {
+                 m_FrontendStatus = (fe_status_t)strtol(s + 5, NULL, 16);
+                 m_FrontendStatusValid = true;
+                 }
+              else if (!strncasecmp(s, "SGNL:", 5)) {
+                 m_Signal = (uint16_t)strtol(s + 5, NULL, 16);
+                 m_SignalValid = true;
+                 }
+              else if (!strncasecmp(s, "SNRA:", 5)) {
+                 m_SNR = (uint16_t)strtol(s + 5, NULL, 16);
+                 m_SNRValid = true;
+                 }
+              else if (!strncasecmp(s, "BERA:", 5)) {
+                 m_BER = (uint32_t)strtol(s + 5, NULL, 16);
+                 m_BERValid = true;
+                 }
+              else if (!strncasecmp(s, "UNCB:", 5)) {
+                 m_UNC = (uint32_t)strtol(s + 5, NULL, 16);
+                 m_UNCValid = true;
+                 }
+              else if (!strncasecmp(s, "VIBR:", 5))
+                 m_SvdrpVideoBitrate = (double)strtol(s + 5, NULL, 10);
+              else if (!strncasecmp(s, "AUBR:", 5))
+                 m_SvdrpAudioBitrate = (double)strtol(s + 5, NULL, 10);
+              }
           }
        DrawInfoWindow();
        DrawStatusWindow();
@@ -493,17 +577,17 @@ void cFemonOsd::Action(void)
 
 void cFemonOsd::Show(void)
 {
-  Dprintf("%s()\n", __PRETTY_FUNCTION__);
-  int apid[2] = {0, 0};
-  int dpid[2] = {0, 0};
+  debug("%s()\n", __PRETTY_FUNCTION__);
   eTrackType track = cDevice::PrimaryDevice()->GetCurrentAudioTrack();
   cString dev = cString::sprintf(FRONTEND_DEVICE, cDevice::ActualDevice()->CardIndex(), 0);
   m_Frontend = open(dev, O_RDONLY | O_NONBLOCK);
   if (m_Frontend >= 0) {
      if (ioctl(m_Frontend, FE_GET_INFO, &m_FrontendInfo) < 0) {
-        esyslog("ERROR: cFemonOsd::Show() cannot read frontend info.");
+        if (!femonConfig.usesvdrp)
+           error("cFemonOsd::Show() cannot read frontend info.");
         close(m_Frontend);
         m_Frontend = -1;
+        memset(&m_FrontendInfo, 0, sizeof(m_FrontendInfo));
         return;
         }
      }
@@ -512,11 +596,11 @@ void cFemonOsd::Show(void)
         return;
      }
   else {
-     esyslog("ERROR: cFemonOsd::Show() cannot open frontend device.");
+     error("cFemonOsd::Show() cannot open frontend device.");
      return;
      }
 
-  m_Osd = cOsdProvider::NewOsd(((cOsd::OsdWidth() - OSDWIDTH) / 2) + cOsd::OsdLeft() + femonConfig.osdoffset, ((cOsd::OsdHeight() - OSDHEIGHT) / 2) + cOsd::OsdTop());
+  m_Osd = cOsdProvider::NewOsd(m_OsdLeft, m_OsdTop);
   if (m_Osd) {
      tArea Areas1[] = { { 0, 0, OSDWIDTH - 1, OSDHEIGHT - 1, 8 } };
      if (Setup.AntiAlias && m_Osd->CanHandleAreas(Areas1, sizeof(Areas1) / sizeof(tArea)) == oeOk) {
@@ -538,9 +622,7 @@ void cFemonOsd::Show(void)
      if (femonConfig.analyzestream) {
         cChannel *channel = Channels.GetByNumber(cDevice::CurrentChannel());
         if (channel) {
-           IS_AUDIO_TRACK(track) ? apid[0] = channel->Apid(int(track - ttAudioFirst)) : apid[0] = channel->Apid(0);
-           IS_DOLBY_TRACK(track) ? dpid[0] = channel->Dpid(int(track - ttDolbyFirst)) : dpid[0] = channel->Dpid(0);
-           m_Receiver = new cFemonReceiver(channel->GetChannelID(), channel->Ca(), channel->Vpid(), apid, dpid);
+           m_Receiver = new cFemonReceiver(channel->Vtype(), channel->Vpid(), channel->Apid(IS_AUDIO_TRACK(track) ? int(track - ttAudioFirst) : 0), channel->Dpid(IS_DOLBY_TRACK(track) ? int(track - ttDolbyFirst) : 0));
            cDevice::ActualDevice()->AttachReceiver(m_Receiver);
            }
         }
@@ -550,9 +632,7 @@ void cFemonOsd::Show(void)
 
 void cFemonOsd::ChannelSwitch(const cDevice * device, int channelNumber)
 {
-  Dprintf("%s(%d,%d)\n", __PRETTY_FUNCTION__, device->DeviceNumber(), channelNumber);
-  int apid[2] = {0, 0};
-  int dpid[2] = {0, 0};
+  debug("%s(%d,%d)\n", __PRETTY_FUNCTION__, device->DeviceNumber(), channelNumber);
   eTrackType track = cDevice::PrimaryDevice()->GetCurrentAudioTrack();
   if (!device->IsPrimaryDevice() || !channelNumber || cDevice::PrimaryDevice()->CurrentChannel() != channelNumber)
      return;
@@ -561,9 +641,11 @@ void cFemonOsd::ChannelSwitch(const cDevice * device, int channelNumber)
   m_Frontend = open(dev, O_RDONLY | O_NONBLOCK);
   if (m_Frontend >= 0) {
      if (ioctl(m_Frontend, FE_GET_INFO, &m_FrontendInfo) < 0) {
-        esyslog("ERROR: cFemonOsd::ChannelSwitch() cannot read frontend info.");
+        if (!femonConfig.usesvdrp)
+           error("cFemonOsd::ChannelSwitch() cannot read frontend info.");
         close(m_Frontend);
         m_Frontend = -1;
+        memset(&m_FrontendInfo, 0, sizeof(m_FrontendInfo));
         return;
         }
      }
@@ -572,7 +654,7 @@ void cFemonOsd::ChannelSwitch(const cDevice * device, int channelNumber)
         return;
      }
   else {
-     esyslog("ERROR: cFemonOsd::ChannelSwitch() cannot open frontend device.");
+     error("cFemonOsd::ChannelSwitch() cannot open frontend device.");
      return;
      }
 
@@ -583,9 +665,7 @@ void cFemonOsd::ChannelSwitch(const cDevice * device, int channelNumber)
   if (femonConfig.analyzestream) {
      cChannel *channel = Channels.GetByNumber(cDevice::CurrentChannel());
      if (channel) {
-        IS_AUDIO_TRACK(track) ? apid[0] = channel->Apid(int(track - ttAudioFirst)) : apid[0] = channel->Apid(0);
-        IS_DOLBY_TRACK(track) ? dpid[0] = channel->Dpid(int(track - ttDolbyFirst)) : dpid[0] = channel->Dpid(0);
-        m_Receiver = new cFemonReceiver(channel->GetChannelID(), channel->Ca(), channel->Vpid(), apid, dpid);
+        m_Receiver = new cFemonReceiver(channel->Vtype(), channel->Vpid(), channel->Apid(IS_AUDIO_TRACK(track) ? int(track - ttAudioFirst) : 0), channel->Dpid(IS_DOLBY_TRACK(track) ? int(track - ttDolbyFirst) : 0));
         cDevice::ActualDevice()->AttachReceiver(m_Receiver);
         }
      }
@@ -593,9 +673,7 @@ void cFemonOsd::ChannelSwitch(const cDevice * device, int channelNumber)
 
 void cFemonOsd::SetAudioTrack(int Index, const char * const *Tracks)
 {
-  Dprintf("%s()\n", __PRETTY_FUNCTION__);
-  int apid[2] = {0, 0};
-  int dpid[2] = {0, 0};
+  debug("%s()\n", __PRETTY_FUNCTION__);
   eTrackType track = cDevice::PrimaryDevice()->GetCurrentAudioTrack();
   if (m_Receiver) {
      m_Receiver->Deactivate();
@@ -604,9 +682,7 @@ void cFemonOsd::SetAudioTrack(int Index, const char * const *Tracks)
   if (femonConfig.analyzestream) {
      cChannel *channel = Channels.GetByNumber(cDevice::CurrentChannel());
      if (channel) {
-        IS_AUDIO_TRACK(track) ? apid[0] = channel->Apid(int(track - ttAudioFirst)) : apid[0] = channel->Apid(0);
-        IS_DOLBY_TRACK(track) ? dpid[0] = channel->Dpid(int(track - ttDolbyFirst)) : dpid[0] = channel->Dpid(0);
-        m_Receiver = new cFemonReceiver(channel->GetChannelID(), channel->Ca(), channel->Vpid(), apid, dpid);
+        m_Receiver = new cFemonReceiver(channel->Vtype(), channel->Vpid(), channel->Apid(IS_AUDIO_TRACK(track) ? int(track - ttAudioFirst) : 0), channel->Dpid(IS_DOLBY_TRACK(track) ? int(track - ttDolbyFirst) : 0));
         cDevice::ActualDevice()->AttachReceiver(m_Receiver);
         }
      }
@@ -614,7 +690,7 @@ void cFemonOsd::SetAudioTrack(int Index, const char * const *Tracks)
 
 bool cFemonOsd::DeviceSwitch(int direction)
 {
-  Dprintf("%s()\n", __PRETTY_FUNCTION__);
+  debug("%s()\n", __PRETTY_FUNCTION__);
   int device = cDevice::ActualDevice()->DeviceNumber();
   direction = sgn(direction);
   if (device >= 0) {
@@ -630,13 +706,13 @@ bool cFemonOsd::DeviceSwitch(int direction)
                  device = cDevice::NumDevices() - 1;
               }
            if (cDevice::GetDevice(device)->ProvidesChannel(channel, 0)) {
-              Dprintf("%s(%d) device(%d)\n", __PRETTY_FUNCTION__, direction, device);
+              debug("%s(%d) device(%d)\n", __PRETTY_FUNCTION__, direction, device);
               cStatus::MsgChannelSwitch(cDevice::PrimaryDevice(), 0);
               cControl::Shutdown();
               cDevice::GetDevice(device)->SwitchChannel(channel, true);
               if (cDevice::GetDevice(device) == cDevice::PrimaryDevice())
                  cDevice::GetDevice(device)->ForceTransferMode();
-              cControl::Launch(new cTransferControl(cDevice::GetDevice(device), channel->GetChannelID(), channel->Vpid(), channel->Apids(), channel->Dpids(), channel->Spids()));
+              cControl::Launch(new cTransferControl(cDevice::GetDevice(device), channel));
               cStatus::MsgChannelSwitch(cDevice::PrimaryDevice(), channel->Number());
               return (true);
               }
@@ -652,24 +728,24 @@ bool cFemonOsd::SvdrpConnect(void)
       m_SvdrpPlugin = cPluginManager::GetPlugin(SVDRPPLUGIN);
       if (m_SvdrpPlugin) {
          m_SvdrpConnection.serverIp = femonConfig.svdrpip;
-         m_SvdrpConnection.serverPort = femonConfig.svdrpport;
+         m_SvdrpConnection.serverPort = (unsigned short)femonConfig.svdrpport;
          m_SvdrpConnection.shared = true;
          m_SvdrpPlugin->Service("SvdrpConnection-v1.0", &m_SvdrpConnection);
          if (m_SvdrpConnection.handle >= 0) {
             SvdrpCommand_v1_0 cmd;
-            cmd.handle = m_SvdrpConnection.handle; 
+            cmd.handle = m_SvdrpConnection.handle;
             cmd.command = cString::sprintf("PLUG %s\r\n", PLUGIN_NAME_I18N);
             m_SvdrpPlugin->Service("SvdrpCommand-v1.0", &cmd);
             if (cmd.responseCode != 214) {
                m_SvdrpPlugin->Service("SvdrpConnection-v1.0", &m_SvdrpConnection); // close connection
-               esyslog("ERROR: cFemonOsd::SvdrpConnect() cannot find plugin '%s' on server %s.", PLUGIN_NAME_I18N, *m_SvdrpConnection.serverIp);
+               error("cFemonOsd::SvdrpConnect() cannot find plugin '%s' on server %s.", PLUGIN_NAME_I18N, *m_SvdrpConnection.serverIp);
                }
             }
          else
-            esyslog("ERROR: cFemonOsd::SvdrpConnect() cannot connect to SVDRP server.");
+            error("cFemonOsd::SvdrpConnect() cannot connect to SVDRP server.");
          }
       else
-         esyslog("ERROR: cFemonOsd::SvdrpConnect() cannot find plugin '%s'.", SVDRPPLUGIN);
+         error("cFemonOsd::SvdrpConnect() cannot find plugin '%s'.", SVDRPPLUGIN);
       }
    return m_SvdrpConnection.handle >= 0;
 }
@@ -680,24 +756,24 @@ bool cFemonOsd::SvdrpTune(void)
       cChannel *channel = Channels.GetByNumber(cDevice::CurrentChannel());
       if (channel) {
          SvdrpCommand_v1_0 cmd;
-         cmd.handle = m_SvdrpConnection.handle; 
+         cmd.handle = m_SvdrpConnection.handle;
          cmd.command = cString::sprintf("CHAN %s\r\n", *channel->GetChannelID().ToString());
          m_SvdrpPlugin->Service("SvdrpCommand-v1.0", &cmd);
          if (cmd.responseCode == 250)
             return true;
-         esyslog("ERROR: cFemonOsd::SvdrpTune() cannot tune server channel.");
+         error("cFemonOsd::SvdrpTune() cannot tune server channel.");
          }
       else
-         esyslog("ERROR: cFemonOsd::SvdrpTune() invalid channel.");
+         error("cFemonOsd::SvdrpTune() invalid channel.");
       }
    else
-      esyslog("ERROR: cFemonOsd::SvdrpTune() unexpected connection state.");
+      error("cFemonOsd::SvdrpTune() unexpected connection state.");
    return false;
 }
 
 double cFemonOsd::GetVideoBitrate(void)
 {
-  Dprintf("%s()\n", __PRETTY_FUNCTION__);
+  debug("%s()\n", __PRETTY_FUNCTION__);
   double value = 0.0;
 
   if (m_Receiver)
@@ -708,7 +784,7 @@ double cFemonOsd::GetVideoBitrate(void)
 
 double cFemonOsd::GetAudioBitrate(void)
 {
-  Dprintf("%s()\n", __PRETTY_FUNCTION__);
+  debug("%s()\n", __PRETTY_FUNCTION__);
   double value = 0.0;
 
   if (m_Receiver)
@@ -719,7 +795,7 @@ double cFemonOsd::GetAudioBitrate(void)
 
 double cFemonOsd::GetDolbyBitrate(void)
 {
-  Dprintf("%s()\n", __PRETTY_FUNCTION__);
+  debug("%s()\n", __PRETTY_FUNCTION__);
   double value = 0.0;
 
   if (m_Receiver)
@@ -729,7 +805,7 @@ double cFemonOsd::GetDolbyBitrate(void)
 }
 
 eOSState cFemonOsd::ProcessKey(eKeys Key)
-{ 
+{
   eOSState state = cOsdObject::ProcessKey(Key);
   if (state == osUnknown) {
      switch (Key) {
@@ -847,6 +923,6 @@ eOSState cFemonOsd::ProcessKey(eKeys Key)
             break;
        }
      state = osContinue;
-  }
+     }
   return state;
 }
