@@ -3,7 +3,6 @@
  *
  * See the README file for copyright information and how to reach the author.
  *
- * $Id: protocolif.h,v 1.8 2008/01/30 21:57:33 rahrenbe Exp $
  */
 
 #ifndef __IPTV_PROTOCOLIF_H
@@ -13,7 +12,7 @@ class cIptvProtocolIf {
 public:
   cIptvProtocolIf() {}
   virtual ~cIptvProtocolIf() {}
-  virtual int Read(unsigned char* *BufferAddr) = 0;
+  virtual int Read(unsigned char* BufferAddr, unsigned int BufferLen) = 0;
   virtual bool Set(const char* Location, const int Parameter, const int Index) = 0;
   virtual bool Open(void) = 0;
   virtual bool Close(void) = 0;
