@@ -3,7 +3,6 @@
  *
  * See the README file for copyright information and how to reach the author.
  *
- * $Id: protocoludp.h,v 1.13 2008/01/04 23:36:37 ajhseppa Exp $
  */
 
 #ifndef __IPTV_PROTOCOLUDP_H
@@ -24,7 +23,7 @@ private:
 public:
   cIptvProtocolUdp();
   virtual ~cIptvProtocolUdp();
-  int Read(unsigned char* *BufferAddr);
+  int Read(unsigned char* BufferAddr, unsigned int BufferLen);
   bool Set(const char* Location, const int Parameter, const int Index);
   bool Open(void);
   bool Close(void);
