@@ -3,7 +3,6 @@
  *
  * See the README file for copyright information and how to reach the author.
  *
- * $Id: streamer.h,v 1.16 2008/04/04 20:55:44 rahrenbe Exp $
  */
 
 #ifndef __IPTV_STREAMER_H
@@ -20,15 +19,17 @@
 class cIptvStreamer : public cThread, public cIptvStreamerStatistics {
 private:
   cRingBufferLinear* ringBuffer;
-  cMutex* mutex;
-  unsigned char* readBuffer;
-  unsigned int readBufferLen;
+  cCondWait sleep;
+  unsigned char* packetBuffer;
+  unsigned int packetBufferLen;
   cIptvProtocolIf* protocol;
 
-public:
-  cIptvStreamer(cRingBufferLinear* RingBuffer, cMutex* Mutex);
-  virtual ~cIptvStreamer();
+protected:
   virtual void Action(void);
+
+public:
+  cIptvStreamer(cRingBufferLinear* RingBuffer, unsigned int PacketLen);
+  virtual ~cIptvStreamer();
   bool Set(const char* Location, const int Parameter, const int Index, cIptvProtocolIf* Protocol);
   bool Open(void);
   bool Close(void);
