@@ -17,9 +17,9 @@ extern rtp_classes rtp_content;
 
 
 /* Premiere */
-#define PREMIERERADIO_TID 1			// new transponder 20070107
+#define PREMIERERADIO_TID 17            // Premiere changes to SKY, EPG working ???
 // EPG-Info
-#define	PEPG_ARTIST "Interpret: "
+#define PEPG_ARTIST "Interpret: "
 #define PEPG_TITEL "Titel: "
 #define PEPG_ALBUM "Album: "
 #define PEPG_JAHR "Jahr: "
@@ -32,7 +32,7 @@ int epg_premiere(const char *epgtitle, const char *epgdescr, time_t epgstart, ti
 /* Kabel Deutschlang */
 #define KDRADIO_TID 10003
 // EPG-Info
-#define	KDEPG_ARTIST "Artist: "
+#define KDEPG_ARTIST "Artist: "
 #define KDEPG_TITEL "Song: "
 #define KDEPG_ALBUM "Album: "
 #define KDEPG_KOMP "Comp: "
@@ -43,7 +43,7 @@ int epg_kdg(const char *epgdescr, time_t epgstart, time_t epgend);
 #define UMRADIO_TID1 111
 #define UMRADIO_TID2 131
 // EPG-Info
-#define	UMEPG_SUFFIX "Music Choice bei Unitymedia"
+#define UMEPG_SUFFIX "Music Choice bei Unitymedia"
 int epg_unitymedia(const char *epgtitle, const char *epgdescr, time_t epgstart, time_t epgend);
 
 
