@@ -22,6 +22,14 @@ enum eRadioSkin
   eRadioSkinEnigmaAppleGreen,
   eRadioSkinEnigmaWomensLike,
   eRadioSkinEnigmaYellowSun,
+  eRadioSkinEnigmaBlack,
+  eRadioSkinEnigmaBlue,
+  eRadioSkinEnigmaBlue2,
+  eRadioSkinEnigmaBlue3,
+  eRadioSkinEnigmaCoolblue,
+  eRadioSkinEnigmaGrey,
+  eRadioSkinEnigmaMoBuntu,
+  eRadioSkinEnigmabgw,
   eRadioSkinDeepBlueDefault,
   eRadioSkinSilverGreenDefault,
   eRadioSkinLightBlueDefault,
@@ -32,20 +40,26 @@ enum eRadioSkin
   eRadioSkinSoppalusikkaBlackberry,
   eRadioSkinSoppalusikkaCitron,
   eRadioSkinElchiDefault,
+  eRadioSkinElchiMVBlack,
+  eRadioSkinElchiMVWhite,
+  eRadioSkinElchiMoose,
   eRadioSkinElchiPluginDefault,
   eRadioSkinEgalOrangeDefault,
   eRadioSkinMoronimoDefault,
   eRadioSkinDuotoneDefault,
+  eRadioSkinEgalT2Default,
+  eRadioSkinEgalT2BluYell,
+  eRadioSkinEgalT2Purple,
   eRadioSkinMaxNumber
 };
 
 struct cRadioSkin
 {
-  const char *name;		// Theme~Skin
-  int clrTitleBack;		// Title-Background
-  int clrTitleText;		// Title-Text
-  int clrBack;			// Background
-  int clrText;			// Text
+  const char *name;     // Theme~Skin
+  int clrTitleBack;     // Title-Background
+  int clrTitleText;     // Title-Text
+  int clrBack;          // Background
+  int clrText;          // Text
 };
 extern const cRadioSkin radioSkin[eRadioSkinMaxNumber];
 
