@@ -374,10 +374,6 @@ void PatFilter::Process(u_short Pid, u_char Tid, const u_char *Data, int Length)
 #else
         char ALangs[MAXAPIDS + 1][4] = { "" };
         char DLangs[MAXDPIDS + 1][4] = { "" };
-#endif
-#if VDRVERSNUM >= 10510
-        int Spids[MAXDPIDS + 1] = { 0 };
-        char SLangs[MAXDPIDS + 1][MAXLANGCODE2] = { "" };
 #endif
         int Tpid = 0;
         int NumApids = 0;
@@ -452,11 +448,7 @@ void PatFilter::Process(u_short Pid, u_char Tid, const u_char *Data, int Length)
                 delete d;
                 }
             }
-#if VDRVERSNUM >= 10510
-        Menu->SetPids(pmt.getServiceId(),Vpid, Vpid ? Ppid : 0, Apids, ALangs, Dpids, DLangs, Spids, SLangs, Tpid);
-#else
         Menu->SetPids(pmt.getServiceId(),Vpid, Vpid ? Ppid : 0, Apids, ALangs, Dpids, DLangs, Tpid);
-#endif
         Menu->SetCaIds(pmt.getServiceId(),CaDescriptors->CaIds());
         Menu->SetCaDescriptors(pmt.getServiceId(),CaDescriptorHandler.AddCaDescriptors(CaDescriptors));
         }
