@@ -129,6 +129,8 @@ cMainMenuRotor::cMainMenuRotor(void):cOsdMenu(tr("Rotor")), cThread("Rotor-Plugi
   int tmp=false;
   cPluginManager::CallFirstService("Text2Skin-TTF",&tmp);
 #endif
+  cDvbTransponderParameters dtp(OldChannel->Parameters());
+  char polarization = dtp.Polarization();
   edw=cSkinDisplay::Current()->EditableWidth();
   oldupdate=Setup.UpdateChannels;
   Setup.UpdateChannels=0;
@@ -139,7 +141,7 @@ cMainMenuRotor::cMainMenuRotor(void):cOsdMenu(tr("Rotor")), cThread("Rotor-Plugi
   RotorPos=RotorPositions.GetfromSource(OldChannel->Source());
   Position=RotorPos->GetPos();
   Frequenz=OldChannel->Frequency();
-  if ((*OldChannel).Polarization() == 'v' || (*OldChannel).Polarization() == 'V')
+  if (toupper(polarization) == 'V')
     Pol='V';
   else
     Pol='H';
@@ -212,7 +214,7 @@ void cMainMenuRotor::AddMenuItems()
   Add(n[5]);
   Add(new cMenuEditFreqItem(tr("Frequency"),&Frequenz,&Pol,osUser1));
   Add(new cMenuEditSymbItem(tr("Symbolrate"),&Symbolrate,0,50000,osUser1));
-  Add(new cOsdItem(CenterText(tr("Scan Transponder"),edw),osUser2));
+//  Add(new cOsdItem(CenterText(tr("Scan Transponder"),edw),osUser2));
   CursorDown();
 }
 
@@ -231,7 +233,7 @@ void cMainMenuRotor::Action(void)
 
       CHECK(ioctl(Fd_Frontend, FE_READ_SNR, &SNRChip));
       usleep(15);
-      for (i=0; ((i+1)*Width(LINE))<=((signed long)SNRChip*(edw-Width("SNR:  99% = -99.9dB"))/65536);i++)
+      for (i=0; ((i+1)*Width(LINE))<=(signed long)(SNRChip*(edw-Width("SNR:  99% = -99.9dB"))/65536);i++) 
         buf[i]=edw>100 ? LINE : '|';
       int bw = i*Width(LINE);
       for ( ; Width(BLANK)<=(signed long)(edw-Width("SNR:  99% = -99.9dB")-bw); i++,bw+=Width(BLANK))
@@ -243,7 +245,7 @@ void cMainMenuRotor::Action(void)
  
       CHECK(ioctl(Fd_Frontend, FE_READ_SIGNAL_STRENGTH, &SSChip));
       usleep(15);
-      for (i=0; ((i+1)*Width(LINE))<=((signed long)SSChip*(edw-Width("SNR:  99% = -99.9dB"))/65535);i++)
+      for (i=0; ((i+1)*Width(LINE))<=(signed long)(SSChip*(edw-Width("SNR:  99% = -99.9dB"))/65535);i++)
         buf[i]=edw>100 ? LINE : '|';
       bw = i*Width(LINE);
       for ( ; Width(BLANK)<=(signed long)(edw-Width("SNR:  99% = -99.9dB")-bw); i++,bw+=Width(BLANK))
@@ -299,6 +301,7 @@ eOSState cMainMenuRotor::ProcessKey(eKeys Key)
   eOSState state = cOsdMenu::ProcessKey(Key);
   if (state == osUser1)
   {
+/*
     cChannel *SChannel = new cChannel;
     *SChannel = *OldChannel;
     SChannel->cChannel::SetSatTransponderData(RotorPos->R_Code(),Frequenz,Pol,Symbolrate,FEC_AUTO);
@@ -308,9 +311,11 @@ eOSState cMainMenuRotor::ProcessKey(eKeys Key)
     Setup.DiSEqC=false;
     cDevice::GetDevice(data.DvbCard)->SwitchChannel(SChannel,HasSwitched);
     Setup.DiSEqC=UseDiseqc;
+*/
   }
   if (state == osUser2)
   {
+/*
     cChannel *SChannel = new cChannel;
     *SChannel = *OldChannel;
     SChannel->cChannel::SetSatTransponderData(RotorPos->R_Code(),Frequenz,Pol,Symbolrate,FEC_AUTO);
@@ -322,6 +327,7 @@ eOSState cMainMenuRotor::ProcessKey(eKeys Key)
     cDevice::GetDevice(data.DvbCard)->SwitchChannel(SChannel,HasSwitched);
     Setup.DiSEqC=UseDiseqc;
     AddSubMenu(new cMenuScan);
+*/
   }
   if (state == osUnknown)
   {
@@ -349,7 +355,7 @@ eOSState cMainMenuRotor::ProcessKey(eKeys Key)
   return state;
 }
 
-
+/*
 cMenuScan::cMenuScan():cOsdMenu("Transponder-Scan")
 {
   num=0;
@@ -381,7 +387,8 @@ eOSState cMenuScan::ProcessKey(eKeys Key)
       case kGreen: for (int i=0; i<num; i++)
                      AddChannel(i);
                    break;
-      case kOk: if (PFilter && PFilter->EndOfScan())
+      case kOk: 
+		if (PFilter && PFilter->EndOfScan())
                 {
                   cDevice::GetDevice(data.DvbCard)->Detach(PFilter);
                   PFilter=NULL;
@@ -389,7 +396,7 @@ eOSState cMenuScan::ProcessKey(eKeys Key)
                   SFilter=NULL;
                 }     
                 cDevice::GetDevice(data.DvbCard)->SwitchChannel(&Channel[Current()],true);
-                break;
+		break;
       default: return state;
       }
     state=osContinue;
@@ -412,10 +419,6 @@ void cMenuScan::AddChannel(int Num)
 #else
     char ALangs[MAXAPIDS + 1][4] = { "" };
     char DLangs[MAXDPIDS + 1][4] = { "" };
-#endif
-#if VDRVERSNUM >= 10510
-    int Spids[MAXDPIDS + 1] = { 0 };
-    char SLangs[MAXDPIDS + 1][MAXLANGCODE2] = { "" };
 #endif
     int CaIds[MAXCAIDS+1] = { 0 };
     for (int i=0; i<=MAXAPIDS; i++)
@@ -430,11 +433,7 @@ void cMenuScan::AddChannel(int Num)
     }
     for (int i=0; i<=MAXCAIDS; i++)
       CaIds[i]=Channel[Num].Ca(i);
-#if VDRVERSNUM >= 10510
-    channel->SetPids(Channel[Num].Vpid(),Channel[Num].Ppid(),Apids,ALangs,Dpids,DLangs,Spids,SLangs,Channel[Num].Tpid());
-#else
     channel->SetPids(Channel[Num].Vpid(),Channel[Num].Ppid(),Apids,ALangs,Dpids,DLangs,Channel[Num].Tpid());
-#endif
     channel->SetCaIds(CaIds);
   }
   else
@@ -463,9 +462,7 @@ void cMenuScan::NewChannel(const cChannel *Transponder, const char *Name, const
   num++;
 }
 
-#if VDRVERSNUM >= 10510
-void cMenuScan::SetPids(int Sid,int Vpid, int Ppid, int *Apids, char ALangs[][MAXLANGCODE2], int *Dpids, char DLangs[][MAXLANGCODE2], int *Spids, char SLangs[][MAXLANGCODE2], int Tpid)
-#elif VDRVERSNUM >= 10332
+#if VDRVERSNUM>=10332
 void cMenuScan::SetPids(int Sid,int Vpid, int Ppid, int *Apids, char ALangs[][MAXLANGCODE2], int *Dpids, char DLangs[][MAXLANGCODE2], int Tpid)
 #else
 void cMenuScan::SetPids(int Sid,int Vpid, int Ppid, int *Apids, char ALangs[][4], int *Dpids, char DLangs[][4], int Tpid)
@@ -474,11 +471,7 @@ void cMenuScan::SetPids(int Sid,int Vpid, int Ppid, int *Apids, char ALangs[][4]
   for (int i=0; i<num; i++)
     if (Sid==Channel[i].Sid())
     {
-#if VDRVERSNUM >= 10510
-      Channel[i].SetPids(Vpid,Ppid,Apids,ALangs,Dpids,DLangs,Spids,SLangs,Tpid);
-#else
       Channel[i].SetPids(Vpid,Ppid,Apids,ALangs,Dpids,DLangs,Tpid);
-#endif
       display(i);
     }
 }
@@ -519,6 +512,7 @@ void cMenuScan::display(int num)
   n[num]->SetText(buf);
   cOsdMenu::Display();
 }
+*/
 
 config data;
 
