@@ -1,18 +1,29 @@
 /*
- * ttxtsubs.c: A plugin for the Video Disk Recorder
+ * vdr-ttxtsubs - A plugin for the Linux Video Disk Recorder
+ * Copyright (c) 2003 - 2008 Ragnar Sundblad <ragge@nada.kth.se>
  *
- * See the README file for copyright information and how to reach the author.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2 of the License, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
+ * details.
+ *
+ * You should have received a copy of the GNU General Public License along with
+ * this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
- * $Id: ttxtsubs.c,v 1.26 2004/05/31 19:59:14 ragge Exp $
  */
 
 #include <vdr/plugin.h>
 #include <vdr/status.h>
 #include <vdr/vdrttxtsubshooks.h>
 #include <vdr/menuitems.h>
-#include <vdr/thread.h>
 #include <vdr/config.h>
-#include <semaphore.h>
+#include <vdr/tools.h>
 
 #define TIMEMEASURE 0
 #if TIMEMEASURE
@@ -28,26 +39,19 @@
 
 #include "ttxtsubsglobals.h"
 #include "ttxtsubsdisplayer.h"
-#include "ttxtsubsrecorder.h"
-#include "ttxtsubsi18n.h"
 #include "utils.h"
-#include "siinfo.h"
-#include "ttxtsubs.h"
 #include "ttxtsubspagemenu.h"
 #include "ttxtsubschannelsettings.h"
+#include "ttxtsubslivereceiver.h"
 
-#if defined(APIVERSNUM) && APIVERSNUM < 10400
-#error "This version of ttxtsubs only works with vdr version >= 1.4.0!"
+#if defined(APIVERSNUM) && APIVERSNUM < 10706
+#error "This version of ttxtsubs only works with vdr version >= 1.7.6!"
+#endif
+#if TTXTSUBSVERSNUM != 2
+#error "This version of ttxtsubs requires the ttxtsubs patch version 2 to be applied to VDR!!"
 #endif
 
-#ifndef trNOOP
-#define trNOOP(s) (s)
-#endif
-#ifndef trVDR
-#define trVDR(s) tr(s)
-#endif
-
-static const char *VERSION        = "0.0.5-RRE";
+static const char *VERSION        = "0.2.2";
 static const char *DESCRIPTION    = trNOOP("Teletext subtitles");
 
 cTtxtsubsConf globals;
@@ -57,7 +61,7 @@ cTtxtSubsChannelSettings TtxtSubsChannelSettings;
 // XXX should be replaced with something that allows for other languages and for real language names!
 // <http://www.avio-systems.com/dtvcc/iso639-2.txt>
 // <http://www.loc.gov/standards/iso639-2/englangn_ascii.html>
-char *gLanguages[][2] = { 
+const char *gLanguages[][2] = { 
   {"",""},       //None
   {"eng",""},    //English
   {"deu","ger"}, //Deutsch
@@ -80,7 +84,8 @@ char *gLanguages[][2] = {
   {"est",""},    //Estonian
   {"dan",""},    //Danish
   {"cze","ces"}, //Czech
-  {"tur",""}     //Turkish
+  {"tur",""},    //Turkish
+  {"und",""}     //Undefined/Manual
 };
 const char *gLanguageNames[] = {
   "-",
@@ -105,11 +110,12 @@ const char *gLanguageNames[] = {
   "Eesti",
   "Dansk",
   "esky",
-  "Trke"
+  "Trke",
+  "Undef/Manual"
 };
 int gNumLanguages = sizeof(gLanguages) / sizeof(gLanguages[0]);
 
-class cPluginTtxtsubs : public cPlugin, public cStatus, public cVDRTtxtsubsHookListener, public cThread {
+class cPluginTtxtsubs : public cPlugin, public cStatus, public cVDRTtxtsubsHookListener {
 public:
   cPluginTtxtsubs(void);
   virtual ~cPluginTtxtsubs();
@@ -143,15 +149,11 @@ public:
   // -- cVDRTtxtsubsHookListener
   virtual void HideOSD(void) { HideTtxt(); }
   virtual void ShowOSD(void) { ShowTtxt(); }
-  virtual void PlayerTeletextData(uint8_t *p, int length);
-  virtual cTtxtSubsRecorderBase *NewTtxtSubsRecorder(cDevice *dev, const cChannel *ch);
+  virtual void PlayerTeletextData(uint8_t *p, int length, bool IsPesRecording, const struct tTeletextSubtitlePage teletextSubtitlePages[], int pageCount);
+  virtual int ManualPageNumber(const cChannel *channel);
 
-  // -- cThread
-  void Action(void);
-  
   // -- internal
  private:
-  void StartTtxtLive(const cDevice *Device, tChannelID chnid, int pid, int page);
   void StartTtxtPlay(int page);
   void StopTtxt(void);
   void ShowTtxt(void);
@@ -160,19 +162,12 @@ public:
   void parseHIs(const char *val);
 
 private:
-  cTtxtSubsDisplayer *mDispl;
+  cTtxtSubsPlayer *mDispl;
+  cMutex mDisplLock;;
 
   char mOldLanguage[4]; // language chosen from previous version
   int mOldHearingImpaired; // HI setting chosen from previous version
-
-  // ugly hack for now
-  int mPage;
-  // wait for channel switch
-  sem_t chswitchwait;
-  cMutex getchmutex;
-  int switchChannel;
-  int lastc;
-  const cDevice *switchDevice;
+  cTtxtSubsLiveReceiver* mLiveReceiver;
 };
 
 class cMenuSetupTtxtsubs : public cMenuSetupPage {
@@ -187,10 +182,6 @@ class cMenuSetupTtxtsubs : public cMenuSetupPage {
   int mLangHI[MAXLANGUAGES];
   int mSavedFrenchSpecial;
   int mDoStore;
-  int mTransparency;
-  int mRed;
-  int mGreen;
-  int mBlue;
   cTtxtsubsConf mConf;
 };
 
@@ -198,7 +189,8 @@ class cMenuSetupTtxtsubs : public cMenuSetupPage {
 cPluginTtxtsubs::cPluginTtxtsubs(void)
   :
   mDispl(NULL),
-  mOldHearingImpaired(0)
+  mOldHearingImpaired(0),
+  mLiveReceiver(0)
 {
   // Initialize any member variables here.
   // DON'T DO ANYTHING ELSE THAT MAY HAVE SIDE EFFECTS, REQUIRE GLOBAL
@@ -206,13 +198,12 @@ cPluginTtxtsubs::cPluginTtxtsubs(void)
   
   memset(mOldLanguage, 0, 4);
   strncpy(globals.mLanguages[0][0], "unk", 4);
-  sem_init(&chswitchwait,0,0);
-  lastc=0;
 }
 
 cPluginTtxtsubs::~cPluginTtxtsubs()
 {
   // Clean up after yourself!
+  DELETENULL(mLiveReceiver);
 }
 
 const char *cPluginTtxtsubs::CommandLineHelp(void)
@@ -231,12 +222,7 @@ bool cPluginTtxtsubs::Start(void)
 {
   // Start any background activities the plugin shall perform.
 
-  cThread::Start();
-
   TtxtSubsChannelSettings.Load(AddDirectory(ConfigDirectory("ttxtsubs"),"channelsettings.dat"));
-#if defined(APIVERSNUM) && APIVERSNUM < 10507
-  RegisterI18n(Phrases);
-#endif
   if(!memcmp(globals.mLanguages[0][0], "unk", 3)) {
     // no language found in setup
     if(strlen(mOldLanguage)) {
@@ -259,20 +245,8 @@ bool cPluginTtxtsubs::Start(void)
       globals.mHearingImpaireds[0][1] = mOldHearingImpaired;
     } else {
       // get lang from OSD lang
-#if defined(APIVERSNUM) && APIVERSNUM < 10507
-      int n = Setup.OSDLanguage + 1;
-      if(n > gNumLanguages) {
-	strncpy(globals.mLanguages[0][0], "eng", 4);
-      } else {
-	strncpy(globals.mLanguages[0][0], gLanguages[n][0], 4);
-	globals.mLanguages[0][0][3] = '\0';
-	strncpy(globals.mLanguages[0][1], gLanguages[n][1], 4);
-	globals.mLanguages[0][1][3] = '\0';
-      }
-#else
       strncpy(globals.mLanguages[0][0], Setup.OSDLanguage, 4);
       globals.mLanguages[0][0][3] = '\0';
-#endif
     }
   }
 
@@ -285,7 +259,6 @@ bool cPluginTtxtsubs::Start(void)
 
 void cPluginTtxtsubs::Stop(void)
 {
-  cThread::Cancel();
   StopTtxt();
 }
 
@@ -296,6 +269,7 @@ void cPluginTtxtsubs::Housekeeping(void)
 
 const char *cPluginTtxtsubs::MainMenuEntry(void)
 {
+  bool haveChannel = Channels.GetByNumber(cDevice::ActualDevice()->CurrentChannel()) != NULL;
   switch(globals.mMainMenuEntry) {
   case 1:
     if(globals.mRealDoDisplay)
@@ -303,12 +277,9 @@ const char *cPluginTtxtsubs::MainMenuEntry(void)
     else
       return tr("Display teletext subtitles");
   case 2:
-    if(globals.mBottomLB)
-      return tr("Position Teletext Subtitles for 4:3/Anamorph");
-    else
-      return tr("Position Teletext Subtitles for Letterbox");
+      return tr("deprecated");
   case 3:
-    return tr("Page Selection");
+    if (haveChannel) return tr("Page Selection");
   default:
     return NULL;
   }
@@ -324,9 +295,6 @@ cOsdObject *cPluginTtxtsubs::MainMenuAction(void)
   case 1:
     globals.mRealDoDisplay = globals.mRealDoDisplay ? 0 : 1;
     return NULL;
-  case 2:
-    globals.mBottomLB = globals.mBottomLB ? 0 : 1;
-    return NULL;
   case 3:
     return new cTtxtsubsPageMenu();
   default:
@@ -347,16 +315,12 @@ bool cPluginTtxtsubs::SetupParse(const char *Name, const char *Value)
   else if(!strcasecmp(Name, "Record")) globals.mDoRecord = atoi(Value);
   else if(!strcasecmp(Name, "LiveDelay")) globals.mLiveDelay = atoi(Value);
   else if(!strcasecmp(Name, "ReplayDelay")) globals.mReplayDelay = atoi(Value);
+  else if(!strcasecmp(Name, "ReplayTsDelay")) globals.mReplayTsDelay = atoi(Value);
   else if(!strcasecmp(Name, "MainMenuEntry")) globals.mMainMenuEntry = atoi(Value);
-  else if(!strcasecmp(Name, "TextPos")) globals.mTextPos = atoi(Value);
-  else if(!strcasecmp(Name, "BottomLB")) globals.mBottomLB = atoi(Value);
-  else if(!strcasecmp(Name, "BottomAdj")) globals.mBottomAdj = atoi(Value);
   else if(!strcasecmp(Name, "FrenchSpecial")) globals.mFrenchSpecial = atoi(Value);
-  else if(!strcasecmp(Name, "LineSpacing")) globals.mLineSpacing = atoi(Value);
   else if(!strcasecmp(Name, "DvbSources")) globals.mDvbSources = atoi(Value);
-  else if(!strcasecmp(Name, "FgColor")) globals.mFgColor = atoi(Value);
-  else if(!strcasecmp(Name, "BgColor")) globals.mBgColor = atoi(Value);
-  else if(!strcasecmp(Name, "CustomColor")) globals.mCustomColor = atoi(Value);
+  else if(!strcasecmp(Name, "FontSize")) globals.mFontSize = atoi(Value);
+  else if(!strcasecmp(Name, "OutlineWidth")) globals.mOutlineWidth = atoi(Value);
   else if(!strcasecmp(Name, "Languages")) parseLanguages(Value);
   else if(!strcasecmp(Name, "HearingImpaireds")) parseHIs(Value);
   // Handle old settings
@@ -368,157 +332,56 @@ bool cPluginTtxtsubs::SetupParse(const char *Name, const char *Value)
   return true;
 }
 
-void cPluginTtxtsubs::Action(void)
-{
-  while(true) {
-    sem_wait(&chswitchwait);
-    int cn;
-    const cDevice *dev;
-    getchmutex.Lock();
-    cn=switchChannel;
-    dev=switchDevice;
-    getchmutex.Unlock();
-    if (cn!=lastc) {
-      StopTtxt();
-      lastc=0;
-      cChannel *c = Channels.GetByNumber(cn);
-      if(c) {
-        //int manual_page = TtxtSubsChannelSettings.Page(c->GetChannelID());
-        //if (manual_page>0) { 
-        //  printf("ttxtsubs: manual page %03x selected\n", manual_page);
-        //  if (c->Tpid()) StartTtxtLive(dev, c->GetChannelID(), c->Tpid(), manual_page);
-        //} else if(manual_page==0) {
-          struct ttxtinfo info;
-          int pid, page;
-          char lang[4] = "";
-          bool geterror=true;
-          if((globals.dvbSources() == 0) || ((globals.dvbSources() == 1) && c->IsSat()) || ((globals.dvbSources() == 2) && c->IsTerr()) || ((globals.dvbSources() == 3) && c->IsCable()))
-          {
-             int tries=0; //sometimes it doesn't find subtitles even if they're there
-                          //the problem is that while the dish is moving it's picking the signal
-                          //from a different satellite, so what really should be done is to
-                          //check if the dish is positioned
-             while(tries<=2) {
-               while(geterror=GetTtxtInfo(dev->ActualDevice()->CardIndex(), c, &info)) {
-                 esyslog("ttxtsubs: Error: GetTtxtInfo failed!");
-                 cCondWait::SleepMs(1000);
-                 getchmutex.Lock();
-                 bool changed=(cn!=switchChannel);
-                 getchmutex.Unlock();
-                 if (changed) {
-                   esyslog("ttxtsubs: Channel changed while looking for teletext info");
-                   break;
-                 }  
-               }  
-               if (!geterror) {
-                 if(FindSubs(&info, &pid, &page, lang)) {
-                   //dprint("CHANNELSWITCH, pid: %d page: %x\n", pid, page);
-                   mPage = page; // XXX remember this for playback (temporary hack)!
-                   for(int i = 0; i < gNumLanguages; i++) {
-                      if(!memcmp(lang, gLanguages[i][0], 3) ||
-                         !memcmp(lang, gLanguages[i][1], 3)) {
-                        globals.mI18nLanguage = i - 1;
-                      }
-                   }
-#if defined(APIVERSNUM) && APIVERSNUM < 10507
-                   if(globals.mI18nLanguage < 0 || globals.mI18nLanguage >= I18nNumLanguages)
-#else
-                   if(globals.mI18nLanguage < 0 || globals.mI18nLanguage >= I18nLanguages()->Size())
-#endif
-                     globals.mI18nLanguage = 0; // default to iso8859-1 if no predefined charset
-                   StartTtxtLive(dev, c->GetChannelID(), pid, page);
-                   FreeTtxtInfoData(&info);
-                   break; 
-                 } else {  //!FindSubs
-                   FreeTtxtInfoData(&info);
-                   tries++;
-                 }  
-               } else break; //if(!geterror)..
-             } //while(tries<=2)  
-          }
-          lastc=cn;
-        //}   
-      }
-    }  
-  }
-}
-
 void cPluginTtxtsubs::ChannelSwitch(const cDevice *Device, int ChannelNumber)
 {
-#if TIMEMEASURE
-  struct timeval tv, tv2;
-  gettimeofday(&tv, NULL);  
-#endif
   //dprint("cPluginTtxtsubs::ChannelSwitch(devicenr: %d, channelnr: %d) - mDispl: %x\n",  Device->DeviceNumber(), ChannelNumber, mDispl); // XXX
-
-  if(Device->IsPrimaryDevice() && !Device->Replaying()) {
-    if(ChannelNumber) {
-      getchmutex.Lock();
-      switchChannel=ChannelNumber;
-      switchDevice=Device;
-      getchmutex.Unlock();
-      sem_post(&chswitchwait);
-    } 
+  if (Device->IsPrimaryDevice() && !Device->Replaying() && ChannelNumber)
+  {
+    StopTtxt();
+    DELETENULL(mLiveReceiver);
+    if (!Device->Replaying() && !Device->Transferring())
+    {
+      cChannel* channel = Channels.GetByNumber(ChannelNumber);
+      if (channel && channel->Tpid())
+      {
+        mLiveReceiver = new cTtxtSubsLiveReceiver(channel, this);
+        cDevice::PrimaryDevice()->AttachReceiver(mLiveReceiver);
+      }
+    }
+    StartTtxtPlay(0x000);
   }
-#if TIMEMEASURE
-  gettimeofday(&tv2, NULL);  
-  TIMEVALSIMPL(tv, tv2);
-  // tv2.tv_sec - tv.tv_sec + 1000000 * (tv2.tv_usec - tv.tv_usec);
-  isyslog("ttxtsubs: Channel switch time: %d.%06d s",
-	  tv2.tv_sec - tv.tv_sec, tv2.tv_usec - tv.tv_usec);
-#endif
 }
 
 void cPluginTtxtsubs::Replaying(const cControl *Control, const char *Name, const char *FileName, bool On)
 {
   //dprint("cPluginTtxtsubs::Replaying\n"); // XXX
   StopTtxt();
-  if(On)
-    StartTtxtPlay(mPage);
-  // XXX this page number is just a fallback for old recordings which
-  // don't have a index page
-}
-
-void cPluginTtxtsubs::PlayerTeletextData(uint8_t *p, int length)
-{
-  cTtxtSubsPlayer *r = dynamic_cast<cTtxtSubsPlayer *>(mDispl);
-
-  if(!r) {
-    esyslog("ttxtsubs: ERROR: not a cTtxtSubsPlayer!");
-    return;
+  if (On)
+  {
+    DELETENULL(mLiveReceiver);
+    StartTtxtPlay(0x000);
   }
-
-  r->PES_data(p, length);
 }
 
-cTtxtSubsRecorderBase *cPluginTtxtsubs::NewTtxtSubsRecorder(cDevice *dev, const cChannel *ch)
+void cPluginTtxtsubs::PlayerTeletextData(uint8_t *p, int length, bool IsPesRecording, const struct tTeletextSubtitlePage teletextSubtitlePages[], int pageCount)
 {
-  if(globals.mDoRecord)
-    return new cTtxtSubsRecorder(dev, ch);
-  else
-    return NULL;
+  mDisplLock.Lock();
+  if (mDispl)
+    mDispl->PES_data(p, length, IsPesRecording, teletextSubtitlePages, pageCount);
+  mDisplLock.Unlock();
 }
 
+int cPluginTtxtsubs::ManualPageNumber(const cChannel *channel)
+{
+    cTtxtSubsChannelSetting *setting = TtxtSubsChannelSettings.Get(channel);
+    if (setting && setting->PageMode() == PAGE_MODE_MANUAL)
+      return setting->PageNumber();
+    else
+      return 0;
+}
 
 // --  internal
 
-void cPluginTtxtsubs::StartTtxtLive(const cDevice *Device, tChannelID chnid, int pid, int page)
-{
-  //dprint("cPluginTtxtsubs::StartTtxtLive(devicenr: %d, pid: %d, page: %03x)\n",
-  //	 Device->DeviceNumber(), pid, page); // XXX
-
-  if(!mDispl) {
-    cTtxtSubsLiveReceiver *r;
-    //dprint("teletext subtitles started on pid %d\n", pid);
-    mDispl = r = new cTtxtSubsLiveReceiver(chnid, pid, page); // takes 0.01-0.015 s
-    if(!cDevice::PrimaryDevice()->ActualDevice()->AttachReceiver(r))
-                                    // takes 0.02-0.04 s on a full featured card
-      esyslog("ttxtsubs: Error: AttachReceiver failed!");
-    ShowTtxt();
-  } else
-    esyslog("ttxtsubs: Error: StartTtxtLive called when already started!");
-}
-
 void cPluginTtxtsubs::StartTtxtPlay(int backup_page)
 {
   //dprint("cPluginTtxtsubs::StartTtxtPlay\n");
@@ -535,12 +398,10 @@ void cPluginTtxtsubs::StopTtxt(void)
 {
   //dprint("cPluginTtxtsubs::StopTtxt\n");
 
-  if(mDispl) {
-    cTtxtSubsDisplayer *d = mDispl;
-    HideTtxt();
-    mDispl = NULL;
-    delete d; // takes 0.03-0.04 s
-  }
+  HideTtxt();
+  mDisplLock.Lock();
+  DELETENULL(mDispl); // takes 0.03-0.04 s
+  mDisplLock.Unlock();
 }
 
 void cPluginTtxtsubs::ShowTtxt(void)
@@ -619,10 +480,7 @@ void cMenuSetupTtxtsubsLanguages::Store(void)
 }
 
 const char * mainMenuAlts[5] = {NULL, NULL, NULL, NULL, NULL};
-const char * textPosAlts[4];
 const char * dvbSources[5];
-const char * textColors[12];
-const char * const colorValues[17] = {"#00", "#11", "#22", "#33", "#44", "#55", "#66", "#77", "#88", "#99", "#AA", "#BB", "#CC", "#DD", "#EE", "#FF", NULL};
 
 cMenuSetupTtxtsubs::cMenuSetupTtxtsubs(cPluginTtxtsubs *ttxtsubs, int doStore)
   :
@@ -630,45 +488,23 @@ cMenuSetupTtxtsubs::cMenuSetupTtxtsubs(cPluginTtxtsubs *ttxtsubs, int doStore)
   mDoStore(doStore),
   mConf(globals)
 {
-  char *buf = NULL;
-  //static char *mainMenuAlts[] = {"off", "Display on/off", "4:3/Letterbox", "This menu"};
+  //static char *mainMenuAlts[] = {"off", "Display on/off", "This menu"};
   // can't get it to store changes in file
   if(mainMenuAlts[0] == NULL) {
     mainMenuAlts[0] = trVDR("off");
     mainMenuAlts[1] = tr("Display on/off");
-    mainMenuAlts[2] = tr("4:3/Letterbox");
+    mainMenuAlts[2] = tr("4:3/Letterbox (deprecated)");
     mainMenuAlts[3] = tr("Page Mode");
     mainMenuAlts[4] = NULL;
 
-    textPosAlts[0] = tr("Left");
-    textPosAlts[1] = tr("Center");
-    textPosAlts[2] = tr("Right");
-    textPosAlts[3] = NULL;
-
     dvbSources[0] = tr("All");
     dvbSources[1] = tr("Only DVB-S");
     dvbSources[2] = tr("Only DVB-T");
     dvbSources[3] = tr("Only DVB-C");
     dvbSources[4] = NULL;
-
-    textColors[0] = tr("Black");
-    textColors[1] = tr("White");
-    textColors[2] = tr("Red");
-    textColors[3] = tr("Green");
-    textColors[4] = tr("Yellow");
-    textColors[5] = tr("Magenta");
-    textColors[6] = tr("Blue");
-    textColors[7] = tr("Cyan");
-    textColors[8] = tr("Custom");
-    textColors[9] = tr("Transparent");
-    textColors[10] = tr("Grey");
-    textColors[11] = NULL;
   }
-  const int numTextPosAlts = sizeof(textPosAlts) / sizeof(textPosAlts[0]) - 1;
   const int numMainMenuAlts = sizeof(mainMenuAlts) / sizeof(mainMenuAlts[0]) - 1;
   const int numDvbSources = sizeof(dvbSources) / sizeof(dvbSources[0]) - 1;
-  const int numTextColors = sizeof(textColors) / sizeof(textColors[0]) - 1;
-  const int numColorValues = sizeof(colorValues) / sizeof(colorValues[0]) - 1;
 
   mSavedFrenchSpecial = mConf.mFrenchSpecial;
 
@@ -688,58 +524,24 @@ cMenuSetupTtxtsubs::cMenuSetupTtxtsubs(cPluginTtxtsubs *ttxtsubs, int doStore)
   Add(new cMenuEditBoolItem(tr("Display Subtitles"), &mConf.mDoDisplay));
   Add(new cMenuEditBoolItem(tr("Record Subtitles"), &mConf.mDoRecord));
   Add(new cMenuEditIntItem(tr("Live Delay"), &mConf.mLiveDelay, 0, 5000));
-  Add(new cMenuEditIntItem(tr("Replay Delay"), &mConf.mReplayDelay, 0, 5000));
+  Add(new cMenuEditIntItem(tr("Replay Delay (PES)"), &mConf.mReplayDelay, 0, 5000));
+  Add(new cMenuEditIntItem(tr("Replay Delay (TS)"), &mConf.mReplayTsDelay, 0, 5000));
   if(mConf.mMainMenuEntry < 0 || mConf.mMainMenuEntry >= numMainMenuAlts)
     mConf.mMainMenuEntry = 0;  // menu item segfaults if out of range
   Add(new cMenuEditStraItem(tr("Main Menu Alternative"), &mConf.mMainMenuEntry,
 			    numMainMenuAlts, mainMenuAlts));
-  if(mConf.mTextPos < 0 || mConf.mTextPos >= numTextPosAlts)
-    mConf.mTextPos = 0;  // menu item segfaults if out of range
-  Add(new cMenuEditStraItem(tr("Text Horizontal Position"), &mConf.mTextPos,
-			    numTextPosAlts, textPosAlts));
-  Add(new cMenuEditBoolItem(tr("Text Vertical Position"),
-			    &mConf.mBottomLB, tr("4:3/Anamorph"), tr("Letterbox")));
-  Add(new cMenuEditIntItem(tr("Text Vertical Adjust"),
-			    &mConf.mBottomAdj, -100, 45));
-  Add(new cMenuEditIntItem( tr("Line Spacing Adjust"),
-			    &mConf.mLineSpacing, -25, 25));
   Add(new cMenuEditBoolItem(tr("Workaround for some French chns"),
 			    &mConf.mFrenchSpecial));
   if(mConf.mDvbSources < 0 || mConf.mDvbSources >= numDvbSources)
     mConf.mDvbSources = 0;  // menu item segfaults if out of range
   Add(new cMenuEditStraItem(tr("DVB Source Selection"),
                           &mConf.mDvbSources, 4, dvbSources));
-  if(mConf.mFgColor < 0 || mConf.mFgColor >= numTextColors)
-    mConf.mFgColor = 1;  // menu item segfaults if out of range
-  Add(new cMenuEditStraItem(tr("Text Color"), &mConf.mFgColor,
-			    numTextColors, textColors));
-  if(mConf.mBgColor < 0 || mConf.mBgColor >= numTextColors)
-    mConf.mBgColor = 0;  // menu item segfaults if out of range
-  Add(new cMenuEditStraItem(tr("Background Color"), &mConf.mBgColor,
-			    numTextColors, textColors));
-
-  // the color code is borrowed from AIO patches
-  mBlue = (mConf.mCustomColor & 0x0000000F);
-  if (mBlue < 0 || mBlue >= numColorValues) mBlue = 0;
-  mGreen = (mConf.mCustomColor & 0x00000F00) >>  8;
-  if (mGreen < 0 || mGreen >= numColorValues) mGreen = 0;
-  mRed = (mConf.mCustomColor & 0x000F0000) >> 16;
-  if (mRed < 0 || mRed >= numColorValues) mRed = 0;
-  mTransparency = (mConf.mCustomColor & 0x0F000000) >> 24;
-  if (mTransparency < 0 || mTransparency >= numColorValues) mTransparency = 0;
-  asprintf(&buf, "%s: --------------------------------------------", tr("Custom Color"));
-  cOsdItem *item = new cOsdItem(buf);
-  item->SetSelectable(false);
-  free(buf);
-  Add(item);
-  Add(new cMenuEditStraItem(tr("Red Value"),          &mRed,          numColorValues, colorValues));
-  Add(new cMenuEditStraItem(tr("Green Value"),        &mGreen,        numColorValues, colorValues));
-  Add(new cMenuEditStraItem(tr("Blue Value"),         &mBlue,         numColorValues, colorValues));
-  Add(new cMenuEditStraItem(tr("Transparency Value"), &mTransparency, numColorValues, colorValues));
+  Add(new cMenuEditIntItem(tr("Font Size (pixel)"), &mConf.mFontSize, 10, MAXFONTSIZE * 2));
+  Add(new cMenuEditIntItem(tr("Font OutlineWidth (pixel)"), &mConf.mOutlineWidth, 1, 10));
 
   for(int n = 0; n < MAXLANGUAGES; n++) {
     char str[100];
-    char *allowedc = "abcdefghijklmnopqrstuvwxyz";
+    const char *allowedc = "abcdefghijklmnopqrstuvwxyz";
 
     cOsdItem *item = new cOsdItem("--------------------------------------------------------");
     item->SetSelectable(false);
@@ -788,20 +590,12 @@ void cMenuSetupTtxtsubs::Store(void)
   SetupStore("Record", mConf.mDoRecord);
   SetupStore("LiveDelay", mConf.mLiveDelay);
   SetupStore("ReplayDelay", mConf.mReplayDelay);
-  SetupStore("TextPos", mConf.mTextPos);
-  SetupStore("BottomLB", mConf.mBottomLB);
-  SetupStore("BottomAdj", mConf.mBottomAdj);
+  SetupStore("ReplayTsDelay", mConf.mReplayTsDelay);
   SetupStore("FrenchSpecial", mConf.mFrenchSpecial);
   SetupStore("MainMenuEntry", mConf.mMainMenuEntry);
-  SetupStore("LineSpacing", mConf.mLineSpacing);
   SetupStore("DvbSources", mConf.mDvbSources);
-  SetupStore("FgColor", mConf.mFgColor);
-  SetupStore("BgColor", mConf.mBgColor);
-  mConf.mCustomColor = (mBlue              ) | (mBlue         <<  4) |
-                       (mGreen        <<  8) | (mGreen        << 12) |
-                       (mRed          << 16) | (mRed          << 20) |
-                       (mTransparency << 24) | (mTransparency << 28);
-  SetupStore("CustomColor", mConf.mCustomColor);
+  SetupStore("FontSize", mConf.mFontSize);
+  SetupStore("OutlineWidth", mConf.mOutlineWidth);
 
   char lstr[MAXLANGUAGES*2*4 + 1];
   char histr[MAXLANGUAGES*2 + 1];
