@@ -677,8 +677,8 @@ eOSState cMenuUndelete::ProcessKey(eKeys Key)
                                   cIndexFile *index = new cIndexFile(NewName, false);
                                   int LastFrame = index->Last() - 1;
                                   if (LastFrame > 0) {
-                                    uchar FileNumber = 0;
-                                    int FileOffset = 0;
+                                    uint16_t FileNumber = 0;
+                                    off_t FileOffset = 0;
                                     index->Get(LastFrame, &FileNumber, &FileOffset);
                                     delete index;
                                     if (FileNumber == 0) {
