@@ -27,6 +27,7 @@
 #include <vdr/plugin.h>
 #include <vdr/timers.h>
 #include <vdr/menu.h>
+#include <vdr/status.h>
 #include <vdr/remote.h>
 #include "recplayer.h"
 #include "mvpreceiver.h"
@@ -41,6 +42,8 @@
 #include "i18n.h"
 #include "vdrcommand.h"
 
+bool ResumeIDLock;
+
 VompClientRRProc::VompClientRRProc(VompClient& x)
  : x(x)
 {
@@ -85,7 +88,7 @@ void VompClientRRProc::threadMethod()
   log->log("RRProc", Log::DEBUG, "threadMethod startup");     
 
   if (req_queue.size() != 0)
-  {
+  { goto entry;
     log->log("RRProc", Log::ERR, "threadMethod err 1");     
     threadUnlock();
     return;
@@ -103,7 +106,7 @@ void VompClientRRProc::threadMethod()
     }
     
     // signalled with something in queue
-    
+entry: 
     log->log("RRProc", Log::DEBUG, "thread woken with req, queue size: %i", req_queue.size());
 
     while (req_queue.size()) 
@@ -232,16 +235,15 @@ bool VompClientRRProc::processPacket()
     case 34:
       result = processGetLanguageContent();
       break;
-    case 666:
-      result = processVDRShutdown();
-      break;
     case VDR_GETMEDIAINFO:
 	     result= processGetMediaInfo();
 	     break;
     case VDR_CLOSECHANNEL:
 	     result= processCloseMediaChannel();
 	     break;
-
+     case 666:
+             result = processVDRShutdown();
+             break;
   }
 
   delete resp;
@@ -278,6 +280,8 @@ int VompClientRRProc::processLogin()
   log->log("RRProc", Log::DEBUG, "written login reply len %lu", resp->getLen());
     
   x.loggedIn = true;
+  x.netLog(); // safe to run here since the client won't start net logging for a while yet
+  
   return 1;
 }
 
@@ -589,12 +593,29 @@ int VompClientRRProc::processGetLanguageContent()
 
 int VompClientRRProc::processGetRecordingsList()
 {
-  int FreeMB;
-  int Percent = VideoDiskSpace(&FreeMB);
-  int Total = (FreeMB / (100 - Percent)) * 100;
-  
   //access protection stuff
+ 
+   char* ShowPinProtectedC = x.config.getValueString("General", "ShowPinProtected");
+   int ShowPinProtected;
+   if (ShowPinProtectedC) {
+      ShowPinProtected = atoi(ShowPinProtectedC);
+      delete[] ShowPinProtectedC;
+   }
+   else
+      ShowPinProtected = 0;  //default if not defined in vomp-MAC.conf is NOT to show pinprotected stuff
+ 
   char* lockDir = x.config.getValueString("General", "LockDir");
+  int FreeMB, Total;
+  int Percent = VideoDiskSpace(&FreeMB);
+  if (Percent == 100)
+  {
+    Total = 0;
+  }
+  else
+  {
+    Total = (FreeMB / (100 - Percent)) * 100;
+  }
+  
   resp->addULONG(Total);
   resp->addULONG(FreeMB);
   resp->addULONG(Percent);
@@ -604,6 +625,17 @@ int VompClientRRProc::processGetRecordingsList()
 
   for (cRecording *recording = Recordings.First(); recording; recording = Recordings.Next(recording))
   {
+    if (!ShowPinProtected) {
+       char fileName[2048];
+       snprintf(fileName, 2047, "%s/protection.fsk", recording->FileName());
+
+       FILE* file = fopen(fileName, "r");
+       if (file) {
+         log->log("RRProc", Log::DEBUG, "Pin-protected entry found: '%s'", (char*)recording->Name() );
+	 continue;
+       }
+    }
+ 
     if( lockDir ) {
         if(strncmp(lockDir,(char*)recording->Name(),strlen(lockDir)) != 0 ) {
            continue;
@@ -644,7 +676,7 @@ int VompClientRRProc::processDeleteRecording()
     {
       if (recording->Delete())
       {
-        // Copy svdrp's way of doing this, see if it works
+        // Copy svdrdeveldevelp's way of doing this, see if it works
 #if VDRVERSNUM > 10300
         ::Recordings.DelByName(recording->FileName());
 #endif
@@ -845,12 +877,28 @@ int VompClientRRProc::processGetChannelsList()
   char* chanConfig = x.config.getValueString("General", "Channels");
   int allChans = 1;
   if (chanConfig) allChans = strcasecmp(chanConfig, "FTA only");
+  char* ShowPinProtectedC = x.config.getValueString("General", "ShowPinProtected");
+  int ShowPinProtected;
+  if (ShowPinProtectedC) {
+    ShowPinProtected = atoi(ShowPinProtectedC);
+    delete[] ShowPinProtectedC;
+  }
+  else
+    ShowPinProtected = 0;  //default if not defined in vomp-MAC.conf is NOT to show pinprotected stuff
+
   //Weired hack for group selection
   char* lockDir = x.config.getValueString("General", "LockDir");
   int locked = 0;
 
   for (cChannel *channel = Channels.First(); channel; channel = Channels.Next(channel))
   {
+#ifdef USE_PINPLUGIN    
+    if (!ShowPinProtected)
+      if (cStatus::MsgChannelProtected(0, channel) == true) {
+	log->log("RRProc", Log::DEBUG, "Channel is pin-protected: %s", channel->Name() );
+        continue;
+      }
+#endif     
     if( lockDir )
     {
       if( channel->GroupSep() )
@@ -886,6 +934,11 @@ int VompClientRRProc::processGetChannelsList()
       resp->addULONG(channel->Number());
       resp->addULONG(type);      
       resp->addString(channel->Name());
+#if VDRVERSNUM < 10703
+      resp->addULONG(2);
+#else
+      resp->addULONG(channel->Vtype());
+#endif      
     }
   }
 
@@ -964,6 +1017,11 @@ int VompClientRRProc::processGetChannelPids()
   // tpid
 
   resp->addULONG(channel->Vpid());
+#if VDRVERSNUM < 10703
+  resp->addULONG(2);
+#else
+  resp->addULONG(channel->Vtype());
+#endif
   resp->addULONG(numApids);
 
 #if VDRVERSNUM < 10300
@@ -1021,6 +1079,7 @@ int VompClientRRProc::processStartStreamingChannel()
   ULONG channelNumber = ntohl(*(ULONG*)req->data);
 
   cChannel* channel = x.channelFromNumber(channelNumber);
+
   if (!channel)
   {
     resp->addULONG(0);
@@ -1156,6 +1215,13 @@ int VompClientRRProc::processStartStreamingRecording()
 
     resp->addULLONG(x.recplayer->getLengthBytes());
     resp->addULONG(x.recplayer->getLengthFrames());
+    
+#if VDRVERSNUM < 10703
+    resp->addUCHAR(true);//added for TS    
+#else
+    resp->addUCHAR(recording->IsPesRecording());//added for TS
+#endif
+
     resp->finalise();
     x.tcp.sendPacket(resp->getPtr(), resp->getLen());
     
@@ -1572,6 +1638,9 @@ int VompClientRRProc::processGetRecInfo()
   time_t timerStart = 0;
   time_t timerStop = 0;
   char* summary = NULL;
+  char* shortext = NULL;
+  char* description = NULL;
+  char* tmp;
   ULONG resumePoint = 0;
 
   if (!recording)
@@ -1595,7 +1664,7 @@ int VompClientRRProc::processGetRecInfo()
     string: language
     string: description
   }
-
+  8 bytes: frames per second
   */
 
   // Get current timer
@@ -1613,25 +1682,70 @@ int VompClientRRProc::processGetRecInfo()
 
   // Get resume point
 
-  char* value = x.config.getValueString("ResumeData", (char*)req->data);
+/*  char* value = x.config.getValueString("ResumeData", (char*)req->data);
   if (value)
   {
     resumePoint = strtoul(value, NULL, 10);
     delete[] value;
+  }*/
+
+  char* ResumeIdC = x.config.getValueString("General", "ResumeId");
+  int ResumeId;
+  if (ResumeIdC) {
+    ResumeId = atoi(ResumeIdC);
+    delete[] ResumeIdC;
   }
+  else
+    ResumeId = 0;  //default if not defined in vomp-MAC.conf
+
+  while (ResumeIDLock)
+    cCondWait::SleepMs(100);
+  ResumeIDLock = true;
+  int OldSetupResumeID = Setup.ResumeID;
+  Setup.ResumeID = ResumeId;				//UGLY: quickly change resumeid
+#if VDRVERSNUM < 10703
+  cResumeFile ResumeFile(recording->FileName());	//get corresponding resume file
+#else
+  cResumeFile ResumeFile(recording->FileName(), recording->IsPesRecording()); //get corresponding resume file
+#endif
+  Setup.ResumeID = OldSetupResumeID;			//and restore it back
+  ResumeIDLock = false;
+
+  int resume = ResumeFile.Read();
+  //isyslog("VOMPDEBUG: resumePoint = %i, resume = %i, ResumeId = %i",resumePoint, resume, ResumeId);
+  if (resume >= 0) 
+    resumePoint = ResumeFile.Read();
+
   log->log("RRProc", Log::DEBUG, "GRI: RP: %lu", resumePoint);
 
   resp->addULONG(resumePoint);
 
-
   // Get summary
 
 #if VDRVERSNUM < 10300
   summary = (char*)recording->Summary();
 #else
   const cRecordingInfo *Info = recording->Info();
-  summary = (char*)Info->ShortText();
-  if (isempty(summary)) summary = (char*)Info->Description();
+  shortext = (char*)Info->ShortText();
+  description = (char*)Info->Description();
+
+  if (isempty(shortext) && !isempty(description)) {
+        // we don't want any "(null)" subtitles
+        summary = description;
+  }
+  else if (!isempty(shortext) && isempty(description)) {
+        // we don't want any "(null)" Description
+        summary = shortext;
+  }
+  else if (isempty(shortext) && isempty(description)) {
+        summary = NULL;
+  }
+  else {
+        asprintf(&tmp, "%s\n\n%s", shortext, description);
+        summary = tmp;
+        // FIXME - Don't know where to set 
+        //free(tmp);
+  }
 #endif
   log->log("RRProc", Log::DEBUG, "GRI: S: %s", summary);
   if (summary)
@@ -1693,6 +1807,13 @@ int VompClientRRProc::processGetRecInfo()
   }
 
 #endif
+  double framespersec;
+#if VDRVERSNUM < 10703
+  framespersec = FRAMESPERSEC;
+#else
+  framespersec = Info->FramesPerSecond();
+#endif
+  resp->adddouble(framespersec);
 
   // Done. send it
 
@@ -1743,7 +1864,11 @@ int VompClientRRProc::processGetMarks()
 
   if (recording)
   {
+#if VDRVERSNUM < 10703
     Marks.Load(recording->FileName());
+#else
+    Marks.Load(recording->FileName(), recording->FramesPerSecond(), recording->IsPesRecording());
+#endif
     if (Marks.Count())
     {
       for (const cMark *m = Marks.First(); m; m = Marks.Next(m))
