@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: config.h 1.310.1.2 2008/09/07 09:33:03 kls Exp $
+ * $Id: config.h 2.28 2010/09/12 11:31:21 kls Exp $
  */
 
 #ifndef __CONFIG_H
@@ -22,13 +22,17 @@
 
 // VDR's own version number:
 
-#define VDRVERSION  "1.6.0-2"
-#define VDRVERSNUM   10600  // Version * 10000 + Major * 100 + Minor
+#define VDRVERSION  "1.7.16"
+#define VDRVERSNUM   10716  // Version * 10000 + Major * 100 + Minor
 
 // The plugin API's version number:
 
-#define APIVERSION  "1.6.0"
-#define APIVERSNUM   10600  // Version * 10000 + Major * 100 + Minor
+#define APIVERSION  "1.7.16"
+#define APIVERSNUM   10716  // Version * 10000 + Major * 100 + Minor
+
+#ifdef USE_YAEPG
+#define YAEPGHDVERSNUM 1
+#endif /* YAEPG */
 
 // When loading plugins, VDR searches them by their APIVERSION, which
 // may be smaller than VDRVERSION in case there have been no changes to
@@ -36,34 +40,22 @@
 // plugins to work with newer versions of the core VDR as long as no
 // VDR header files have changed.
 
-#define VDREXTENSIONS 64
+#ifdef USE_CHANNELBIND
+#define CHANNELBINDINGVERSNUM 2
+#endif /* CHANNELBIND */
 
 #ifdef USE_JUMPPLAY
-#define JUMPPLAYVERSNUM 9
+#define JUMPPLAYVERSNUM 100
 #endif /* JUMPPLAY */
 
-#ifdef USE_LIEMIKUUTIO
-#define LIEMIKUUTIO  121
-#define MAXMAINMENUTITLE 4
-#define MaxTitleName 64
-#endif /* LIEMIKUUTIO */
-
-#ifdef USE_CMDSUBMENU
-#define CMDSUBMENUVERSNUM 7
-#endif /* CMDSUBMENU */
+#ifdef USE_LIEMIEXT
+#define LIEMIKUUTIO  130
+#endif /* LIEMIEXT */
 
 #ifdef USE_MAINMENUHOOKS
 #define MAINMENUHOOKSVERSNUM 1.0
 #endif /* MAINMENUHOOKS */
 
-#ifdef USE_PINPLUGIN
-#define PIN_PLUGIN_PATCH 120
-#endif /* PINPLUGIN */
-
-#ifdef USE_IPTV
-#define PLUGINPARAMPATCHVERSNUM 1
-#endif /* IPTV */
-
 #define MAXPRIORITY 99
 #define MAXLIFETIME 99
 
@@ -71,57 +63,15 @@
 #define DVLVIDPREFER_MAX 12
 #endif /* DVLVIDPREFER */
 
-#define MINOSDWIDTH  480
-#define MAXOSDWIDTH  672
-#define MINOSDHEIGHT 324
-#define MAXOSDHEIGHT 567
-
-#ifdef USE_SOURCECAPS
-#define MAXDEVICES         16 // the maximum number of devices in the system
-#define MAXSOURCECAPS     128 // the maximum number of different sources per device
-#endif /* SOURCECAPS */
-
-#ifdef USE_LNBSHARE
-#ifndef MAXDEVICES
-#define MAXDEVICES         16 // the maximum number of devices in the system
-#endif
-#endif /* LNBSHARE */
+#define MINOSDWIDTH   480
+#define MAXOSDWIDTH  1920
+#define MINOSDHEIGHT  324
+#define MAXOSDHEIGHT 1200
 
 #define MaxFileName 256
 #define MaxSkinName 16
 #define MaxThemeName 16
 
-#ifdef USE_CMDSUBMENU
-class cCommands;
-#endif /* CMDSUBMENU */
-
-class cCommand : public cListObject {
-private:
-  char *title;
-  char *command;
-  bool confirm;
-#ifdef USE_CMDSUBMENU
-  int nIndent;
-  cCommands *childs;
-#endif /* CMDSUBMENU */
-  static char *result;
-public:
-  cCommand(void);
-  virtual ~cCommand();
-  bool Parse(const char *s);
-  const char *Title(void) { return title; }
-  bool Confirm(void) { return confirm; }
-  const char *Execute(const char *Parameters = NULL);
-#ifdef USE_CMDSUBMENU
-  int getIndent(void) { return nIndent; }
-  void setIndent(int nNewIndent) { nIndent = nNewIndent; }
-  cCommands *getChilds(void) { return childs; }
-  int getChildCount(void);
-  bool hasChilds(void) { return getChildCount() > 0; }
-  void addChild(cCommand *newChild);
-#endif /* CMDSUBMENU */
-  };
-
 typedef uint32_t in_addr_t; //XXX from /usr/include/netinet/in.h (apparently this is not defined on systems with glibc < 2.2)
 
 class cSVDRPhost : public cListObject {
@@ -131,6 +81,7 @@ private:
 public:
   cSVDRPhost(void);
   bool Parse(const char *s);
+  bool IsLocalhost(void);
   bool Accepts(in_addr_t Address);
   };
 
@@ -147,9 +98,6 @@ private:
 public:
   cConfig(void) { fileName = NULL; }
   virtual ~cConfig() { free(fileName); }
-#ifdef USE_CMDSUBMENU
-  virtual void AddConfig(T *Object) { cList<T>::Add(Object); }
-#endif /* CMDSUBMENU */
   const char *FileName(void) { return fileName; }
   bool Load(const char *FileName = NULL, bool AllowComments = false, bool MustExist = false)
   {
@@ -179,16 +127,11 @@ public:
                 if (!isempty(s)) {
                    T *l = new T;
                    if (l->Parse(s))
-#ifdef USE_CMDSUBMENU
-                      AddConfig(l);
-#else
                       Add(l);
-#endif /* CMDSUBMENU */
                    else {
                       esyslog("ERROR: error in %s, line %d", fileName, line);
                       delete l;
                       result = false;
-                      break;
                       }
                    }
                 }
@@ -225,25 +168,43 @@ public:
   }
   };
 
-#ifdef USE_CMDSUBMENU
-class cCommands : public cConfig<cCommand> {
+class cNestedItem : public cListObject {
+private:
+  char *text;
+  cList<cNestedItem> *subItems;
 public:
-  virtual void AddConfig(cCommand *Object);
+  cNestedItem(const char *Text, bool WithSubItems = false);
+  virtual ~cNestedItem();
+  virtual int Compare(const cListObject &ListObject) const;
+  const char *Text(void) const { return text; }
+  cList<cNestedItem> *SubItems(void) { return subItems; }
+  void AddSubItem(cNestedItem *Item);
+  void SetText(const char *Text);
+  void SetSubItems(bool On);
+  };
+
+class cNestedItemList : public cList<cNestedItem> {
+private:
+  char *fileName;
+  bool Parse(FILE *f, cList<cNestedItem> *List, int &Line);
+  bool Write(FILE *f, cList<cNestedItem> *List, int Indent = 0);
+public:
+  cNestedItemList(void);
+  virtual ~cNestedItemList();
+  void Clear(void);
+  bool Load(const char *FileName);
+  bool Save(void);
   };
-#else
-class cCommands : public cConfig<cCommand> {};
-#endif /* CMDSUBMENU */
 
 class cSVDRPhosts : public cConfig<cSVDRPhost> {
 public:
+  bool LocalhostOnly(void);
   bool Acceptable(in_addr_t Address);
   };
 
-extern cCommands Commands;
-extern cCommands RecordingCommands;
-#ifdef USE_TIMERCMD
-extern cCommands TimerCommands;
-#endif /* TIMERCMD */
+extern cNestedItemList Folders;
+extern cNestedItemList Commands;
+extern cNestedItemList RecordingCommands;
 extern cSVDRPhosts SVDRPhosts;
 
 class cSetupLine : public cListObject {
@@ -269,25 +230,22 @@ private:
   void StoreLanguages(const char *Name, int *Values);
   bool ParseLanguages(const char *Value, int *Values);
   bool Parse(const char *Name, const char *Value);
-#ifdef USE_SOURCECAPS
-  void StoreSourceCaps(const char *Name);
-  bool ParseSourceCaps(const char *Value);
-#endif /* SOURCECAPS */
   cSetupLine *Get(const char *Name, const char *Plugin = NULL);
   void Store(const char *Name, const char *Value, const char *Plugin = NULL, bool AllowMultiple = false);
   void Store(const char *Name, int Value, const char *Plugin = NULL);
+  void Store(const char *Name, double &Value, const char *Plugin = NULL);
 public:
   // Also adjust cMenuSetup (menu.c) when adding parameters here!
   int __BeginData__;
   char OSDLanguage[I18N_MAX_LOCALE_LEN];
   char OSDSkin[MaxSkinName];
   char OSDTheme[MaxThemeName];
-#ifdef USE_WAREAGLEICON
-  int WarEagleIcons;
-#endif /* WAREAGLEICON */
 #ifdef USE_VALIDINPUT
   int ShowValidInput;
 #endif /* VALIDINPUT */
+#ifdef USE_WAREAGLEICON
+  int WarEagleIcons;
+#endif /* WAREAGLEICON */
   int PrimaryDVB;
   int ShowInfoOnChSwitch;
   int TimeoutRequChInfo;
@@ -305,12 +263,24 @@ public:
   int TimeSource;
   int TimeTransponder;
   int MarginStart, MarginStop;
+#ifdef USE_JUMPINGSECONDS
+  int JumpSeconds, JumpSecondsSlow, JumpSecondsRepeat;
+#endif /* JUMPINGSECONDS */
   int AudioLanguages[I18N_MAX_LANGUAGES + 1];
   int DisplaySubtitles;
+#ifdef USE_TTXTSUBS
+  int SupportTeletext;
+#endif /* TTXTSUBS */
   int SubtitleLanguages[I18N_MAX_LANGUAGES + 1];
   int SubtitleOffset;
   int SubtitleFgTransparency, SubtitleBgTransparency;
   int EPGLanguages[I18N_MAX_LANGUAGES + 1];
+#ifdef USE_DDEPGENTRY
+  int DoubleEpgTimeDelta;
+  int DoubleEpgAction;
+  int MixEpgAction;
+  int DisableVPS;
+#endif /* DDEPPGENTRY */
   int EPGScanTimeout;
   int EPGBugfixLevel;
   int EPGLinger;
@@ -320,58 +290,48 @@ public:
   int PrimaryLimit;
   int DefaultPriority, DefaultLifetime;
   int PausePriority, PauseLifetime;
+  int PauseKeyHandling;
   int UseSubtitle;
   int UseVps;
   int VpsMargin;
   int RecordingDirs;
+  int FoldersInTimerMenu;
+  int NumberKeysForChars;
   int VideoDisplayFormat;
   int VideoFormat;
   int UpdateChannels;
+#ifdef USE_CHANNELBIND
+  int ChannelBindingByRid;
+#endif /* CHANNELBIND */
   int UseDolbyDigital;
-#ifdef USE_DOLBYINREC
-  int UseDolbyInRecordings;
-#endif /* DOLBYINREC */
-#ifdef USE_DVBSETUP
-  int DolbyTransferFix;
-  int ChannelBlocker;
-  int ChannelBlockerMode;
-#endif /* DVBSETUP */
-#ifdef USE_SYNCEARLY
-  int UseSyncEarlyPatch;
-#endif /* SYNCEARLY */
   int ChannelInfoPos;
   int ChannelInfoTime;
+  double OSDLeftP, OSDTopP, OSDWidthP, OSDHeightP;
   int OSDLeft, OSDTop, OSDWidth, OSDHeight;
+  double OSDAspect;
   int OSDMessageTime;
   int UseSmallFont;
   int AntiAlias;
   char FontOsd[MAXFONTNAME];
   char FontSml[MAXFONTNAME];
   char FontFix[MAXFONTNAME];
+  double FontOsdSizeP;
+  double FontSmlSizeP;
+  double FontFixSizeP;
   int FontOsdSize;
   int FontSmlSize;
   int FontFixSize;
   int MaxVideoFileSize;
 #ifdef USE_HARDLINKCUTTER
   int MaxRecordingSize;
-#endif /* HARDLINKCUTTER */
-  int SplitEditedFiles;
-#ifdef USE_HARDLINKCUTTER
   int HardLinkCutter;
 #endif /* HARDLINKCUTTER */
-#ifdef USE_DELTIMESHIFTREC
+  int SplitEditedFiles;
   int DelTimeshiftRec;
-#endif /* DELTIMESHIFTREC */
   int MinEventTimeout, MinUserInactivity;
   time_t NextWakeupTime;
   int MultiSpeedMode;
   int ShowReplayMode;
-#ifdef USE_DDEPGENTRY
-  int DoubleEpgTimeDelta;
-  int DoubleEpgAction;
-  int MixEpgAction;
-  int DisableVPS;
-#endif /* DDEPGENTRY */
   int ResumeID;
 #ifdef USE_JUMPPLAY
   int JumpPlay;
@@ -379,13 +339,12 @@ public:
   int PauseLastMark;
   int ReloadMarks;
 #endif /* JUMPPLAY */
-#ifdef USE_SOURCECAPS
-  int SourceCaps[MAXDEVICES][MAXSOURCECAPS];
-  bool SourceCapsSet;
-#endif /* SOURCECAPS */
   int CurrentChannel;
   int CurrentVolume;
   int CurrentDolby;
+#ifdef USE_CHANNELPROVIDE
+  int LocalChannelProvide;
+#endif /* CHANNELPROVIDE */
   int InitialChannel;
   int InitialVolume;
 #ifdef USE_VOLCTRL
@@ -393,68 +352,41 @@ public:
   int LRChannelGroups;
   int LRForwardRewind;
 #endif /* VOLCTRL */
+  int ChannelsWrap;
   int EmergencyExit;
-#ifdef USE_NOEPG
-  int noEPGMode;
-#endif /* NOEPG */
 #ifdef USE_LIRCSETTINGS
   int LircRepeatDelay;
   int LircRepeatFreq;
   int LircRepeatTimeout;
 #endif /* LIRCSETTINGS */
-#ifdef USE_LIEMIKUUTIO
+#ifdef USE_LIEMIEXT
   int ShowRecDate, ShowRecTime, ShowRecLength, ShowProgressBar, MenuCmdPosition;
-  int JumpSeconds;
-  int JumpSecondsSlow;
-  int ShowTimerStop;
-  int MainMenuTitle;
-  char CustomMainMenuTitle[MaxTitleName];
-#endif /* LIEMIKUUTIO */
-#ifdef USE_SORTRECORDS
-  int RecordingsSortMode;
-  int RecordingsSortDirsFirst;
-#endif /* SORTRECORDS */
-#ifdef USE_CUTTERQUEUE
-  int CutterAutoDelete;
-#endif /* CUTTERQUEUE */
-#ifdef USE_LIVEBUFFER
-  int LiveBuffer;
-  int KeepPaused;
-  int LiveBufferSize;
-  int InRAM;
-  int MemBufSize;
-  int ExtendBuffer;
-#endif /* LIVEBUFFER */
-#ifdef USE_DVDARCHIVE
-  int DvdDisplayMode;
-  int DvdDisplayZeros;
-  int DvdTrayMode;
-  int DvdSpeedLimit;
-#endif /* DVDARCHIVE */
+#endif /* LIEMIEXT */
 #ifdef USE_LNBSHARE
   int VerboseLNBlog;
-  int CardUsesLNBnr[MAXDEVICES];
+  #define MAXDEVICES 16 // Since VDR 1.3.32 we can not #include "device.h" for MAXDEVICES anymore.
+                        // With this workaround a warning will be shown during compilation if
+                        // MAXDEVICES changes in device.h.
+  int CardUsesLnbNr[MAXDEVICES];
 #endif /* LNBSHARE */
+#ifdef USE_NOEPG
+  int noEPGMode;
+#endif /* NOEPG */
 #ifdef USE_DVLVIDPREFER
   int UseVidPrefer;  // 0 = VDR's default, 1 = use
   int nVidPrefer;
   int VidPreferPrio[DVLVIDPREFER_MAX];
   int VidPreferSize[DVLVIDPREFER_MAX];
 #endif /* DVLVIDPREFER */
-#ifdef USE_DVLFRIENDLYFNAMES
-  int UseFriendlyFNames;
-#endif /* DVLFRIENDLYFNAMES */
+
   int __EndData__;
-#ifdef USE_DVBSETUP
-  char *ChannelBlockerList;
-#endif /* DVBSETUP */
 #ifdef USE_NOEPG
   char *noEPGList; // pointer not to be flat-copied
 #endif /* NOEPG */
   cSetup(void);
-#if defined (USE_DVBSETUP) || defined (USE_NOEPG)
+#ifdef USE_NOEPG
   ~cSetup();
-#endif /* DVBSETUP + NOEPG */
+#endif /* NOEPG */
   cSetup& operator= (const cSetup &s);
   bool Load(const char *FileName);
   bool Save(void);
@@ -462,8 +394,4 @@ public:
 
 extern cSetup Setup;
 
-#ifdef USE_CMDRECCMDI18N
-bool LoadCommandsI18n(cCommands & cmds, const char *FileName = NULL, bool AllowComments = false, bool MustExist = false);
-#endif /* CMDRECCMDI18N */
-
 #endif //__CONFIG_H
