@@ -8,7 +8,7 @@
  * Robert Schneider <Robert.Schneider@web.de> and Rolf Hakenes <hakenes@hippomi.de>.
  * Adapted to 'libsi' for VDR 1.3.0 by Marcel Wiesweg <marcel.wiesweg@gmx.de>.
  *
- * $Id: eit.c 1.126.1.1 2008/04/13 11:46:38 kls Exp $
+ * $Id: eit.c 2.12 2010/05/14 14:08:35 kls Exp $
  */
 
 #include "eit.h"
@@ -17,9 +17,7 @@
 #include "libsi/section.h"
 #include "libsi/descriptor.h"
 
-#ifdef USE_SETTIME
-extern char *SetTime;
-#endif /* SETTIME */
+#define VALID_TIME (31536000 * 2) // two years
 
 // --- cEIT ------------------------------------------------------------------
 
@@ -34,18 +32,14 @@ private:
 
 #ifdef USE_NOEPG
 bool cEIT::allowedEPG(tChannelID kanalID) {
-  bool rc;
+  bool found = false;
+  bool rc = false;
 
-  if (Setup.noEPGMode == 1) {
-     rc = false;
-     if (strstr(::Setup.noEPGList,kanalID.ToString()) != NULL)
-        rc = true;
-     }
-  else {
+  if (::Setup.noEPGList && strstr(::Setup.noEPGList, kanalID.ToString()))
+     found = true;
+
+  if ((Setup.noEPGMode == 1 && found) || (Setup.noEPGMode == 0 && !found))
      rc = true;
-     if (strstr(::Setup.noEPGList,kanalID.ToString()) != NULL)
-        rc = false;
-     }
 
   return rc;
 }
@@ -77,20 +71,28 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data, bo
   bool HasExternalData = false;
   time_t SegmentStart = 0;
   time_t SegmentEnd = 0;
+  time_t Now = time(NULL);
+  struct tm tm_r;
+  struct tm t = *localtime_r(&Now, &tm_r); // this initializes the time zone in 't'
+
+  if (Now < VALID_TIME)
+     return; // we need the current time for handling PDC descriptors
 
   SI::EIT::Event SiEitEvent;
   for (SI::Loop::Iterator it; eventLoop.getNext(SiEitEvent, it); ) {
       bool ExternalData = false;
+      time_t StartTime = SiEitEvent.getStartTime();
+      int Duration = SiEitEvent.getDuration();
       // Drop bogus events - but keep NVOD reference events, where all bits of the start time field are set to 1, resulting in a negative number.
-      if (SiEitEvent.getStartTime() == 0 || SiEitEvent.getStartTime() > 0 && SiEitEvent.getDuration() == 0)
+      if (StartTime == 0 || StartTime > 0 && Duration == 0)
          continue;
       Empty = false;
       if (!SegmentStart)
-         SegmentStart = SiEitEvent.getStartTime();
-      SegmentEnd = SiEitEvent.getStartTime() + SiEitEvent.getDuration();
+         SegmentStart = StartTime;
+      SegmentEnd = StartTime + Duration;
       cEvent *newEvent = NULL;
       cEvent *rEvent = NULL;
-      cEvent *pEvent = (cEvent *)pSchedule->GetEvent(SiEitEvent.getEventId(), SiEitEvent.getStartTime());
+      cEvent *pEvent = (cEvent *)pSchedule->GetEvent(SiEitEvent.getEventId(), StartTime);
       if (!pEvent) {
          if (OnlyRunningStatus)
             continue;
@@ -105,11 +107,14 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data, bo
          pEvent->SetSeen();
          // If the existing event has a zero table ID it was defined externally and shall
          // not be overwritten.
-         if (pEvent->TableID() == 0x00) {
+         uchar TableID = pEvent->TableID();
+         if (TableID == 0x00) {
+            if (pEvent->Version() == getVersionNumber())
 #ifdef USE_DDEPGENTRY
-            if (pEvent->Version() == getVersionNumber()) {
-               if (Setup.MixEpgAction == 0)
-                  continue;
+            {
+               if(Setup.MixEpgAction == 0)
+                   continue;
+
                //printf("in");
                //printf("%s", pEvent->GetTimeString());
                // to use the info of the original epg, update the extern one,
@@ -121,64 +126,76 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data, bo
                //SI::ShortEventDescriptor *sed = NULL;
                //SI::TimeShiftedEventDescriptor *tsed = NULL;
                //cLinkChannels *LinkChannels = NULL;
-               for (SI::Loop::Iterator it2; (d = SiEitEvent.eventDescriptors.getNext(it2));) {
-                  if (d->getDescriptorTag() == SI::ShortEventDescriptorTag) {
+               for (SI::Loop::Iterator it2; (d = SiEitEvent.eventDescriptors.getNext(it2));)
+               {
+                  if(d->getDescriptorTag() == SI::ShortEventDescriptorTag)
+                  {
                      int LanguagePreferenceShort = -1;
                      SI::ShortEventDescriptor *sed = (SI::ShortEventDescriptor *)d;
-                     if (I18nIsPreferredLanguage(Setup.EPGLanguages, sed->languageCode, LanguagePreferenceShort) || !ShortEventDescriptor) {
+                     if (I18nIsPreferredLanguage(Setup.EPGLanguages, sed->languageCode, LanguagePreferenceShort) || !ShortEventDescriptor)
+                     {
                         delete ShortEventDescriptor;
                         ShortEventDescriptor = sed;
                         d = NULL; // so that it is not deleted
-                        }
                      }
-                  else if (d->getDescriptorTag() == SI::ExtendedEventDescriptorTag) {
+                  }
+                  else if(d->getDescriptorTag() == SI::ExtendedEventDescriptorTag)
+                  {
                      int LanguagePreferenceExt = -1;
                      bool UseExtendedEventDescriptor = false;
                      SI::ExtendedEventDescriptor *eed = (SI::ExtendedEventDescriptor *)d;
-                     if (I18nIsPreferredLanguage(Setup.EPGLanguages, eed->languageCode, LanguagePreferenceExt) || !ExtendedEventDescriptors) {
+                     if (I18nIsPreferredLanguage(Setup.EPGLanguages, eed->languageCode, LanguagePreferenceExt) || !ExtendedEventDescriptors)
+                     {
                         delete ExtendedEventDescriptors;
                         ExtendedEventDescriptors = new SI::ExtendedEventDescriptors;
                         UseExtendedEventDescriptor = true;
-                        }
-                     if (UseExtendedEventDescriptor) {
+                     }
+                     if (UseExtendedEventDescriptor)
+                     {
                         ExtendedEventDescriptors->Add(eed);
                         d = NULL; // so that it is not deleted
-                        }
+                     }
                      if (eed->getDescriptorNumber() == eed->getLastDescriptorNumber())
                         UseExtendedEventDescriptor = false;
-                     }
-                     delete d;
                   }
-                  if (pEvent) {
-                     if (ShortEventDescriptor) {
-                        char buffer[256];
-                        if (ShortEventDescriptor->text.getText(buffer, sizeof(buffer)) && pEvent->ShortText() && (strlen(ShortEventDescriptor->text.getText(buffer, sizeof(buffer))) > strlen(pEvent->ShortText()))) {
-                           pEvent->SetShortText(ShortEventDescriptor->text.getText(buffer, sizeof(buffer)));
-                           pEvent->FixEpgBugs();
-                           }
-                        }
-                     if (ExtendedEventDescriptors) {
-                        char buffer[ExtendedEventDescriptors->getMaximumTextLength(": ") + 1];
-                        //pEvent->SetDescription(ExtendedEventDescriptors->getText(buffer, sizeof(buffer), ": "));
-                        if (ExtendedEventDescriptors->getText(buffer, sizeof(buffer), ": ") && pEvent->Description() && (strlen(ExtendedEventDescriptors->getText(buffer, sizeof(buffer), ": ")) > strlen(pEvent->Description()))) {
-                           pEvent->SetDescription(ExtendedEventDescriptors->getText(buffer, sizeof(buffer), ": "));
-                           pEvent->FixEpgBugs();
-                           }
-                        }
-                     }
-                  delete ExtendedEventDescriptors;
-                  delete ShortEventDescriptor;
-                  continue;
+                  delete d;
                }
+               if(pEvent)
+               {
+
+                  if(ShortEventDescriptor)
+                  {
+                     char buffer[256];
+                     if(ShortEventDescriptor->text.getText(buffer, sizeof(buffer)) && pEvent->ShortText() && (strlen(ShortEventDescriptor->text.getText(buffer, sizeof(buffer))) > strlen(pEvent->ShortText())))
+                     {
+                        pEvent->SetShortText(ShortEventDescriptor->text.getText(buffer, sizeof(buffer)));
+                        pEvent->FixEpgBugs();
+                     }
+                  }
+                  if(ExtendedEventDescriptors)
+                  {
+                     char buffer[ExtendedEventDescriptors->getMaximumTextLength(": ") + 1];
+                     //pEvent->SetDescription(ExtendedEventDescriptors->getText(buffer, sizeof(buffer), ": "));
+
+                     if(ExtendedEventDescriptors->getText(buffer, sizeof(buffer), ": ") && pEvent->Description() && (strlen(ExtendedEventDescriptors->getText(buffer, sizeof(buffer), ": ")) > strlen(pEvent->Description())))
+                     {
+                        pEvent->SetDescription(ExtendedEventDescriptors->getText(buffer, sizeof(buffer), ": "));
+                        pEvent->FixEpgBugs();
+                     }
+                  }
+               }
+               delete ExtendedEventDescriptors;
+               delete ShortEventDescriptor;
+               continue;
+            }
 #else
-            if (pEvent->Version() == getVersionNumber())
                continue;
 #endif /* DDEPGENTRY */
             HasExternalData = ExternalData = true;
             }
          // If the new event has a higher table ID, let's skip it.
          // The lower the table ID, the more "current" the information.
-         else if (Tid > pEvent->TableID())
+         else if (Tid > TableID)
             continue;
          // If the new event comes from the same table and has the same version number
          // as the existing one, let's skip it to avoid unnecessary work.
@@ -186,14 +203,14 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data, bo
          // the actual Premiere transponder and the Sat.1/Pro7 transponder), but use different version numbers on
          // each of them :-( So if one DVB card is tuned to the Premiere transponder, while an other one is tuned
          // to the Sat.1/Pro7 transponder, events will keep toggling because of the bogus version numbers.
-         else if (Tid == pEvent->TableID() && pEvent->Version() == getVersionNumber())
+         else if (Tid == TableID && pEvent->Version() == getVersionNumber())
             continue;
          }
       if (!ExternalData) {
          pEvent->SetEventID(SiEitEvent.getEventId()); // unfortunately some stations use different event ids for the same event in different tables :-(
          pEvent->SetTableID(Tid);
-         pEvent->SetStartTime(SiEitEvent.getStartTime());
-         pEvent->SetDuration(SiEitEvent.getDuration());
+         pEvent->SetStartTime(StartTime);
+         pEvent->SetDuration(Duration);
          }
       if (newEvent)
          pSchedule->AddEvent(newEvent);
@@ -247,53 +264,43 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data, bo
                     }
                  }
                  break;
-            case SI::ContentDescriptorTag:
-#ifdef USE_PARENTALRATING
-                 {
-                 int NumContents = 0;
-                 uchar Contents[MAXEVCONTENTS + 1] = { 0 };
+            case SI::ContentDescriptorTag: {
                  SI::ContentDescriptor *cd = (SI::ContentDescriptor *)d;
                  SI::ContentDescriptor::Nibble Nibble;
+                 int NumContents = 0;
+                 uchar Contents[MaxEventContents] = { 0 };
                  for (SI::Loop::Iterator it3; cd->nibbleLoop.getNext(Nibble, it3); ) {
-                     if (NumContents < MAXEVCONTENTS) {
+                     if (NumContents < MaxEventContents) {
                         Contents[NumContents] = ((Nibble.getContentNibbleLevel1() & 0xF) << 4) | (Nibble.getContentNibbleLevel2() & 0xF);
                         NumContents++;
                         }
                      }
                  pEvent->SetContents(Contents);
                  }
-#endif /* PARENTALRATING */
                  break;
-            case SI::ParentalRatingDescriptorTag:
-#ifdef USE_PARENTALRATING
-                 {
+            case SI::ParentalRatingDescriptorTag: {
                  int LanguagePreferenceRating = -1;
                  SI::ParentalRatingDescriptor *prd = (SI::ParentalRatingDescriptor *)d;
                  SI::ParentalRatingDescriptor::Rating Rating;
                  for (SI::Loop::Iterator it3; prd->ratingLoop.getNext(Rating, it3); ) {
                      if (I18nIsPreferredLanguage(Setup.EPGLanguages, Rating.languageCode, LanguagePreferenceRating)) {
-                        int rate = (Rating.getRating() & 0xFF);
-                        switch (rate) {
-                          case 0x01 ... 0x0F: // minimum age = rating + 3 years
-                               rate += 3;
-                               break;
-                          case 0:             // undefined
-                          case 0x10 ... 0xFF: // defined by the broadcaster
-                          default:
-                               rate = 0; 
-                               break;
+                        int ParentalRating = (Rating.getRating() & 0xFF);
+                        switch (ParentalRating) {
+                          // values defined by the DVB standard (minimum age = rating + 3 years):
+                          case 0x01 ... 0x0F: ParentalRating += 3; break;
+                          // values defined by broadcaster CSAT (now why didn't they just use 0x07, 0x09 and 0x0D?):
+                          case 0x11:          ParentalRating = 10; break;
+                          case 0x12:          ParentalRating = 12; break;
+                          case 0x13:          ParentalRating = 16; break;
+                          default:            ParentalRating = 0;
                           }
-                        pEvent->SetParentalRating(rate);
+                        pEvent->SetParentalRating(ParentalRating);
                         }
                      }
                  }
-#endif /* PARENTALRATING */
                  break;
             case SI::PDCDescriptorTag: {
                  SI::PDCDescriptor *pd = (SI::PDCDescriptor *)d;
-                 time_t now = time(NULL);
-                 struct tm tm_r;
-                 struct tm t = *localtime_r(&now, &tm_r); // this initializes the time zone in 't'
                  t.tm_isdst = -1; // makes sure mktime() will determine the correct DST setting
                  int month = t.tm_mon;
                  t.tm_mon = pd->getMonth() - 1;
@@ -326,8 +333,7 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data, bo
                  SI::LinkageDescriptor *ld = (SI::LinkageDescriptor *)d;
                  tChannelID linkID(Source, ld->getOriginalNetworkId(), ld->getTransportStreamId(), ld->getServiceId());
                  if (ld->getLinkageType() == 0xB0) { // Premiere World
-                    time_t now = time(NULL);
-                    bool hit = SiEitEvent.getStartTime() <= now && now < SiEitEvent.getStartTime() + SiEitEvent.getDuration();
+                    bool hit = StartTime <= Now && Now < StartTime + Duration;
                     if (hit) {
                        char linkName[ld->privateData.getLength() + 1];
                        strn0cpy(linkName, (const char *)ld->privateData.getData(), sizeof(linkName));
@@ -362,7 +368,7 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data, bo
                  SI::ComponentDescriptor *cd = (SI::ComponentDescriptor *)d;
                  uchar Stream = cd->getStreamContent();
                  uchar Type = cd->getComponentType();
-                 if (1 <= Stream && Stream <= 3 && Type != 0) { // 1=video, 2=audio, 3=subtitles
+                 if (1 <= Stream && Stream <= 6 && Type != 0) { // 1=MPEG2-video, 2=MPEG1-audio, 3=subtitles, 4=AC3-audio, 5=H.264-video, 6=HEAAC-audio
                     if (!Components)
                        Components = new cComponents;
                     char buffer[Utf8BufSize(256)];
@@ -402,54 +408,74 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data, bo
       if (LinkChannels)
          channel->SetLinkChannels(LinkChannels);
       Modified = true;
+
 #ifdef USE_DDEPGENTRY
       //to avoid double epg-entrys from ext and int epg sources :EW
-      if (pEvent && pEvent->TableID() != 0x00) {
+      if (pEvent && pEvent->TableID() != 0x00)
+         {
          cEvent *pPreviousEvent = (cEvent *)pSchedule->GetPreviousEvent(pEvent);
-         if (pPreviousEvent) {
-            if (Setup.DoubleEpgAction == 0) {
+
+         if (pPreviousEvent)
+            {
+            if(Setup.DoubleEpgAction == 0)
+               {
                pPreviousEvent->SetStartTime(pEvent->StartTime());
                pPreviousEvent->SetDuration(pEvent->Duration());
-               if (Setup.DisableVPS == 0) {
-                  if (channel)
+
+               if(Setup.DisableVPS == 0)
+                  {
+                  if(channel)
                      pPreviousEvent->SetRunningStatus(pEvent->RunningStatus(), channel);
                   else
                      pPreviousEvent->SetRunningStatus(pEvent->RunningStatus());
                   }
+
                // to use the info of the original epg, update the extern one,
                // if it has less info
                char buffer_short_intern[256];
                char buffer_short_extern[256];
                int len_short_intern = 0;
                int len_short_extern = 0;
+
                if (pEvent->ShortText())
-                  len_short_intern = snprintf (buffer_short_intern, sizeof(buffer_short_intern)-1, "%s", pEvent->ShortText());
+                  len_short_intern = snprintf (buffer_short_intern, sizeof(buffer_short_intern), "%s", pEvent->ShortText());
+
                if (pPreviousEvent->ShortText())
-                  len_short_extern = snprintf (buffer_short_extern, sizeof(buffer_short_extern)-1, "%s", pPreviousEvent->ShortText());
-               if (len_short_intern > 0) {
-                  if (len_short_extern < 1)
+                  len_short_extern = snprintf (buffer_short_extern, sizeof(buffer_short_extern), "%s",pPreviousEvent->ShortText());
+
+               if(len_short_intern > 0)
+                  {
+                  if(len_short_extern < 1)
                      pPreviousEvent->SetShortText(buffer_short_intern);
                   else if (len_short_intern > len_short_extern)
                      pPreviousEvent->SetShortText(buffer_short_intern);
                   }
-               if (pEvent->Description()) {
+
+               if(pEvent->Description())
+                  {
                   char buffer_title_intern[4096];
                   char buffer_title_extern[4096];
                   int len_title_intern = 0;
                   int len_title_extern = 0;
+
                   if (pEvent->Description())
-                     len_title_intern = snprintf (buffer_title_intern, sizeof(buffer_title_intern)-1, "%s", pEvent->Description());
+                     len_title_intern = snprintf (buffer_title_intern, sizeof(buffer_title_intern), "%s", pEvent->Description());
+
                   if (pPreviousEvent->Description())
-                     len_title_extern = snprintf (buffer_title_extern, sizeof(buffer_title_extern)-1, "%s", pPreviousEvent->Description());
-                  if (len_title_intern > 0) {
-                     if (len_title_extern < 1)
+                     len_title_extern = snprintf (buffer_title_extern, sizeof(buffer_title_extern), "%s", pPreviousEvent->Description());
+
+                  if(len_title_intern > 0)
+                     {
+                     if(len_title_extern < 1)
                         pPreviousEvent->SetDescription(buffer_title_intern);
                      else if (len_title_intern > len_title_extern)
                         pPreviousEvent->SetDescription(buffer_title_intern);
                      }
                   }
-               if (pPreviousEvent->Vps() == 0 && pEvent->Vps() != 0)
+
+               if(pPreviousEvent->Vps() == 0 && pEvent->Vps() != 0)
                   pPreviousEvent->SetVps(pEvent->Vps());
+
                pSchedule->DelEvent(pEvent);
                pPreviousEvent->FixEpgBugs();
                }
@@ -459,11 +485,12 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data, bo
          }
 #endif /* DDEPGENTRY */
       }
-  if (Empty && Tid == 0x4E && getSectionNumber() == 0)
-     // ETR 211: an empty entry in section 0 of table 0x4E means there is currently no event running
-     pSchedule->ClrRunningStatus(channel);
-  if (Tid == 0x4E)
+  if (Tid == 0x4E) {
+     if (Empty && getSectionNumber() == 0)
+        // ETR 211: an empty entry in section 0 of table 0x4E means there is currently no event running
+        pSchedule->ClrRunningStatus(channel);
      pSchedule->SetPresentSeen();
+     }
   if (OnlyRunningStatus)
      return;
   if (Modified) {
@@ -495,26 +522,10 @@ cTDT::cTDT(const u_char *Data)
   time_t sattim = getTime();
   time_t loctim = time(NULL);
 
-#ifdef USE_SETTIME
-  char timestr[20];
-  struct tm *ptm;
-  struct tm tm_r;
-  ptm = localtime_r(&sattim, &tm_r);
-#endif /* SETTIME */
-
   int diff = abs(sattim - loctim);
   if (diff > 2) {
      mutex.Lock();
      if (abs(diff - lastDiff) < 3) {
-#ifdef USE_SETTIME
-        if (SetTime) {
-           strftime(timestr, 20, "%m%d%H%M%Y.%S", ptm);
-           cString cmd = cString::sprintf("%s %s %ld", SetTime, timestr, sattim);
-           dsyslog("Executing: %s", *cmd);
-           SystemExec(cmd);
-           }
-        else
-#endif /* SETTIME */
         if (stime(&sattim) == 0)
            isyslog("system time changed from %s (%ld) to %s (%ld)", *TimeToString(loctim), loctim, *TimeToString(sattim), sattim);
         else
@@ -527,31 +538,45 @@ cTDT::cTDT(const u_char *Data)
 
 // --- cEitFilter ------------------------------------------------------------
 
+time_t cEitFilter::disableUntil = 0;
+
 cEitFilter::cEitFilter(void)
 {
-  Set(0x12, 0x4E, 0xFE);  // event info, actual(0x4E)/other(0x4F) TS, present/following
-  Set(0x12, 0x50, 0xF0);  // event info, actual TS, schedule(0x50)/schedule for future days(0x5X)
-  Set(0x12, 0x60, 0xF0);  // event info, other  TS, schedule(0x60)/schedule for future days(0x6X)
-  Set(0x14, 0x70);        // TDT
+  Set(0x12, 0x40, 0xC0);  // event info now&next actual/other TS (0x4E/0x4F), future actual/other TS (0x5X/0x6X)
+  if (Setup.SetSystemTime && Setup.TimeTransponder)
+     Set(0x14, 0x70);     // TDT
+}
+
+void cEitFilter::SetDisableUntil(time_t Time)
+{
+  disableUntil = Time;
 }
 
 void cEitFilter::Process(u_short Pid, u_char Tid, const u_char *Data, int Length)
 {
+  if (disableUntil) {
+     if (time(NULL) > disableUntil)
+        disableUntil = 0;
+     else
+        return;
+     }
   switch (Pid) {
     case 0x12: {
-         cSchedulesLock SchedulesLock(true, 10);
-         cSchedules *Schedules = (cSchedules *)cSchedules::Schedules(SchedulesLock);
-         if (Schedules)
-            cEIT EIT(Schedules, Source(), Tid, Data);
-         else {
-            // If we don't get a write lock, let's at least get a read lock, so
-            // that we can set the running status and 'seen' timestamp (well, actually
-            // with a read lock we shouldn't be doing that, but it's only integers that
-            // get changed, so it should be ok)
-            cSchedulesLock SchedulesLock;
+         if (Tid >= 0x4E && Tid <= 0x6F) {
+            cSchedulesLock SchedulesLock(true, 10);
             cSchedules *Schedules = (cSchedules *)cSchedules::Schedules(SchedulesLock);
             if (Schedules)
-               cEIT EIT(Schedules, Source(), Tid, Data, true);
+               cEIT EIT(Schedules, Source(), Tid, Data);
+            else {
+               // If we don't get a write lock, let's at least get a read lock, so
+               // that we can set the running status and 'seen' timestamp (well, actually
+               // with a read lock we shouldn't be doing that, but it's only integers that
+               // get changed, so it should be ok)
+               cSchedulesLock SchedulesLock;
+               cSchedules *Schedules = (cSchedules *)cSchedules::Schedules(SchedulesLock);
+               if (Schedules)
+                  cEIT EIT(Schedules, Source(), Tid, Data, true);
+               }
             }
          }
          break;
@@ -560,5 +585,6 @@ void cEitFilter::Process(u_short Pid, u_char Tid, const u_char *Data, int Length
             cTDT TDT(Data);
          }
          break;
+    default: ;
     }
 }
