@@ -7,7 +7,7 @@
  * Original version (as used in VDR before 1.3.0) written by
  * Robert Schneider <Robert.Schneider@web.de> and Rolf Hakenes <hakenes@hippomi.de>.
  *
- * $Id: epg.c 1.83 2008/02/16 16:09:12 kls Exp $
+ * $Id: epg.c 2.6 2010/02/28 14:24:55 kls Exp $
  */
 
 #include "epg.h"
@@ -88,8 +88,10 @@ void cComponents::SetComponent(int Index, uchar Stream, uchar Type, const char *
 tComponent *cComponents::GetComponent(int Index, uchar Stream, uchar Type)
 {
   for (int i = 0; i < numComponents; i++) {
-      // In case of an audio stream the 'type' check actually just distinguishes between "normal" and "Dolby Digital":
-      if (components[i].stream == Stream && (Stream != 2 || (components[i].type < 5) == (Type < 5))) {
+      if (components[i].stream == Stream && (
+          Type == 0 || // don't care about the actual Type
+          Stream == 2 && (components[i].type < 5) == (Type < 5) // fallback "Dolby" component according to the "Premiere pseudo standard"
+         )) {
          if (!Index--)
             return &components[i];
          }
@@ -110,13 +112,10 @@ cEvent::cEvent(tEventID EventID)
   shortText = NULL;
   description = NULL;
   components = NULL;
+  memset(contents, 0, sizeof(contents));
+  parentalRating = 0;
   startTime = 0;
   duration = 0;
-#ifdef USE_PARENTALRATING
-  for (int i = 0; i < MAXEVCONTENTS; i++)
-      contents[i] = 0;
-  parentalRating = 0;
-#endif /* PARENTALRATING */
   vps = 0;
   SetSeen();
 }
@@ -189,6 +188,17 @@ void cEvent::SetComponents(cComponents *Components)
   components = Components;
 }
 
+void cEvent::SetContents(uchar *Contents)
+{
+  for (int i = 0; i < MaxEventContents; i++)
+      contents[i] = Contents[i];
+}
+
+void cEvent::SetParentalRating(int ParentalRating)
+{
+  parentalRating = ParentalRating;
+}
+
 void cEvent::SetStartTime(time_t StartTime)
 {
   if (startTime != StartTime) {
@@ -205,19 +215,6 @@ void cEvent::SetDuration(int Duration)
   duration = Duration;
 }
 
-#ifdef USE_PARENTALRATING
-void cEvent::SetContents(uchar *Contents)
-{
-  for (int i = 0; i < MAXEVCONTENTS; i++)
-      contents[i] = Contents[i];
-}
-
-void cEvent::SetParentalRating(uchar ParentalRating)
-{
-  parentalRating = ParentalRating;
-}
-#endif /* PARENTALRATING */
-
 void cEvent::SetVps(time_t Vps)
 {
   vps = Vps;
@@ -250,6 +247,155 @@ bool cEvent::IsRunning(bool OrAboutToStart) const
   return runningStatus >= (OrAboutToStart ? SI::RunningStatusStartsInAFewSeconds : SI::RunningStatusPausing);
 }
 
+const char *cEvent::ContentToString(uchar Content)
+{
+  switch (Content & 0xF0) {
+    case ecgMovieDrama:
+         switch (Content & 0x0F) {
+           default:
+           case 0x00: return tr("Content$Movie/Drama");
+           case 0x01: return tr("Content$Detective/Thriller");
+           case 0x02: return tr("Content$Adventure/Western/War");
+           case 0x03: return tr("Content$Science Fiction/Fantasy/Horror");
+           case 0x04: return tr("Content$Comedy");
+           case 0x05: return tr("Content$Soap/Melodrama/Folkloric");
+           case 0x06: return tr("Content$Romance");
+           case 0x07: return tr("Content$Serious/Classical/Religious/Historical Movie/Drama");
+           case 0x08: return tr("Content$Adult Movie/Drama");
+           }
+         break;
+    case ecgNewsCurrentAffairs:
+         switch (Content & 0x0F) {
+           default:
+           case 0x00: return tr("Content$News/Current Affairs");
+           case 0x01: return tr("Content$News/Weather Report");
+           case 0x02: return tr("Content$News Magazine");
+           case 0x03: return tr("Content$Documentary");
+           case 0x04: return tr("Content$Discussion/Inverview/Debate");
+           }
+         break;
+    case ecgShow:
+         switch (Content & 0x0F) {
+           default:
+           case 0x00: return tr("Content$Show/Game Show");
+           case 0x01: return tr("Content$Game Show/Quiz/Contest");
+           case 0x02: return tr("Content$Variety Show");
+           case 0x03: return tr("Content$Talk Show");
+           }
+         break;
+    case ecgSports:
+         switch (Content & 0x0F) {
+           default:
+           case 0x00: return tr("Content$Sports");
+           case 0x01: return tr("Content$Special Event");
+           case 0x02: return tr("Content$Sport Magazine");
+           case 0x03: return tr("Content$Football/Soccer");
+           case 0x04: return tr("Content$Tennis/Squash");
+           case 0x05: return tr("Content$Team Sports");
+           case 0x06: return tr("Content$Athletics");
+           case 0x07: return tr("Content$Motor Sport");
+           case 0x08: return tr("Content$Water Sport");
+           case 0x09: return tr("Content$Winter Sports");
+           case 0x0A: return tr("Content$Equestrian");
+           case 0x0B: return tr("Content$Martial Sports");
+           }
+         break;
+    case ecgChildrenYouth:
+         switch (Content & 0x0F) {
+           default:
+           case 0x00: return tr("Content$Children's/Youth Programme");
+           case 0x01: return tr("Content$Pre-school Children's Programme");
+           case 0x02: return tr("Content$Entertainment Programme for 6 to 14");
+           case 0x03: return tr("Content$Entertainment Programme for 10 to 16");
+           case 0x04: return tr("Content$Informational/Educational/School Programme");
+           case 0x05: return tr("Content$Cartoons/Puppets");
+           }
+         break;
+    case ecgMusicBalletDance:
+         switch (Content & 0x0F) {
+           default:
+           case 0x00: return tr("Content$Music/Ballet/Dance");
+           case 0x01: return tr("Content$Rock/Pop");
+           case 0x02: return tr("Content$Serious/Classical Music");
+           case 0x03: return tr("Content$Folk/Tradional Music");
+           case 0x04: return tr("Content$Jazz");
+           case 0x05: return tr("Content$Musical/Opera");
+           case 0x06: return tr("Content$Ballet");
+           }
+         break;
+    case ecgArtsCulture:
+         switch (Content & 0x0F) {
+           default:
+           case 0x00: return tr("Content$Arts/Culture");
+           case 0x01: return tr("Content$Performing Arts");
+           case 0x02: return tr("Content$Fine Arts");
+           case 0x03: return tr("Content$Religion");
+           case 0x04: return tr("Content$Popular Culture/Traditional Arts");
+           case 0x05: return tr("Content$Literature");
+           case 0x06: return tr("Content$Film/Cinema");
+           case 0x07: return tr("Content$Experimental Film/Video");
+           case 0x08: return tr("Content$Broadcasting/Press");
+           case 0x09: return tr("Content$New Media");
+           case 0x0A: return tr("Content$Arts/Culture Magazine");
+           case 0x0B: return tr("Content$Fashion");
+           }
+         break;
+    case ecgSocialPoliticalEconomics:
+         switch (Content & 0x0F) {
+           default:
+           case 0x00: return tr("Content$Social/Political/Economics");
+           case 0x01: return tr("Content$Magazine/Report/Documentary");
+           case 0x02: return tr("Content$Economics/Social Advisory");
+           case 0x03: return tr("Content$Remarkable People");
+           }
+         break;
+    case ecgEducationalScience:
+         switch (Content & 0x0F) {
+           default:
+           case 0x00: return tr("Content$Education/Science/Factual");
+           case 0x01: return tr("Content$Nature/Animals/Environment");
+           case 0x02: return tr("Content$Technology/Natural Sciences");
+           case 0x03: return tr("Content$Medicine/Physiology/Psychology");
+           case 0x04: return tr("Content$Foreign Countries/Expeditions");
+           case 0x05: return tr("Content$Social/Spiritual Sciences");
+           case 0x06: return tr("Content$Further Education");
+           case 0x07: return tr("Content$Languages");
+           }
+         break;
+    case ecgLeisureHobbies:
+         switch (Content & 0x0F) {
+           default:
+           case 0x00: return tr("Content$Leisure/Hobbies");
+           case 0x01: return tr("Content$Tourism/Travel");
+           case 0x02: return tr("Content$Handicraft");
+           case 0x03: return tr("Content$Motoring");
+           case 0x04: return tr("Content$Fitness & Health");
+           case 0x05: return tr("Content$Cooking");
+           case 0x06: return tr("Content$Advertisement/Shopping");
+           case 0x07: return tr("Content$Gardening");
+           }
+         break;
+    case ecgSpecial:
+         switch (Content & 0x0F) {
+           case 0x00: return tr("Content$Original Language");
+           case 0x01: return tr("Content$Black & White");
+           case 0x02: return tr("Content$Unpublished");
+           case 0x03: return tr("Content$Live Broadcast");
+           default: ;
+           }
+         break;
+    default: ;
+    }
+  return "";
+}
+
+cString cEvent::GetParentalRatingString(void) const
+{
+  if (parentalRating)
+     return cString::sprintf(tr("ParentalRating$from %d"), parentalRating);
+  return NULL;
+}
+
 cString cEvent::GetDateString(void) const
 {
   return DateString(startTime);
@@ -273,340 +419,6 @@ cString cEvent::GetVpsString(void) const
   return buf;
 }
 
-#ifdef USE_PARENTALRATING
-cString cEvent::GetContentsString(int i) const
-{
-  cString str("");
-  switch (contents[i] & 0xF0) {
-    case EVCONTENTMASK_MOVIEDRAMA:
-         switch (contents[i] & 0x0F) {
-           default:
-           case 0x00:
-                str = cString(tr("Content$Movie/Drama"));
-                break;
-           case 0x01:
-                str = cString(tr("Content$Detective/Thriller"));
-                break;
-           case 0x02:
-                str = cString(tr("Content$Adventure/Western/War"));
-                break;
-           case 0x03:
-                str = cString(tr("Content$Science Fiction/Fantasy/Horror"));
-                break;
-           case 0x04:
-                str = cString(tr("Content$Comedy"));
-                break;
-           case 0x05:
-                str = cString(tr("Content$Soap/Melodrama/Folkloric"));
-                break;
-           case 0x06:
-                str = cString(tr("Content$Romance"));
-                break;
-           case 0x07:
-                str = cString(tr("Content$Serious/Classical/Religious/Historical Movie/Drama"));
-                break;
-           case 0x08:
-                str = cString(tr("Content$Adult Movie/Drama"));
-                break;
-            }
-         break;
-
-    case EVCONTENTMASK_NEWSCURRENTAFFAIRS:
-         switch (contents[i] & 0x0F) {
-           default:
-           case 0x00:
-                str = cString(tr("Content$News/Current Affairs"));
-                break;
-           case 0x01:
-                str = cString(tr("Content$News/Weather Report"));
-                break;
-           case 0x02:
-                str = cString(tr("Content$News Magazine"));
-                break;
-           case 0x03:
-                str = cString(tr("Content$Documentary"));
-                break;
-           case 0x04:
-                str = cString(tr("Content$Discussion/Inverview/Debate"));
-                break;
-           }
-         break;
-
-    case EVCONTENTMASK_SHOW:
-         switch (contents[i] & 0x0F) {
-           default:
-           case 0x00:
-                str = cString(tr("Content$Show/Game Show"));
-                break;
-           case 0x01:
-                str = cString(tr("Content$Game Show/Quiz/Contest"));
-                break;
-           case 0x02:
-                str = cString(tr("Content$Variety Show"));
-                break;
-           case 0x03:
-                str = cString(tr("Content$Talk Show"));
-                break;
-           }
-         break;
-
-    case EVCONTENTMASK_SPORTS:
-         switch (contents[i] & 0x0F) {
-           default:
-           case 0x00:
-                str = cString(tr("Content$Sports"));
-                break;
-           case 0x01:
-                str = cString(tr("Content$Special Event"));
-                break;
-           case 0x02:
-                str = cString(tr("Content$Sport Magazine"));
-                break;
-           case 0x03:
-                str = cString(tr("Content$Football"));
-                break;
-           case 0x04:
-                str = cString(tr("Content$Tennis/Squash"));
-                break;
-           case 0x05:
-                str = cString(tr("Content$Team Sports"));
-                break;
-           case 0x06:
-                str = cString(tr("Content$Athletics"));
-                break;
-           case 0x07:
-                str = cString(tr("Content$Motor Sport"));
-                break;
-           case 0x08:
-                str = cString(tr("Content$Water Sport"));
-                break;
-           case 0x09:
-                str = cString(tr("Content$Winter Sports"));
-                break;
-           case 0x0A:
-                str = cString(tr("Content$Equestrian"));
-                break;
-           case 0x0B:
-                str = cString(tr("Content$Martial Sports"));
-                break;
-           }
-         break;
-
-    case EVCONTENTMASK_CHILDRENYOUTH:
-         switch (contents[i] & 0x0F) {
-           default:
-           case 0x00:
-                str = cString(tr("Content$Children's/Youth Programmes"));
-                break;
-           case 0x01:
-                str = cString(tr("Content$Pre-school Children's Programmes"));
-                break;
-           case 0x02:
-                str = cString(tr("Content$Entertainment Programmes for 6 to 14"));
-                break;
-           case 0x03:
-                str = cString(tr("Content$Entertainment Programmes for 10 to 16"));
-                break;
-           case 0x04:
-                str = cString(tr("Content$Informational/Educational/School Programme"));
-                break;
-           case 0x05:
-                str = cString(tr("Content$Cartoons/Puppets"));
-                break;
-           }
-         break;
-
-    case EVCONTENTMASK_MUSICBALLETDANCE:
-         switch (contents[i] & 0x0F) {
-           default:
-           case 0x00:
-                str = cString(tr("Content$Music/Ballet/Dance"));
-                break;
-           case 0x01:
-                str = cString(tr("Content$Rock/Pop"));
-                break;
-           case 0x02:
-                str = cString(tr("Content$Serious/Classical Music"));
-                break;
-           case 0x03:
-                str = cString(tr("Content$Folk/Tradional Music"));
-                break;
-           case 0x04:
-                str = cString(tr("Content$Jazz"));
-                break;
-           case 0x05:
-                str = cString(tr("Content$Musical/Opera"));
-                break;
-           case 0x06:
-                str = cString(tr("Content$Ballet"));
-                break;
-           }
-         break;
-
-    case EVCONTENTMASK_ARTSCULTURE:
-         switch (contents[i] & 0x0F) {
-           default:
-           case 0x00:
-                str = cString(tr("Content$Arts/Culture"));
-                break;
-           case 0x01:
-                str = cString(tr("Content$Performing Arts"));
-                break;
-           case 0x02:
-                str = cString(tr("Content$Fine Arts"));
-                break;
-           case 0x03:
-                str = cString(tr("Content$Religion"));
-                break;
-           case 0x04:
-                str = cString(tr("Content$Popular Culture/Traditional Arts"));
-                break;
-           case 0x05:
-                str = cString(tr("Content$Literature"));
-                break;
-           case 0x06:
-                str = cString(tr("Content$Film/Cinema"));
-                break;
-           case 0x07:
-                str = cString(tr("Content$Experimental Film/Video"));
-                break;
-           case 0x08:
-                str = cString(tr("Content$Broadcasting/Press"));
-                break;
-           case 0x09:
-                str = cString(tr("Content$New Media"));
-                break;
-           case 0x0A:
-                str = cString(tr("Content$Arts/Culture Magazines"));
-                break;
-           case 0x0B:
-                str = cString(tr("Content$Fashion"));
-                break;
-           }
-         break;
-
-    case EVCONTENTMASK_SOCIALPOLITICALECONOMICS:
-         switch (contents[i] & 0x0F) {
-           default:
-           case 0x00:
-                str = cString(tr("Content$Social/Political/Economics"));
-                break;
-           case 0x01:
-                str = cString(tr("Content$Magazines/Reports/Documentary"));
-                break;
-           case 0x02:
-                str = cString(tr("Content$Economics/Social Advisory"));
-                break;
-           case 0x03:
-                str = cString(tr("Content$Remarkable People"));
-                break;
-           }
-         break;
-
-    case EVCONTENTMASK_EDUCATIONALSCIENCE:
-         switch (contents[i] & 0x0F) {
-           default:
-           case 0x00:
-                str = cString(tr("Content$Education/Science/Factual"));
-                break;
-           case 0x01:
-                str = cString(tr("Content$Nature/Animals/Environment"));
-                break;
-           case 0x02:
-                str = cString(tr("Content$Technology/Natural Sciences"));
-                break;
-           case 0x03:
-                str = cString(tr("Content$Medicine/Physiology/Psychology"));
-                break;
-           case 0x04:
-                str = cString(tr("Content$Foreign Countries/Expeditions"));
-                break;
-           case 0x05:
-                str = cString(tr("Content$Social/Spiritual Sciences"));
-                break;
-           case 0x06:
-                str = cString(tr("Content$Further Education"));
-                break;
-           case 0x07:
-                str = cString(tr("Content$Languages"));
-                break;
-           }
-         break;
-
-    case EVCONTENTMASK_LEISUREHOBBIES:
-         switch (contents[i] & 0x0F) {
-           default:
-           case 0x00:
-                str = cString(tr("Content$Leisure/Hobbies"));
-                break;
-           case 0x01:
-                str = cString(tr("Content$Tourism/Travel"));
-                break;
-           case 0x02:
-                str = cString(tr("Content$Handicraft"));
-                break;
-           case 0x03:
-                str = cString(tr("Content$Motoring"));
-                break;
-           case 0x04:
-                str = cString(tr("Content$Fitness & Health"));
-                break;
-           case 0x05:
-                str = cString(tr("Content$Cooking"));
-                break;
-           case 0x06:
-                str = cString(tr("Content$Advertisement/Shopping"));
-                break;
-           case 0x07:
-                str = cString(tr("Content$Gardening"));
-                break;
-           }
-         break;
-
-    case EVCONTENTMASK_SPECIAL:
-         switch (contents[i] & 0x0F) {
-           case 0x00:
-                str = cString(tr("Content$Original Language"));
-                break;
-           case 0x01:
-                str = cString(tr("Content$Black & White"));
-                break;
-           case 0x02:
-                str = cString(tr("Content$Unpublished"));
-                break;
-           case 0x03:
-                str = cString(tr("Content$Live Broadcast"));
-                break;
-           default:
-                str = cString(tr("Content$Special Characteristics"));
-                break;
-           }
-         break;
-
-    case EVCONTENTMASK_USERDEFINED:
-         switch (contents[i] & 0x0F) {
-           case 0x00:
-                str = cString(tr("Content$Drama")); // UK Freeview
-                break;
-            default:
-                break;
-           }
-         break;
-
-    default:
-         break;
-    }
-  return str;
-}
-
-cString cEvent::GetParentalRatingString(void) const
-{
-  if (parentalRating > 0)
-     return cString::sprintf(tr("Suitable for those aged %d and over"), parentalRating);
-  return NULL;
-}
-#endif /* PARENTALRATING */
-
 void cEvent::Dump(FILE *f, const char *Prefix, bool InfoOnly) const
 {
   if (InfoOnly || startTime + duration + Setup.EPGLinger * 60 >= time(NULL)) {
@@ -620,11 +432,17 @@ void cEvent::Dump(FILE *f, const char *Prefix, bool InfoOnly) const
         fprintf(f, "%sD %s\n", Prefix, description);
         strreplace(description, '|', '\n');
         }
+     if (contents[0]) {
+        fprintf(f, "%sG", Prefix);
+        for (int i = 0; Contents(i); i++)
+            fprintf(f, " %02X", Contents(i));
+        fprintf(f, "\n");
+        }
+     if (parentalRating)
+        fprintf(f, "%sR %d\n", Prefix, parentalRating);
      if (components) {
         for (int i = 0; i < components->NumComponents(); i++) {
             tComponent *p = components->Component(i);
-            if (!Setup.UseDolbyDigital && p->stream == 0x02 && p->type == 0x05)
-               continue;
             fprintf(f, "%sX %s\n", Prefix, *p->ToString());
             }
         }
@@ -646,6 +464,22 @@ bool cEvent::Parse(char *s)
     case 'D': strreplace(t, '|', '\n');
               SetDescription(t);
               break;
+    case 'G': {
+                memset(contents, 0, sizeof(contents));
+                for (int i = 0; i < MaxEventContents; i++) {
+                    char *tail = NULL;
+                    int c = strtol(t, &tail, 16);
+                    if (0x00 < c && c <= 0xFF) {
+                       contents[i] = c;
+                       t = tail;
+                       }
+                    else
+                       break;
+                    }
+              }
+              break;
+    case 'R': SetParentalRating(atoi(t));
+              break;
     case 'X': if (!components)
                  components = new cComponents;
               components->SetComponent(components->NumComponents(), t);
@@ -953,6 +787,7 @@ void cEvent::FixEpgBugs(void)
                      case 0x0F: p->description = strdup("HD 16:9"); break;
                      case 0x0C:
                      case 0x10: p->description = strdup("HD >16:9"); break;
+                     default: ;
                      }
                    EpgBugFixStat(9, ChannelID());
                    }
@@ -971,12 +806,13 @@ void cEvent::FixEpgBugs(void)
                 if (!p->description) {
                    switch (p->type) {
                      case 0x05: p->description = strdup("Dolby Digital"); break;
-                     // all others will just display the language
+                     default: ; // all others will just display the language
                      }
                    EpgBugFixStat(11, ChannelID());
                    }
                 }
                 break;
+           default: ;
            }
          }
      }
@@ -1097,22 +933,25 @@ const cEvent *cSchedule::GetEventAround(time_t Time) const
 #ifdef USE_DDEPGENTRY
 const cEvent *cSchedule::GetPreviousEvent(cEvent *Event) const
 {
-  if (!Event || Event->Duration() == 0 || Event->StartTime() == 0)
-     return NULL;
-  // Returns either the event info to the previous/following event to the given EventID or, if that one can't be found NULL :EW
-  cEvent *pt = NULL;
-  int epgTimeDelta = Setup.DoubleEpgTimeDelta * 60 + 1;
-  for (pt = events.First(); pt; pt = events.Next(pt))
-     if (pt && pt->TableID() == 0x00)
-        if ((Event->StartTime() - pt->StartTime()) > -  epgTimeDelta && (Event->StartTime() - pt->StartTime()) < epgTimeDelta) {
-           if ((pt->Duration() + (pt->Duration()/ 5) + 1) > Event->Duration() && (pt->Duration() - (pt->Duration()/ 5) - 1) < Event->Duration())
-              return pt;
-           else if (pt->Title() && Event->Title() && (strcmp(pt->Title(), ".") != 0 && strcmp(Event->Title(), ".") != 0)) {
-              if (strstr(pt->Title(), Event->Title()) != NULL || strstr(Event->Title(), pt->Title()) != NULL)
-                 return pt;
-              }
-           }
-  return NULL;
+
+   if(!Event || Event->Duration() == 0 || Event->StartTime() == 0)
+      return NULL;
+   // Returns either the event info to the previous/following event to the given EventID or, if that one can't be found NULL :EW
+   cEvent *pt = NULL;
+   int epgTimeDelta = Setup.DoubleEpgTimeDelta * 60 + 1;
+   for (pt = events.First(); pt; pt = events.Next(pt))
+      if(pt && pt->TableID() == 0x00)
+         if ((Event->StartTime() - pt->StartTime()) > -  epgTimeDelta && (Event->StartTime() - pt->StartTime()) < epgTimeDelta)
+         {
+            if((pt->Duration() + (pt->Duration()/ 5) + 1) > Event->Duration() && (pt->Duration() - (pt->Duration()/ 5) - 1) < Event->Duration())
+               return pt;
+            else if (pt->Title() && Event->Title() && (strcmp(pt->Title(), ".") != 0 && strcmp(Event->Title(), ".") != 0))
+            {
+               if (strstr(pt->Title(), Event->Title()) != NULL || strstr(Event->Title(), pt->Title()) != NULL)
+                  return pt;
+            }
+         }
+   return NULL;
 }
 #endif /* DDEPGENTRY */
 
@@ -1235,6 +1074,7 @@ void cSchedule::Dump(FILE *f, const char *Prefix, eDumpMode DumpMode, time_t AtT
                p->Dump(f, Prefix);
             }
             break;
+       default: esyslog("ERROR: unknown DumpMode %d (%s %d)", DumpMode, __FUNCTION__, __LINE__);
        }
      fprintf(f, "%sc\n", Prefix);
      }
