@@ -7,7 +7,7 @@
  * Original version (as used in VDR before 1.3.0) written by
  * Robert Schneider <Robert.Schneider@web.de> and Rolf Hakenes <hakenes@hippomi.de>.
  *
- * $Id: epg.h 1.35 2006/10/07 13:47:19 kls Exp $
+ * $Id: epg.h 2.4 2010/01/08 15:20:34 kls Exp $
  */
 
 #ifndef __EPG_H
@@ -19,6 +19,23 @@
 
 #define MAXEPGBUGFIXLEVEL 3
 
+enum { MaxEventContents = 4 };
+
+enum eEventContentGroup {
+  ecgMovieDrama               = 0x10,
+  ecgNewsCurrentAffairs       = 0x20,
+  ecgShow                     = 0x30,
+  ecgSports                   = 0x40,
+  ecgChildrenYouth            = 0x50,
+  ecgMusicBalletDance         = 0x60,
+  ecgArtsCulture              = 0x70,
+  ecgSocialPoliticalEconomics = 0x80,
+  ecgEducationalScience       = 0x90,
+  ecgLeisureHobbies           = 0xA0,
+  ecgSpecial                  = 0xB0,
+  ecgUserDefined              = 0xF0
+  };
+
 enum eDumpMode { dmAll, dmPresent, dmFollowing, dmAtTime };
 
 struct tComponent {
@@ -50,40 +67,23 @@ class cSchedule;
 
 typedef u_int32_t tEventID;
 
-#ifdef USE_PARENTALRATING
-#define MAXEVCONTENTS                          4
-#define EVCONTENTMASK_MOVIEDRAMA               0x10
-#define EVCONTENTMASK_NEWSCURRENTAFFAIRS       0x20
-#define EVCONTENTMASK_SHOW                     0x30
-#define EVCONTENTMASK_SPORTS                   0x40
-#define EVCONTENTMASK_CHILDRENYOUTH            0x50
-#define EVCONTENTMASK_MUSICBALLETDANCE         0x60
-#define EVCONTENTMASK_ARTSCULTURE              0x70
-#define EVCONTENTMASK_SOCIALPOLITICALECONOMICS 0x80
-#define EVCONTENTMASK_EDUCATIONALSCIENCE       0x90
-#define EVCONTENTMASK_LEISUREHOBBIES           0xA0
-#define EVCONTENTMASK_SPECIAL                  0xB0
-#define EVCONTENTMASK_USERDEFINED              0xF0
-#endif /* PARENTALRATING */
-
 class cEvent : public cListObject {
   friend class cSchedule;
 private:
+  // The sequence of these parameters is optimized for minimal memory waste!
   cSchedule *schedule;     // The Schedule this event belongs to
   tEventID eventID;        // Event ID of this event
   uchar tableID;           // Table ID this event came from
   uchar version;           // Version number of section this event came from
-  int runningStatus;       // 0=undefined, 1=not running, 2=starts in a few seconds, 3=pausing, 4=running
+  uchar runningStatus;     // 0=undefined, 1=not running, 2=starts in a few seconds, 3=pausing, 4=running
+  uchar parentalRating;    // Parental rating of this event
   char *title;             // Title of this event
   char *shortText;         // Short description of this event (typically the episode name in case of a series)
   char *description;       // Description of this event
   cComponents *components; // The stream components of this event
+  uchar contents[MaxEventContents]; // Contents of this event
   time_t startTime;        // Start time of this event
   int duration;            // Duration of this event in seconds
-#ifdef USE_PARENTALRATING
-  uchar contents[MAXEVCONTENTS + 1]; // Contents of this event; list is zero-terminated
-  uchar parentalRating;    // Parental rating of this event
-#endif /* PARENTALRATING */
   time_t vps;              // Video Programming Service timestamp (VPS, aka "Programme Identification Label", PIL)
   time_t seen;             // When this event was last seen in the data stream
 public:
@@ -100,26 +100,22 @@ public:
   const char *ShortText(void) const { return shortText; }
   const char *Description(void) const { return description; }
   const cComponents *Components(void) const { return components; }
+  uchar Contents(int i = 0) const { return (0 <= i && i < MaxEventContents) ? contents[i] : 0; }
+  int ParentalRating(void) const { return parentalRating; }
   time_t StartTime(void) const { return startTime; }
   time_t EndTime(void) const { return startTime + duration; }
   int Duration(void) const { return duration; }
-#ifdef USE_PARENTALRATING
-  uchar Contents(int i = 0) const { return (0 <= i && i < MAXEVCONTENTS) ? contents[i] : 0; }
-  uchar ParentalRating(void) const { return parentalRating; }
-#endif /* PARENTALRATING */
   time_t Vps(void) const { return vps; }
   time_t Seen(void) const { return seen; }
   bool SeenWithin(int Seconds) const { return time(NULL) - seen < Seconds; }
   bool HasTimer(void) const;
   bool IsRunning(bool OrAboutToStart = false) const;
+  static const char *ContentToString(uchar Content);
+  cString GetParentalRatingString(void) const;
   cString GetDateString(void) const;
   cString GetTimeString(void) const;
   cString GetEndTimeString(void) const;
   cString GetVpsString(void) const;
-#ifdef USE_PARENTALRATING
-  cString GetContentsString(int i = 0) const;
-  cString GetParentalRatingString(void) const;
-#endif /* PARENTALRATING */
   void SetEventID(tEventID EventID);
   void SetTableID(uchar TableID);
   void SetVersion(uchar Version);
@@ -128,12 +124,10 @@ public:
   void SetShortText(const char *ShortText);
   void SetDescription(const char *Description);
   void SetComponents(cComponents *Components); // Will take ownership of Components!
+  void SetContents(uchar *Contents);
+  void SetParentalRating(int ParentalRating);
   void SetStartTime(time_t StartTime);
   void SetDuration(int Duration);
-#ifdef USE_PARENTALRATING
-  void SetContents(uchar *Contents);
-  void SetParentalRating(uchar ParentalRating);
-#endif /* PARENTALRATING */
   void SetVps(time_t Vps);
   void SetSeen(void);
   cString ToDescr(void) const;
