@@ -4,13 +4,14 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: menuitems.h 1.25 2008/02/16 16:09:58 kls Exp $
+ * $Id: menuitems.h 2.4 2010/06/06 10:32:38 kls Exp $
  */
 
 #ifndef __MENUITEMS_H
 #define __MENUITEMS_H
 
 #include <limits.h>
+#include "dvbdevice.h"
 #include "osdbase.h"
 
 extern const char *FileNameChars;
@@ -68,6 +69,18 @@ public:
   virtual eOSState ProcessKey(eKeys Key);
   };
 
+class cMenuEditPrcItem : public cMenuEditItem {
+protected:
+  double *value;
+  double min, max;
+  int decimals;
+  int factor;
+  virtual void Set(void);
+public:
+  cMenuEditPrcItem(const char *Name, double *Value, double Min = 0.0, double Max = 1.0, int Decimals = 0);
+  virtual eOSState ProcessKey(eKeys Key);
+  };
+
 class cMenuEditChrItem : public cMenuEditItem {
 private:
   char *value;
@@ -82,46 +95,27 @@ public:
 
 class cMenuEditStrItem : public cMenuEditItem {
 private:
-#ifdef USE_LIEMIKUUTIO
-  int offset;
-#else
   char *value;
   int length;
   const char *allowed;
   int pos, offset;
-#endif /* LIEMIKUUTIO */
   bool insert, newchar, uppercase;
-#ifndef USE_LIEMIKUUTIO
   int lengthUtf8;
   uint *valueUtf8;
-#endif /* LIEMIKUUTIO */
   uint *allowedUtf8;
   uint *charMapUtf8;
   uint *currentCharUtf8;
   eKeys lastKey;
   cTimeMs autoAdvanceTimeout;
-#ifndef USE_LIEMIKUUTIO
   void SetHelpKeys(void);
-#endif /* LIEMIKUUTIO */
   uint *IsAllowed(uint c);
   void AdvancePos(void);
-#ifndef USE_LIEMIKUUTIO
   virtual void Set(void);
-#endif /* LIEMIKUUTIO */
   uint Inc(uint c, bool Up);
+  void Type(uint c);
   void Insert(void);
   void Delete(void);
 protected:
-#ifdef USE_LIEMIKUUTIO
-  char *value;
-  int length;
-  uint *valueUtf8;
-  int lengthUtf8;
-  const char *allowed;
-  int pos;
-  void SetHelpKeys(void);
-  virtual void Set(void);
-#endif /* LIEMIKUUTIO */
   void EnterEditMode(void);
   void LeaveEditMode(bool SaveValue = false);
   bool InEditMode(void) { return valueUtf8 != NULL; }
@@ -131,21 +125,6 @@ public:
   virtual eOSState ProcessKey(eKeys Key);
   };
 
-#ifdef USE_LIEMIKUUTIO
-class cMenuEditRecPathItem : public cMenuEditStrItem {
-protected:
-  char base[MaxFileName];
-  virtual void SetHelpKeys(void);
-  void SetBase(const char* Path);
-  void FindNextLevel();
-  void Find(bool Next);
-public:
-  cMenuEditRecPathItem(const char* Name, char* Path, int Length);
-  ~cMenuEditRecPathItem();
-  virtual eOSState ProcessKey(eKeys Key);
-  };
-#endif /* LIEMIKUUTIO */
-
 class cMenuEditStraItem : public cMenuEditIntItem {
 private:
   const char * const *strings;
@@ -199,6 +178,17 @@ public:
   virtual eOSState ProcessKey(eKeys Key);
   };
 
+class cMenuEditMapItem : public cMenuEditItem {
+protected:
+  int *value;
+  const tDvbParameterMap *map;
+  const char *zeroString;
+  virtual void Set(void);
+public:
+  cMenuEditMapItem(const char *Name, int *Value, const tDvbParameterMap *Map, const char *ZeroString = NULL);
+  virtual eOSState ProcessKey(eKeys Key);
+  };
+
 class cPlugin;
 
 class cMenuSetupPage : public cOsdMenu {
