@@ -15,6 +15,9 @@
 #ifndef SUBMENU_H
 #include "submenu.h"
 #include "plugin.h"
+#ifdef USE_WAREAGLEICON
+#include "iconpatch.h"
+#endif /* WAREAGLEICON */
 
 static const char* TAG_SYSTEM      = "system";
 static const char* TAG_PLUGIN      = "plugin";
@@ -29,14 +32,14 @@ static const char* TRUE_STR        = "yes";
 //# SubMenuNode
 //################################################################################
 
-cSubMenuNode::cSubMenuNode(TiXmlElement * xml, int level,  cSubMenuNodes *currentMenu, cSubMenuNodes *parentMenu)
+cSubMenuNode::cSubMenuNode(TiXmlElement *xml, int level,  cSubMenuNodes *currentMenu, cSubMenuNodes *parentMenu)
 {
   init();
   _parentMenu  = parentMenu;
   _currentMenu = currentMenu;
   _level       = level;
 
-  if (xml != NULL && xml->Type() == TiXmlNode::ELEMENT) {
+  if (xml != NULL && xml->Type() == TiXmlNode::TINYXML_ELEMENT) {
      const char *tag = xml->Value();
 
      if (cSubMenuNode::IsType(tag) != cSubMenuNode::UNDEFINED) {
@@ -44,7 +47,7 @@ cSubMenuNode::cSubMenuNode(TiXmlElement * xml, int level,  cSubMenuNodes *curren
         SetName(xml->Attribute("name"));
         if ((_type == COMMAND) || (_type == THREAD)) {
            SetCommand(xml->Attribute("execute"));
-           const char * confirmStr = xml->Attribute("confirm");
+           const char *confirmStr = xml->Attribute("confirm");
            if (confirmStr != NULL && strcmp(confirmStr, TRUE_STR) == 0)
               _commandConfirm = true;
            }
@@ -134,7 +137,7 @@ void cSubMenuNode::SetPlugin()
 }
 
 
-bool cSubMenuNode::SaveXml(TiXmlElement * root)
+bool cSubMenuNode::SaveXml(TiXmlElement *root)
 {
   bool ok = true;
 
@@ -204,7 +207,7 @@ cSubMenuNode::Type cSubMenuNode::IsType(const char *name)
   return(type);
 }
 
-void cSubMenuNode::SetType(const char * name)
+void cSubMenuNode::SetType(const char *name)
 {
    _type = IsType(name);
 }
@@ -220,7 +223,7 @@ cSubMenuNode::Type cSubMenuNode::GetType()
   return(_type);
 }
 
-const char * cSubMenuNode::GetTypeAsString()
+const char *cSubMenuNode::GetTypeAsString()
 {
   const char *str=NULL;
   switch(_type) {
@@ -248,7 +251,7 @@ const char * cSubMenuNode::GetTypeAsString()
   return(str);
 }
 
-void cSubMenuNode::SetCommand(const char * command)
+void cSubMenuNode::SetCommand(const char *command)
 {
   if (_command != NULL)
      free((void*)_command);
@@ -259,7 +262,7 @@ void cSubMenuNode::SetCommand(const char * command)
      _command = NULL;
 }
 
-const char * cSubMenuNode::GetCommand()
+const char *cSubMenuNode::GetCommand()
 {
   return(_command);
 }
@@ -277,7 +280,7 @@ void cSubMenuNode::SetCommandConfirm(int val)
      _commandConfirm = false;
 }
 
-void cSubMenuNode::SetCustomTitle(const char * title)
+void cSubMenuNode::SetCustomTitle(const char *title)
 {
   if (_title != NULL)
      free((void*)_title);
@@ -288,12 +291,12 @@ void cSubMenuNode::SetCustomTitle(const char * title)
      _title = NULL;
 }
 
-const char * cSubMenuNode::GetCustomTitle()
+const char *cSubMenuNode::GetCustomTitle()
 {
   return(_title);
 }
 
-void cSubMenuNode::SetName(const char * name)
+void cSubMenuNode::SetName(const char *name)
 {
   if (_name)
      free ((void*)_name);
@@ -304,7 +307,7 @@ void cSubMenuNode::SetName(const char * name)
      _name = NULL;
 }
 
-const char * cSubMenuNode::GetName()
+const char *cSubMenuNode::GetName()
 {
   return(_name);
 }
@@ -333,7 +336,7 @@ void cSubMenuNode::SetPluginIndex(int index)
   _pluginIndex = index;
 }
 
-void cSubMenuNode::SetPluginMainMenuEntry(const char * mainMenuEntry)
+void cSubMenuNode::SetPluginMainMenuEntry(const char *mainMenuEntry)
 {
   if (_pluginMainMenuEntry != NULL)
      free((void*)_pluginMainMenuEntry);
@@ -346,35 +349,34 @@ void cSubMenuNode::SetPluginMainMenuEntry(const char * mainMenuEntry)
      _pluginMainMenuEntry = NULL;
 }
 
-const char * cSubMenuNode::GetPluginMainMenuEntry()
+const char *cSubMenuNode::GetPluginMainMenuEntry()
 {
   return(_pluginMainMenuEntry);
 }
 
 
-
-cSubMenuNodes * cSubMenuNode::GetParentMenu()
+cSubMenuNodes *cSubMenuNode::GetParentMenu()
 {
   return(_parentMenu);
 }
 
-void cSubMenuNode::SetParentMenu(cSubMenuNodes * parent)
+void cSubMenuNode::SetParentMenu(cSubMenuNodes *parent)
 {
   _parentMenu = parent;
 }
 
-cSubMenuNodes * cSubMenuNode::GetCurrentMenu()
+cSubMenuNodes *cSubMenuNode::GetCurrentMenu()
 {
   return(_currentMenu);
 }
 
-void cSubMenuNode::SetCurrentMenu(cSubMenuNodes * current)
+void cSubMenuNode::SetCurrentMenu(cSubMenuNodes *current)
 {
   _currentMenu = current;
 }
 
 
-cSubMenuNodes * cSubMenuNode::GetSubMenus()
+cSubMenuNodes *cSubMenuNode::GetSubMenus()
 {
   return(&_subMenus);
 }
@@ -410,8 +412,6 @@ void cSubMenuNode::Print(int index)
 //################################################################################
 cSubMenu::cSubMenu()
 {
-  _menuSuffix            = NULL;
-  _fname                 = NULL;
   _commandResult         = NULL;
   _currentMenuTree       = &_menuTree;
   _currentParentMenuTree = NULL;
@@ -425,10 +425,6 @@ cSubMenu::cSubMenu()
 
 cSubMenu::~cSubMenu()
 {
-  if (_menuSuffix)
-     free(_menuSuffix);
-  if (_fname)
-     free(_fname);
   if (_commandResult)
      free(_commandResult);
   if (_nodeArray)
@@ -437,26 +433,28 @@ cSubMenu::~cSubMenu()
 }
 
 
-bool cSubMenu::LoadXml(char *fname)
+bool cSubMenu::LoadXml(cString fname)
 {
   TiXmlDocument  xmlDoc = TiXmlDocument(fname);
   TiXmlElement  *root   = NULL;
   cSubMenuNode  *node   = NULL;
 
   bool  ok = true;
-  //Clear previously loaded Menu
-  if (_fname != NULL)
-     free(_fname);
+  // Clear previously loaded Menu
   _menuTree.Clear();
-  _fname = strdup(fname);
+  _fname = fname;
 
   if ((ok = xmlDoc.LoadFile())) {
      if ((root = xmlDoc.FirstChildElement("menus")) != NULL) {
-        char *tmp = NULL;
-        if ((tmp = (char*)root->Attribute("suffix")) == NULL)
-           asprintf(&_menuSuffix, " "); // set default menuSuffix // asprintf(&_menuSuffix, " ...");
+        cString tmp = root->Attribute("suffix");
+#ifdef USE_WAREAGLEICON
+        if      (strcmp(tmp, "ICON_FOLDER") == 0)      tmp = cString::sprintf(" %s", IsLangUtf8() ? ICON_FOLDER_UTF8 : ICON_FOLDER);
+        else if (strcmp(tmp, "ICON_MOVE_FOLDER") == 0) tmp = cString::sprintf(" %s", IsLangUtf8() ? ICON_MOVE_FOLDER_UTF8 : ICON_MOVE_FOLDER);
+#endif /* WAREAGLEICON */
+        if (*tmp)
+           _menuSuffix = tmp;
         else
-           asprintf(&_menuSuffix, tmp);
+           _menuSuffix = cString::sprintf(" ");
 
         if ((root = root->FirstChildElement()) != NULL) {
            do {
@@ -474,13 +472,13 @@ bool cSubMenu::LoadXml(char *fname)
            }
         }
      else {
-        esyslog("ERROR: in %s, missing Tag <menus>\n", fname);
+        esyslog("ERROR: in %s, missing Tag <menus>\n", *fname);
         ok = false;
         }
      }
   else {
      esyslog("ERROR: in %s : %s  Col=%d Row=%d\n",
-            fname,
+            *fname,
             xmlDoc.ErrorDesc(),
             xmlDoc.ErrorCol(),
             xmlDoc.ErrorRow());
@@ -497,11 +495,11 @@ bool cSubMenu::SaveXml()
 }
 
 
-bool cSubMenu::SaveXml(char *fname)
+bool cSubMenu::SaveXml(cString fname)
 {
   bool ok = true;
 
-  if (_fname != NULL) {
+  if (*_fname) {
      TiXmlDocument xml = TiXmlDocument(fname);
      TiXmlComment  comment;
      comment.SetValue("\n\
@@ -549,8 +547,7 @@ bool cSubMenu::SaveXml(char *fname)
 }
 
 
-
-cSubMenuNodes * cSubMenu::GetMenuTree()
+cSubMenuNodes *cSubMenu::GetMenuTree()
 {
   return(_currentMenuTree);
 }
@@ -582,7 +579,7 @@ int cSubMenu::GetNrOfNodes()
  * @param index position in the current menu
  * @return node or null if not found
  */
-cSubMenuNode * cSubMenu::GetNode(int index)
+cSubMenuNode *cSubMenu::GetNode(int index)
 {
   cSubMenuNode *node = NULL;
   if (_currentMenuTree == NULL || (node=_currentMenuTree->Get(index)) == NULL)
@@ -597,7 +594,7 @@ cSubMenuNode * cSubMenu::GetNode(int index)
  * @param index specfies the absolut indes in the list of all nodes
  * @return node or NULL if not found
  */
-cSubMenuNode * cSubMenu::GetAbsNode(int index)
+cSubMenuNode *cSubMenu::GetAbsNode(int index)
 {
   cSubMenuNode *node = NULL;
   GetNrOfNodes();
@@ -676,7 +673,7 @@ bool cSubMenu::Up(int *parentIndex)
   return(ok);
 }
 
-const char * cSubMenu::ExecuteCommand(const char * cmd)
+const char *cSubMenu::ExecuteCommand(const char *cmd)
 {
   free(_commandResult);
   _commandResult = NULL;
@@ -752,7 +749,7 @@ void cSubMenu::MoveMenu(int index, int toIndex, enum Where where)
  * @param index index of destination
  * @param menuTitle  Titel of new Menu entry
  */
-void cSubMenu::CreateMenu(int index, const char * menuTitle)
+void cSubMenu::CreateMenu(int index, const char *menuTitle)
 {
   if (index >= 0 && index < _nrNodes) {
      cSubMenuNode *srcNode  = GetAbsNode(index);
@@ -786,7 +783,7 @@ void cSubMenu::DeleteMenu(int index)
 
 // Private Methods
 
-int cSubMenu::countNodes(cSubMenuNodes * tree)
+int cSubMenu::countNodes(cSubMenuNodes *tree)
 {
   int count = 0;
   if (tree != NULL) {
@@ -800,7 +797,7 @@ int cSubMenu::countNodes(cSubMenuNodes * tree)
 }
 
 
-void cSubMenu::tree2Array(cSubMenuNodes * tree, int &index)
+void cSubMenu::tree2Array(cSubMenuNodes *tree, int &index)
 {
   if (tree != NULL) {
      for (cSubMenuNode *node = tree->First(); node; node = tree->Next(node)) {
@@ -812,7 +809,7 @@ void cSubMenu::tree2Array(cSubMenuNodes * tree, int &index)
 
 }
 
-bool cSubMenu::IsPluginInMenu(const char * name)
+bool cSubMenu::IsPluginInMenu(const char *name)
 {
   bool found = false;
   for (int i = 0; i < _nrNodes && found == false; i++) {
@@ -827,7 +824,7 @@ bool cSubMenu::IsPluginInMenu(const char * name)
  * Adds the given plugin to the Menu-Tree if not allready in List
  * @param name specifies the name of the plugin
  */
-void cSubMenu::AddPlugin(const char * name)
+void cSubMenu::AddPlugin(const char *name)
 {
   if (! IsPluginInMenu(name)) {
      cSubMenuNode *node = new  cSubMenuNode(&_menuTree, NULL);
@@ -855,7 +852,7 @@ void cSubMenu::addMissingPlugins()
  * Adds the given command to the Menu-Tree
  * @param name specifies the name of the command
  */
-void cSubMenu::CreateCommand(int index, const char * name, const char * execute, int confirm)
+void cSubMenu::CreateCommand(int index, const char *name, const char *execute, int confirm)
 {
   if (index >= 0 && index < _nrNodes) {
      cSubMenuNode *srcNode  = GetAbsNode(index);
@@ -875,7 +872,7 @@ void cSubMenu::CreateCommand(int index, const char * name, const char * execute,
      }
 }
 
-void cSubMenu::CreateThread(int index, const char * name, const char * execute, int confirm)
+void cSubMenu::CreateThread(int index, const char *name, const char *execute, int confirm)
 {
   if (index >= 0 && index < _nrNodes) {
      cSubMenuNode *srcNode  = GetAbsNode(index);
@@ -933,7 +930,7 @@ void cSubMenu::removeUndefinedNodes()
 */
 const char *cSubMenu::GetParentMenuTitel()
 {
-  const char * result = "";
+  const char *result = "";
 
   if (_currentMenuTree != NULL && _currentParentMenuTree != NULL) {
      cSubMenuNode *node = NULL;
