@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: timers.c 1.73.1.1 2008/04/13 12:47:12 kls Exp $
+ * $Id: timers.c 2.4 2010/01/16 11:18:53 kls Exp $
  */
 
 #include "timers.h"
@@ -13,9 +13,6 @@
 #include "device.h"
 #include "i18n.h"
 #include "libsi/si.h"
-#ifdef USE_LIVEBUFFER
-#include "livebuffer.h"
-#endif /* LIVEBUFFER */
 #include "recording.h"
 #include "remote.h"
 #include "status.h"
@@ -57,6 +54,11 @@ cTimer::cTimer(bool Instant, bool Pause, cChannel *Channel)
   event = NULL;
   if (Instant && channel)
      snprintf(file, sizeof(file), "%s%s", Setup.MarkInstantRecord ? "@" : "", *Setup.NameInstantRecord ? Setup.NameInstantRecord : channel->Name());
+  if (VfatFileSystem && (Utf8StrLen(file) > VFAT_MAX_FILENAME)) {
+     dsyslog("timer file name too long for VFAT file system: '%s'", file);
+     file[Utf8SymChars(file, VFAT_MAX_FILENAME)] = 0;
+     dsyslog("timer file name truncated to '%s'", file);
+     }
 }
 
 cTimer::cTimer(const cEvent *Event)
@@ -92,6 +94,11 @@ cTimer::cTimer(const cEvent *Event)
   const char *Title = Event->Title();
   if (!isempty(Title))
      Utf8Strn0Cpy(file, Event->Title(), sizeof(file));
+  if (VfatFileSystem && (Utf8StrLen(file) > VFAT_MAX_FILENAME)) {
+     dsyslog("timer file name too long for VFAT file system: '%s'", file);
+     file[Utf8SymChars(file, VFAT_MAX_FILENAME)] = 0;
+     dsyslog("timer file name truncated to '%s'", file);
+     }
   aux = NULL;
   event = NULL; // let SetEvent() be called to get a log message
 #ifdef USE_PINPLUGIN
@@ -306,18 +313,18 @@ bool cTimer::Parse(const char *s)
      //TODO add more plausibility checks
      result = ParseDay(daybuffer, day, weekdays);
      if (VfatFileSystem) {
-        char *p = strrchr(filebuffer, '~');
+        char *p = strrchr(filebuffer, FOLDERDELIMCHAR);
         if (p)
            p++;
         else
            p = filebuffer;
-        if (strlen(p) > VFAT_MAX_FILENAME) {
+        if (Utf8StrLen(p) > VFAT_MAX_FILENAME) {
            dsyslog("timer file name too long for VFAT file system: '%s'", p);
-           p[VFAT_MAX_FILENAME] = 0;
+           p[Utf8SymChars(p, VFAT_MAX_FILENAME)] = 0;
            dsyslog("timer file name truncated to '%s'", p);
            }
         }
-     Utf8Strn0Cpy(file, filebuffer, MaxFileName);
+     Utf8Strn0Cpy(file, filebuffer, sizeof(file));
      strreplace(file, '|', ':');
      if (isnumber(channelbuffer))
         channel = Channels.GetByNumber(atoi(channelbuffer));
@@ -481,7 +488,7 @@ int cTimer::Matches(const cEvent *Event, int *Overlap) const
 
 bool cTimer::Expired(void) const
 {
-  return IsSingleEvent() && !Recording() && StopTime() + EXPIRELATENCY <= time(NULL) && (!HasFlags(tfVps) || !event);
+  return IsSingleEvent() && !Recording() && StopTime() + EXPIRELATENCY <= time(NULL) && (!HasFlags(tfVps) || !event || !event->Vps());
 }
 
 time_t cTimer::StartTime(void) const
@@ -653,15 +660,17 @@ void cTimer::SetFskProtection(int aFlag)
       // add protection info to aux
 
       if (aux) { tmp = strdup(aux); free(aux); }
-      asprintf(&aux,"%s<pin-plugin><protected>yes</protected></pin-plugin>", tmp ? tmp : "");
+      if (asprintf(&aux,"%s<pin-plugin><protected>yes</protected></pin-plugin>", tmp ? tmp : "") < 0 )
+         aux = NULL;
    }
    else if (!fskProtection && aux && (p = strstr(aux, "<pin-plugin><protected>yes</protected></pin-plugin>")))
    {
       // remove protection info to aux
 
-      asprintf(&tmp, "%.*s%s", p-aux, aux, p+strlen("<pin-plugin><protected>yes</protected></pin-plugin>"));
-      free(aux);
-      aux = strdup(tmp);
+      if (asprintf(&tmp, "%.*s%s", p-aux, aux, p+strlen("<pin-plugin><protected>yes</protected></pin-plugin>")) >= 0 ) {
+         free(aux);
+         aux = strdup(tmp);
+         }
    }
 
    if (tmp)
@@ -698,11 +707,7 @@ cTimer *cTimers::GetMatch(time_t t)
   static int LastPending = -1;
   cTimer *t0 = NULL;
   for (cTimer *ti = First(); ti; ti = Next(ti)) {
-#ifdef USE_LIVEBUFFER
-      if (!ti->Recording() && (ti->Matches(t) || cLiveBufferManager::InLiveBuffer(ti))) {
-#else
       if (!ti->Recording() && ti->Matches(t)) {
-#endif /* LIVEBUFFER */
          if (ti->Pending()) {
             if (ti->Index() > LastPending)
                LastPending = ti->Index();
@@ -803,11 +808,7 @@ void cTimers::DeleteExpired(void)
   cTimer *ti = First();
   while (ti) {
         cTimer *next = Next(ti);
-#ifdef USE_LIVEBUFFER
-        if (ti->Expired() && (!cLiveBufferManager::InLiveBuffer(ti) || !ti->HasFlags(tfActive))) {
-#else
         if (ti->Expired()) {
-#endif /* LIVEBUFFER */
            isyslog("deleting timer %s", *ti->ToDescr());
            Del(ti);
            SetModified();
