@@ -18,11 +18,11 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  * Or, point your browser to http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
  *
- * The author can be reached at kls@cadsoft.de
+ * The author can be reached at kls@tvdr.de
  *
- * The project's page is at http://www.cadsoft.de/vdr
+ * The project's page is at http://www.tvdr.de
  *
- * $Id: vdr.c 1.313.1.1 2008/09/06 11:07:52 kls Exp $
+ * $Id: vdr.c 2.19 2010/04/05 10:06:16 kls Exp $
  */
 
 #include <getopt.h>
@@ -50,9 +50,6 @@
 #include "keys.h"
 #include "libsi/si.h"
 #include "lirc.h"
-#ifdef USE_LIVEBUFFER
-#include "livebuffer.h"
-#endif /* LIVEBUFFER */
 #include "menu.h"
 #include "osdbase.h"
 #include "plugin.h"
@@ -61,6 +58,7 @@
 #include "shutdown.h"
 #include "skinclassic.h"
 #include "skinsttng.h"
+#include "sourceparams.h"
 #include "sources.h"
 #include "themes.h"
 #include "timers.h"
@@ -115,10 +113,10 @@ static bool SetUser(const char *UserName, bool UserDump)//XXX name?
   return true;
 }
 
-static bool SetCapSysTime(void)
+static bool DropCaps(void)
 {
-  // drop all capabilities except cap_sys_time
-  cap_t caps = cap_from_text("= cap_sys_time=ep");
+  // drop all capabilities except selected ones
+  cap_t caps = cap_from_text("= cap_sys_nice,cap_sys_time=ep");
   if (!caps) {
      fprintf(stderr, "vdr: cap_from_text failed: %s\n", strerror(errno));
      return false;
@@ -142,17 +140,10 @@ static bool SetKeepCaps(bool On)
   return true;
 }
 
-#ifdef USE_SETTIME
-char *SetTime = NULL;
-#endif /* SETTIME */
-
 static void SignalHandler(int signum)
 {
   switch (signum) {
     case SIGPIPE:
-#ifdef USE_EM84XX
-    case SIGINT:
-#endif /* EM84XX */
          break;
     case SIGHUP:
          LastSignal = signum;
@@ -186,7 +177,7 @@ int main(int argc, char *argv[])
 
   // Command line options:
 
-#define DEFAULTSVDRPPORT 2001
+#define DEFAULTSVDRPPORT 6419
 #define DEFAULTWATCHDOG     0 // seconds
 #define DEFAULTCONFDIR CONFDIR
 #define DEFAULTPLUGINDIR PLUGINDIR
@@ -199,7 +190,6 @@ int main(int argc, char *argv[])
   const char *AudioCommand = NULL;
   const char *ConfigDirectory = NULL;
   const char *EpgDataFileName = DEFAULTEPGDATAFILENAME;
-  bool DisplayExtensions = false;
   bool DisplayHelp = false;
   bool DisplayVersion = false;
   bool DaemonMode = false;
@@ -220,9 +210,6 @@ int main(int argc, char *argv[])
 #elif defined(REMOTE_RCU)
   RcuDevice = RCU_DEVICE;
 #endif
-#if defined(VFAT)
-  VfatFileSystem = true;
-#endif
 #if defined(VDR_USER)
   VdrUser = VDR_USER;
 #endif
@@ -231,16 +218,15 @@ int main(int argc, char *argv[])
 
   static struct option long_options[] = {
       { "audio",    required_argument, NULL, 'a' },
-#ifdef USE_LIVEBUFFER
-      { "buffer",   required_argument, NULL, 'b' },
-#endif /* LIVEBUFFER */
       { "config",   required_argument, NULL, 'c' },
       { "daemon",   no_argument,       NULL, 'd' },
       { "device",   required_argument, NULL, 'D' },
+      { "edit",     required_argument, NULL, 'e' | 0x100 },
       { "epgfile",  required_argument, NULL, 'E' },
-      { "extensions",no_argument,      NULL, 'e' | 0x100 },
+      { "genindex", required_argument, NULL, 'g' | 0x100 },
       { "grab",     required_argument, NULL, 'g' },
       { "help",     no_argument,       NULL, 'h' },
+      { "instance", required_argument, NULL, 'i' },
       { "lib",      required_argument, NULL, 'L' },
       { "lirc",     optional_argument, NULL, 'l' | 0x100 },
       { "localedir",required_argument, NULL, 'l' | 0x200 },
@@ -253,41 +239,20 @@ int main(int argc, char *argv[])
       { "record",   required_argument, NULL, 'r' },
       { "shutdown", required_argument, NULL, 's' },
       { "terminal", required_argument, NULL, 't' },
-#ifdef USE_SETTIME
-      { "timeset",  required_argument, NULL, 'T' },
-#endif /* SETTIME */
       { "user",     required_argument, NULL, 'u' },
       { "userdump", no_argument,       NULL, 'u' | 0x100 },
       { "version",  no_argument,       NULL, 'V' },
       { "vfat",     no_argument,       NULL, 'v' | 0x100 },
       { "video",    required_argument, NULL, 'v' },
       { "watchdog", required_argument, NULL, 'w' },
-      { NULL }
+      { NULL,       no_argument,       NULL,  0  }
     };
 
   int c;
-#ifdef USE_SETTIME
-#ifdef USE_LIVEBUFFER
-  while ((c = getopt_long(argc, argv, "a::b:c:dD:E:g:hl:L:mp:P:r:s:t:T:u:v:Vw:", long_options, NULL)) != -1) {
-#else
-  while ((c = getopt_long(argc, argv, "a:c:dD:E:g:hl:L:mp:P:r:s:t:T:u:v:Vw:", long_options, NULL)) != -1) {
-#endif /* LIVEBUFFER */
-#else
-#ifdef USE_LIVEBUFFER
-  while ((c = getopt_long(argc, argv, "a::b:c:dD:E:g:hl:L:mp:P:r:s:t:u:v:Vw:", long_options, NULL)) != -1) {
-#else
-  while ((c = getopt_long(argc, argv, "a:c:dD:E:g:hl:L:mp:P:r:s:t:u:v:Vw:", long_options, NULL)) != -1) {
-#endif /* LIVEBUFFER */
-#endif /* SETTIME */
+  while ((c = getopt_long(argc, argv, "a:c:dD:e:E:g:hi:l:L:mp:P:r:s:t:u:v:Vw:", long_options, NULL)) != -1) {
         switch (c) {
           case 'a': AudioCommand = optarg;
                     break;
-#ifdef USE_LIVEBUFFER
-          case 'b': BufferDirectory = optarg;
-                    while (optarg && *optarg && optarg[strlen(optarg) - 1] == '/')
-                          optarg[strlen(optarg) - 1] = 0;
-                    break;
-#endif /* LIVEBUFFER */
           case 'c': ConfigDirectory = optarg;
                     break;
           case 'd': DaemonMode = true; break;
@@ -302,14 +267,22 @@ int main(int argc, char *argv[])
                     return 2;
                     break;
           case 'e' | 0x100:
-                    DisplayExtensions = true;
-                    break;
+                    return CutRecording(optarg) ? 0 : 2;
           case 'E': EpgDataFileName = (*optarg != '-' ? optarg : NULL);
                     break;
+          case 'g' | 0x100:
+                    return GenerateIndex(optarg) ? 0 : 2;
           case 'g': cSVDRP::SetGrabImageDir(*optarg != '-' ? optarg : NULL);
                     break;
           case 'h': DisplayHelp = true;
                     break;
+          case 'i': if (isnumber(optarg)) {
+                       InstanceId = atoi(optarg);
+                       if (InstanceId >= 0)
+                          break;
+                       }
+                    fprintf(stderr, "vdr: invalid instance id: %s\n", optarg);
+                    return 2;
           case 'l': {
                       char *p = strchr(optarg, '.');
                       if (p)
@@ -375,10 +348,6 @@ int main(int argc, char *argv[])
                     break;
           case 's': ShutdownHandler.SetShutdownCommand(optarg);
                     break;
-#ifdef USE_SETTIME
-          case 'T': SetTime = strdup(optarg);
-                    break;
-#endif /* SETTIME */
           case 't': Terminal = optarg;
                     if (access(Terminal, R_OK | W_OK) < 0) {
                        fprintf(stderr, "vdr: can't access terminal: %s\n", Terminal);
@@ -425,7 +394,7 @@ int main(int argc, char *argv[])
            return 2;
         if (!SetKeepCaps(false))
            return 2;
-        if (!SetCapSysTime())
+        if (!DropCaps())
            return 2;
         }
      }
@@ -439,26 +408,24 @@ int main(int argc, char *argv[])
      if (DisplayHelp) {
         printf("Usage: vdr [OPTIONS]\n\n"          // for easier orientation, this is column 80|
                "  -a CMD,   --audio=CMD    send Dolby Digital audio to stdin of command CMD\n"
-#ifdef USE_LIVEBUFFER
-               "  -b DIR,   --buffer=DIR   use DIR as LiveBuffer directory (default is to write\n"
-               "                           it to the video directory)\n"
-#endif /* LIVEBUFFER */
                "  -c DIR,   --config=DIR   read config files from DIR (default: %s)\n"
                "  -d,       --daemon       run in daemon mode\n"
                "  -D NUM,   --device=NUM   use only the given DVB device (NUM = 0, 1, 2...)\n"
                "                           there may be several -D options (default: all DVB\n"
                "                           devices will be used)\n"
-               "            --extensions   print patchlevel and exit\n"
+               "            --edit=REC     cut recording REC and exit\n"
                "  -E FILE,  --epgfile=FILE write the EPG data into the given FILE (default is\n"
                "                           '%s' in the video directory)\n"
                "                           '-E-' disables this\n"
                "                           if FILE is a directory, the default EPG file will be\n"
                "                           created in that directory\n"
+               "            --genindex=REC generate index for recording REC and exit\n"
                "  -g DIR,   --grab=DIR     write images from the SVDRP command GRAB into the\n"
                "                           given DIR; DIR must be the full path name of an\n"
                "                           existing directory, without any \"..\", double '/'\n"
                "                           or symlinks (default: none, same as -g-)\n"
                "  -h,       --help         print this help and exit\n"
+               "  -i ID,    --instance=ID  use ID as the id of this VDR instance (default: 0)\n"
                "  -l LEVEL, --log=LEVEL    set log level (default: 3)\n"
                "                           0 = no logging, 1 = errors only,\n"
                "                           2 = errors and info, 3 = errors, info and debug\n"
@@ -479,9 +446,6 @@ int main(int argc, char *argv[])
                "  -r CMD,   --record=CMD   call CMD before and after a recording\n"
                "  -s CMD,   --shutdown=CMD call CMD to shutdown the computer\n"
                "  -t TTY,   --terminal=TTY controlling tty\n"
-#ifdef USE_SETTIME
-               "  -T CMD,   --timeset=CMD  call CMD to set the system time\n"
-#endif /* SETTIME */
                "  -u USER,  --user=USER    run as user USER; only applicable if started as\n"
                "                           root\n"
                "            --userdump     allow coredumps if -u is given (debugging)\n"
@@ -525,205 +489,6 @@ int main(int argc, char *argv[])
      return 0;
      }
 
-  if (DisplayExtensions) {
-     printf("VDR %s\n", VDRVERSION);
-     printf("VDREXTENSIONS %d\n", VDREXTENSIONS);
-
-#ifdef USE_ANALOGTV
-     printf("  ANALOGTV\n");
-#endif /* ANALOGTV */
-
-#ifdef USE_ATSC
-     printf("  ATSC\n");
-#endif /* ATSC */
-
-#ifdef USE_CHANNELSCAN
-     printf("  CHANNELSCAN\n");
-#endif /* CHANNELSCAN */
-
-#ifdef USE_CMDRECCMDI18N
-     printf("  CMDRECCMDI18N\n");
-#endif /* CMDRECCMDI18N */
-
-#ifdef USE_CMDSUBMENU
-     printf("  CMDSUBMENU %d\n", CMDSUBMENUVERSNUM);
-#endif /* CMDSUBMENU */
-
-#ifdef USE_CUTTERLIMIT
-     printf("  CUTTERLIMIT\n");
-#endif /* CUTTERLIMIT */
-
-#ifdef USE_CUTTERQUEUE
-     printf("  CUTTERQUEUE\n");
-#endif /* CUTTERQUEUE */
-
-#ifdef USE_CUTTIME
-     printf("  CUTTIME\n");
-#endif /* CUTTIME */
-
-#ifdef USE_DDEPGENTRY
-     printf("  DDEPGENTRY\n");
-#endif /* DDEPGENTRY */
-
-#ifdef USE_DELTIMESHIFTREC
-     printf("  DELTIMESHIFTREC\n");
-#endif /* DELTIMESHIFTREC */
-
-#ifdef USE_DOLBYINREC
-     printf("  DOLBYINREC\n");
-#endif /* DOLBYINREC */
-
-#ifdef USE_DVBPLAYER
-     printf("  DVBPLAYER\n");
-#endif /* DVBPLAYER */
-
-#ifdef USE_DVBSETUP
-     printf("  DVBSETUP\n");
-#endif /* DVBSETUP */
-
-#ifdef USE_DVDARCHIVE
-     printf("  DVDARCHIVE\n");
-#endif /* DVDARCHIVE */
-
-#ifdef USE_DVDCHAPJUMP
-     printf("  DVDCHAPJUMP\n");
-#endif /* DVDCHAPJUMP */
-
-#ifdef USE_DVLRECSCRIPTADDON
-     printf("  DVLRECSCRIPTADDON\n");
-#endif /* DVLRECSCRIPTADDON */
-
-#ifdef USE_DVLVIDPREFER
-     printf("  DVLVIDPREFER\n");
-#endif /* DVLVIDPREFER */
-
-#ifdef USE_DVLFRIENDLYFNAMES
-     printf("  DVLFRIENDLYFNAMES\n");
-#endif /* DVLFRIENDLYFNAMES */
-
-#ifdef USE_EM84XX
-     printf("  EM84XX\n");
-#endif /* EM84XX */
-
-#ifdef USE_GRAPHTFT
-     printf("  GRAPHTFT\n");
-#endif /* GRAPHTFT */
-
-#ifdef USE_HARDLINKCUTTER
-     printf("  HARDLINKCUTTER\n");
-#endif /* HARDLINKCUTTER */
-
-#ifdef USE_IPTV
-     printf("  IPTV %d\n", PLUGINPARAMPATCHVERSNUM);
-#endif /* IPTV */
-
-#ifdef USE_JUMPPLAY
-     printf("  JUMPPLAY %d\n", JUMPPLAYVERSNUM);
-#endif /* JUMPPLAY */
-
-#ifdef USE_LIEMIKUUTIO
-     printf("  LIEMIKUUTIO %d\n", LIEMIKUUTIO);
-#endif /* LIEMIKUUTIO */
-
-#ifdef USE_LIRCSETTINGS
-     printf("  LIRCSETTINGS\n");
-#endif /* LIRCSETTINGS */
-
-#ifdef USE_LIVEBUFFER
-     printf("  LIVEBUFFER\n");
-#endif /* LIVEBUFFER */
-
-#ifdef USE_LNBSHARE
-     printf("  LNBSHARE\n");
-#endif /* LNBSHARE */
-
-#ifdef USE_MAINMENUHOOKS
-     printf("  MAINMENUHOOKS %3.1f\n", MAINMENUHOOKSVERSNUM);
-#endif /* MAINMENUHOOKS */
-
-#ifdef USE_MENUORG
-     printf("  MENUORG\n");
-#endif /* MENUORG */
-
-#ifdef USE_NOEPG
-     printf("  NOEPG\n");
-#endif /* NOEPG */
-
-#ifdef USE_OSDMAXITEMS
-     printf("  OSDMAXITEMS\n");
-#endif /* OSDMAXITEMS */
-
-#ifdef USE_PARENTALRATING
-     printf("  PARENTALRATING\n");
-#endif /* PARENTALRATING */
-
-#ifdef USE_PINPLUGIN
-     printf("  PINPLUGIN %d\n", PIN_PLUGIN_PATCH);
-#endif /* PINPLUGIN */
-
-#ifdef USE_PLUGINAPI
-     printf("  PLUGINAPI\n");
-#endif /* PLUGINAPI */
-
-#ifdef USE_PLUGINMISSING
-     printf("  PLUGINMISSING\n");
-#endif /* PLUGINMISSING */
-
-#ifdef USE_ROTOR
-     printf("  ROTOR\n");
-#endif /* ROTOR */
-
-#ifdef USE_SETTIME
-     printf("  SETTIME\n");
-#endif /* SETTIME */
-
-#ifdef USE_SETUP
-     printf("  SETUP\n");
-#endif /* SETUP */
-
-#ifdef USE_SOURCECAPS
-     printf("  SOURCECAPS\n");
-#endif /* SOURCECAPS */
-
-#ifdef USE_SORTRECORDS
-     printf("  SORTRECORDS\n");
-#endif /* SORTRECORDS */
-
-#ifdef USE_SYNCEARLY
-     printf("  SYNCEARLY\n");
-#endif /* SYNCEARLY */
-
-#ifdef USE_TIMERCMD
-     printf("  TIMERCMD\n");
-#endif /* TIMERCMD */
-
-#ifdef USE_TIMERINFO
-     printf("  TIMERINFO\n");
-#endif /* TIMERINFO */
-
-#ifdef USE_TTXTSUBS
-     printf("  TTXTSUBS\n");
-#endif /* TTXTSUBS*/
-
-#ifdef USE_VALIDINPUT
-     printf("  VALIDINPUT\n");
-#endif /* VALIDINPUT */
-
-#ifdef USE_VOLCTRL
-     printf("  VOLCTRL\n");
-#endif /* VOLCTRL */
-
-#ifdef USE_WAREAGLEICON
-     printf("  WAREAGLEICON\n");
-#endif /* WAREAGLEICON */
-
-#ifdef USE_YAEPG
-     printf("  YAEPG\n");
-#endif /* YAEPG */
-
-     return 0;
-     }
-
   // Log file:
 
   if (SysLogLevel > 0)
@@ -751,6 +516,7 @@ int main(int argc, char *argv[])
      stdout = freopen(Terminal, "w", stdout);
      stderr = freopen(Terminal, "w", stderr);
      HasStdin = true;
+     tcgetattr(STDIN_FILENO, &savedTm);
      }
 
   isyslog("VDR version %s started", VDRVERSION);
@@ -778,6 +544,7 @@ int main(int argc, char *argv[])
      isyslog("codeset is '%s' - %s", CodeSet, known ? "known" : "unknown");
      cCharSetConv::SetSystemCharacterTable(CodeSet);
      }
+  setlocale(LC_NUMERIC, "C"); // makes sure any floating point numbers written use a decimal point
 
   // Initialize internationalization:
 
@@ -803,11 +570,6 @@ int main(int argc, char *argv[])
   if (!PluginManager.LoadPlugins(true))
      EXIT(2);
 
-#ifdef USE_LIVEBUFFER
-  if (!BufferDirectory)
-     BufferDirectory = VideoDirectory;
-#endif /* LIVEBUFFER */
-
   // Configuration data:
 
   if (!ConfigDirectory)
@@ -817,29 +579,21 @@ int main(int argc, char *argv[])
   cThemes::SetThemesDirectory(AddDirectory(ConfigDirectory, "themes"));
 
   Setup.Load(AddDirectory(ConfigDirectory, "setup.conf"));
-  if (!(Sources.Load(AddDirectory(ConfigDirectory, "sources.conf"), true, true) &&
-        Diseqcs.Load(AddDirectory(ConfigDirectory, "diseqc.conf"), true, Setup.DiSEqC) &&
-        Channels.Load(AddDirectory(ConfigDirectory, "channels.conf"), false, true) &&
-        Timers.Load(AddDirectory(ConfigDirectory, "timers.conf")) &&
-#ifdef USE_CMDRECCMDI18N
-        LoadCommandsI18n(Commands,AddDirectory(ConfigDirectory, "commands.conf"), true) &&
-        LoadCommandsI18n(RecordingCommands, AddDirectory(ConfigDirectory, "reccmds.conf"), true) &&
+  Sources.Load(AddDirectory(ConfigDirectory, "sources.conf"), true, true);
+  Diseqcs.Load(AddDirectory(ConfigDirectory, "diseqc.conf"), true, Setup.DiSEqC);
+  Channels.Load(AddDirectory(ConfigDirectory, "channels.conf"), false, true);
+  Timers.Load(AddDirectory(ConfigDirectory, "timers.conf"));
+  Commands.Load(AddDirectory(ConfigDirectory, "commands.conf"));
+  RecordingCommands.Load(AddDirectory(ConfigDirectory, "reccmds.conf"));
+  SVDRPhosts.Load(AddDirectory(ConfigDirectory, "svdrphosts.conf"), true);
+  Keys.Load(AddDirectory(ConfigDirectory, "remote.conf"));
+#ifdef USE_ALTERNATECHANNEL
+  KeyMacros.Load(AddDirectory(ConfigDirectory, "keymacros.conf"), true) &&
+    Channels.LoadAlternativeChannels(AddDirectory(ConfigDirectory, "channel_alternative.conf"));
 #else
-        Commands.Load(AddDirectory(ConfigDirectory, "commands.conf"), true) &&
-        RecordingCommands.Load(AddDirectory(ConfigDirectory, "reccmds.conf"), true) &&
-#endif /* CMDRECCMDI18N */
-#ifdef USE_TIMERCMD
-#ifdef USE_CMDRECCMDI18N
-        LoadCommandsI18n(TimerCommands, AddDirectory(ConfigDirectory, "timercmds.conf"), true) &&
-#else
-        TimerCommands.Load(AddDirectory(ConfigDirectory, "timercmds.conf"), true) &&
-#endif /* CMDRECCMDI18N */
-#endif /* TIMERCMD */
-        SVDRPhosts.Load(AddDirectory(ConfigDirectory, "svdrphosts.conf"), true) &&
-        Keys.Load(AddDirectory(ConfigDirectory, "remote.conf")) &&
-        KeyMacros.Load(AddDirectory(ConfigDirectory, "keymacros.conf"), true)
-        ))
-     EXIT(2);
+  KeyMacros.Load(AddDirectory(ConfigDirectory, "keymacros.conf"), true);
+#endif /* ALTERNATECHANNEL */
+  Folders.Load(AddDirectory(ConfigDirectory, "folders.conf"));
 
   if (!*cFont::GetFontFileName(Setup.FontOsd)) {
      const char *msg = "no fonts available - OSD will not show any text!";
@@ -997,11 +751,7 @@ int main(int argc, char *argv[])
         // Make sure we have a visible programme in case device usage has changed:
         if (!EITScanner.Active() && cDevice::PrimaryDevice()->HasDecoder() && !cDevice::PrimaryDevice()->HasProgramme()) {
            static time_t lastTime = 0;
-#ifdef USE_CHANNELSCAN
-           if (!scanning_on_receiving_device && (!Menu || CheckHasProgramme) && Now - lastTime > MINCHANNELWAIT) {
-#else
            if ((!Menu || CheckHasProgramme) && Now - lastTime > MINCHANNELWAIT) { // !Menu to avoid interfering with the CAM if a CAM menu is open
-#endif /* CHANNELSCAN */
               cChannel *Channel = Channels.GetByNumber(cDevice::CurrentChannel());
               if (Channel && (Channel->Vpid() || Channel->Apid(0))) {
                  if (!Channels.SwitchTo(cDevice::CurrentChannel()) // try to switch to the original channel...
@@ -1013,6 +763,14 @@ int main(int argc, char *argv[])
               CheckHasProgramme = false;
               }
            }
+        // Update the OSD size:
+        {
+          static time_t lastOsdSizeUpdate = 0;
+          if (Now != lastOsdSizeUpdate) { // once per second
+             cOsdProvider::UpdateOsdSize();
+             lastOsdSizeUpdate = Now;
+             }
+        }
         // Restart the Watchdog timer:
         if (WatchdogTimeout > 0) {
            int LatencyTime = WatchdogTimeout - alarm(WatchdogTimeout);
@@ -1057,17 +815,8 @@ int main(int argc, char *argv[])
            }
         // Channel display:
         if (!EITScanner.Active() && cDevice::CurrentChannel() != LastChannel) {
-#ifdef USE_LIVEBUFFER
-           if (!Menu) {
-              if (cControl::Control(true))
-                 cControl::Control(true)->Hide();
-#else
            if (!Menu)
-#endif /* LIVEBUFFER */
               Menu = new cDisplayChannel(cDevice::CurrentChannel(), LastChannel >= 0);
-#ifdef USE_LIVEBUFFER
-              }
-#endif /* LIVEBUFFER */
            LastChannel = cDevice::CurrentChannel();
            LastChannelChanged = Now;
            }
@@ -1175,6 +924,21 @@ int main(int argc, char *argv[])
            Recordings.Update();
            DeletedRecordings.Update();
            }
+#ifdef USE_MCLI
+        cPlugin *mcliPlugin = cPluginManager::GetPlugin("mcli");
+        if (mcliPlugin) {
+           if (!ShutdownHandler.countdown) { // if kPower has been pressed, cMenuShutdown takes precedence over other menus
+              cOsdObject *MyMenu = mcliPlugin->AltMenuAction();
+              if (MyMenu) { // is there any cam-menu waiting?
+                 DELETE_MENU;
+                 if (cControl::Control())
+                    cControl::Control()->Hide();
+                    Menu = MyMenu;
+                    Menu->Show();
+                 }
+              }
+           }
+#endif /* MCLI */
         // CAM control:
         if (!Menu && !cOsd::IsOpen())
            Menu = CamControl();
@@ -1182,15 +946,8 @@ int main(int argc, char *argv[])
         if (!Skins.IsOpen())
            Skins.ProcessQueuedMessages();
         // User Input:
-#ifdef USE_LIVEBUFFER
-        cOsdObject *Interact = Menu ? Menu : cControl::Control(true);
-#else
         cOsdObject *Interact = Menu ? Menu : cControl::Control();
-#endif /* LIVEBUFFER */
         eKeys key = Interface->GetKey(!Interact || !Interact->NeedsFastResponse());
-#ifdef USE_LIVEBUFFER
-        Interact = Menu ? Menu : cControl::Control();
-#endif /* LIVEBUFFER */
         if (ISREALKEY(key)) {
 #ifdef USE_PINPLUGIN
            cStatus::MsgUserAction(key, Interact);
@@ -1211,17 +968,9 @@ int main(int argc, char *argv[])
                bool WasMenu = Interact && Interact->IsMenu();
                if (Menu)
                   DELETE_MENU;
-#ifdef USE_LIVEBUFFER
-               else if (cControl::Control(true)) {
-#else
                else if (cControl::Control()) {
-#endif /* LIVEBUFFER */
                   if (cOsd::IsOpen())
-#ifdef USE_LIVEBUFFER
-                     cControl::Control(true)->Hide();
-#else
                      cControl::Control()->Hide();
-#endif /* LIVEBUFFER */
                   else
                      WasOpen = false;
                   }
@@ -1254,39 +1003,25 @@ int main(int argc, char *argv[])
                }
                break;
           // Direct main menu functions:
-#ifdef USE_LIVEBUFFER
-          #define DirectMainFunction(function)\
-            DELETE_MENU;\
-            if (cControl::Control(true))\
-               cControl::Control(true)->Hide();\
-            Menu = new cMenuMain(function);\
-            key = kNone; // nobody else needs to see this key
-#else
           #define DirectMainFunction(function)\
             DELETE_MENU;\
             if (cControl::Control())\
                cControl::Control()->Hide();\
             Menu = new cMenuMain(function);\
             key = kNone; // nobody else needs to see this key
-#endif /* LIVEBUFFER */
           case kSchedule:   DirectMainFunction(osSchedule); break;
           case kChannels:   DirectMainFunction(osChannels); break;
           case kTimers:     DirectMainFunction(osTimers); break;
           case kRecordings: DirectMainFunction(osRecordings); break;
           case kSetup:      DirectMainFunction(osSetup); break;
           case kCommands:   DirectMainFunction(osCommands); break;
-          case kUser1 ... kUser9: cRemote::PutMacro(key); key = kNone; break;
+          case kUser0 ... kUser9: cRemote::PutMacro(key); key = kNone; break;
           case k_Plugin: {
                const char *PluginName = cRemote::GetPlugin();
                if (PluginName) {
                   DELETE_MENU;
-#ifdef USE_LIVEBUFFER
-                  if (cControl::Control(true))
-                     cControl::Control(true)->Hide();
-#else
                   if (cControl::Control())
                      cControl::Control()->Hide();
-#endif /* LIVEBUFFER */
                   cPlugin *plugin = cPluginManager::GetPlugin(PluginName);
                   if (plugin) {
 #ifdef USE_PINPLUGIN
@@ -1310,17 +1045,8 @@ int main(int argc, char *argv[])
           case kChanUp:
           case kChanDn|k_Repeat:
           case kChanDn:
-#ifdef USE_LIVEBUFFER
-               if (!Interact) {
-                  if (cControl::Control(true))
-                     cControl::Control(true)->Hide();
-#else
                if (!Interact)
-#endif /* LIVEBUFFER */
                   Menu = new cDisplayChannel(NORMALKEY(key));
-#ifdef USE_LIVEBUFFER
-                  }
-#endif /* LIVEBUFFER */
                else if (cDisplayChannel::IsOpen() || cControl::Control()) {
                   Interact->ProcessKey(key);
                   continue;
@@ -1350,13 +1076,8 @@ int main(int argc, char *argv[])
                break;
           // Audio track control:
           case kAudio:
-#ifdef USE_LIVEBUFFER
-               if (cControl::Control(true))
-                  cControl::Control(true)->Hide();
-#else
                if (cControl::Control())
                   cControl::Control()->Hide();
-#endif /* LIVEBUFFER */
                if (!cDisplayTracks::IsOpen()) {
                   DELETE_MENU;
                   Menu = cDisplayTracks::Create();
@@ -1379,24 +1100,20 @@ int main(int argc, char *argv[])
                break;
           // Pausing live video:
           case kPause:
-#ifdef USE_LIVEBUFFER
-               if (!cControl::Control() && !cLiveBufferManager::GetLiveBufferControl()) {
-#else
                if (!cControl::Control()) {
-#endif /* LIVEBUFFER */
                   DELETE_MENU;
-                  if (!cRecordControls::PauseLiveVideo())
-                     Skins.Message(mtError, tr("No free DVB device to record!"));
+                  if (Setup.PauseKeyHandling) {
+                     if (Setup.PauseKeyHandling > 1 || Interface->Confirm(tr("Pause live video?"))) {
+                        if (!cRecordControls::PauseLiveVideo())
+                           Skins.Message(mtError, tr("No free DVB device to record!"));
+                        }
+                     }
                   key = kNone; // nobody else needs to see this key
                   }
                break;
           // Instant recording:
           case kRecord:
-#ifdef USE_LIVEBUFFER
-               if (!cControl::Control() && !cLiveBufferManager::GetLiveBufferControl()) {
-#else
                if (!cControl::Control()) {
-#endif /* LIVEBUFFER */
                   if (cRecordControls::Start())
                      Skins.Message(mtInfo, tr("Recording started"));
                   key = kNone; // nobody else needs to see this key
@@ -1448,33 +1165,12 @@ int main(int argc, char *argv[])
                  state = osEnd;
               }
            switch (state) {
-#ifdef USE_LIVEBUFFER
-             case osPause:  if (cLiveBufferManager::GetLiveBufferControl()) {
-                               DELETE_MENU;
-                               if (cControl::Control(true)) {
-                                  cControl::Control(true)->ProcessKey(kPlay);
-                                  cControl::Control(true)->ProcessKey(kPause);
-                                  }
-                               break;
-                               }
-                            DELETE_MENU;
-#else
              case osPause:  DELETE_MENU;
-#endif /* LIVEBUFFER */
                             cControl::Shutdown(); // just in case
                             if (!cRecordControls::PauseLiveVideo())
                                Skins.Message(mtError, tr("No free DVB device to record!"));
                             break;
-#ifdef USE_LIVEBUFFER
-             case osRecord: if (cLiveBufferManager::GetLiveBufferControl()) {
-                               if (cControl::Control(true))
-                                  cControl::Control(true)->ProcessKey(kRecord);
-                               break;
-                               }
-                            DELETE_MENU;
-#else
              case osRecord: DELETE_MENU;
-#endif /* LIVEBUFFER */
                             if (cRecordControls::Start())
                                Skins.Message(mtInfo, tr("Recording started"));
                             break;
@@ -1513,12 +1209,6 @@ int main(int argc, char *argv[])
              }
            }
         else {
-#ifdef USE_LIVEBUFFER
-           eOSState state = osUnknown;
-           if (cLiveBufferManager::GetLiveBufferControl())
-              state = cLiveBufferManager::GetLiveBufferControl()->ProcessKey(key);
-           if (state == osUnknown) {
-#endif /* LIVEBUFFER */
            // Key functions in "normal" viewing mode:
            if (key != kNone && KeyMacros.Get(key)) {
               cRemote::PutMacro(key);
@@ -1563,10 +1253,6 @@ int main(int argc, char *argv[])
              case kUp:
              case kDown|k_Repeat:
              case kDown:
-#ifdef USE_LIVEBUFFER
-                  if (cControl::Control(true))
-                     cControl::Control(true)->Hide();
-#endif /* LIVEBUFFER */
                   Menu = new cDisplayChannel(NORMALKEY(key));
                   break;
              // Viewing Control:
@@ -1587,9 +1273,6 @@ int main(int argc, char *argv[])
              default:    break;
              }
            }
-#ifdef USE_LIVEBUFFER
-           }
-#endif /* LIVEBUFFER */
         if (!Menu) {
            if (!InhibitEpgScan)
               EITScanner.Process();
@@ -1659,9 +1342,6 @@ Exit:
   signal(SIGPIPE, SIG_DFL);
   signal(SIGALRM, SIG_DFL);
 
-#ifdef USE_LIVEBUFFER
-  cLiveBufferManager::Shutdown();
-#endif /* LIVEBUFFER */
   PluginManager.StopPlugins();
   cRecordControls::Shutdown();
   cCutter::Stop();
@@ -1672,6 +1352,7 @@ Exit:
   Remotes.Clear();
   Audios.Clear();
   Skins.Clear();
+  SourceParams.Clear();
   if (ShutdownHandler.GetExitCode() != 2) {
      Setup.CurrentChannel = cDevice::CurrentChannel();
      Setup.CurrentVolume  = cDevice::CurrentVolume();
