@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: videodir.c 1.15 2008/02/16 13:00:03 kls Exp $
+ * $Id: videodir.c 2.0 2008/02/16 13:00:03 kls Exp $
  */
 
 #include "videodir.h"
@@ -19,14 +19,7 @@
 #include "recording.h"
 #include "tools.h"
 
-#ifdef USE_HARDLINKCUTTER
-//#define HARDLINK_TEST_ONLY
-#endif /* HARDLINKCUTTER */
-
 const char *VideoDirectory = VIDEODIR;
-#ifdef USE_LIVEBUFFER
-const char *BufferDirectory = NULL;
-#endif /* LIVEBUFFER */
 
 class cVideoDirectory {
 private:
@@ -198,13 +191,13 @@ static bool StatNearestDir(const char *FileName, struct stat *Stat)
 {
   cString Name(FileName);
   char *p;
-  while ((p = strrchr((const char*)Name + 1, '/')) != NULL) {
-        *p = 0; // truncate at last '/'
-        if (stat(Name, Stat) == 0) {
-           isyslog("StatNearestDir: Stating %s", (const char*)Name);
-           return true;
-           }
+  while ((p = strrchr((char*)(const char*)Name + 1, '/')) != NULL) {
+     *p = 0; // truncate at last '/'
+     if (stat(Name, Stat) == 0) {
+        isyslog("StatNearestDir: Stating %s", (const char*)Name);
+        return true;
         }
+     }
   return false;
 }
 
@@ -258,16 +251,16 @@ bool HardLinkVideoFile(const char *OldName, const char *NewName)
      // Search in video01 and upwards
      bool found = false;
      while (Dir.Next()) {
-           Dir.Store();
-           const char *TmpNewName = Dir.Adjust(NewName);
-           if (StatNearestDir(TmpNewName, &StatDir) && StatDir.st_dev == StatOldName.st_dev) {
-              isyslog("HardLinkVideoFile: %s is on %i (match)", TmpNewName, (int)StatDir.st_dev);
-              ActualNewName = TmpNewName;
-              found = true;
-              break;
-              }
-           isyslog("HardLinkVideoFile: %s is on %i", TmpNewName, (int)StatDir.st_dev);
+        Dir.Store();
+        const char *TmpNewName = Dir.Adjust(NewName);
+        if (StatNearestDir(TmpNewName, &StatDir) && StatDir.st_dev == StatOldName.st_dev) {
+           isyslog("HardLinkVideoFile: %s is on %i (match)", TmpNewName, (int)StatDir.st_dev);
+           ActualNewName = TmpNewName;
+           found = true;
+           break;
            }
+        isyslog("HardLinkVideoFile: %s is on %i", TmpNewName, (int)StatDir.st_dev);
+        }
      if (ActualNewName == NewName) {
         esyslog("HardLinkVideoFile: No matching video folder to hard link %s", (const char*)ActualOldName);
         return false;
@@ -276,7 +269,7 @@ bool HardLinkVideoFile(const char *OldName, const char *NewName)
      // Looking good, we have a match. Create necessary folders.
      if (!MakeDirs(ActualNewName, false))
         return false;
-     // There's no guarantee that the directory of ActualNewName 
+     // There's no guarantee that the directory of ActualNewName
      // is on the same device as the dir that StatNearestDir found.
      // But worst case is that the link fails.
      }
@@ -284,7 +277,7 @@ bool HardLinkVideoFile(const char *OldName, const char *NewName)
 #ifdef HARDLINK_TEST_ONLY
   // Do the hard link to *.vdr_ for testing only
   char *name = NULL;
-  asprintf(&name, "%s_", ActualNewName);
+  asprintf(&name, "%s_",ActualNewName);
   link(ActualOldName, name);
   free(name);
   return false;
@@ -307,6 +300,7 @@ bool HardLinkVideoFile(const char *OldName, const char *NewName)
      }
   return true;
 }
+
 #endif /* HARDLINKCUTTER */
 
 bool VideoFileSpaceAvailable(int SizeMB)
