@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: videodir.h 1.7 2008/02/16 12:53:11 kls Exp $
+ * $Id: videodir.h 2.0 2008/02/16 12:53:11 kls Exp $
  */
 
 #ifndef __VIDEODIR_H
@@ -14,9 +14,6 @@
 #include "tools.h"
 
 extern const char *VideoDirectory;
-#ifdef USE_LIVEBUFFER
-extern const char *BufferDirectory;
-#endif /* LIVEBUFFER */
 
 cUnbufferedFile *OpenVideoFile(const char *FileName, int Flags);
 int CloseVideoFile(cUnbufferedFile *File);
